/*
    This file is part of Repetier-Firmware.

    Repetier-Firmware is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Repetier-Firmware is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Repetier-Firmware.  If not, see <http://www.gnu.org/licenses/>.

*/

#if !defined(UI_DISPLAY_CHARSET) || UI_DISPLAY_CHARSET > 3
#define UI_DISPLAY_CHARSET 1
#endif

//Symbolic character values for specific symbols.
//May be overridden for different controllers, charactersets, etc.
#define cUP "\001"
#define cDEG "\002"
#define cSEL "\003"
#define cUNSEL "\004"
#define cTEMP "\005"
#define cFOLD "\006"
#define bFOLD 6
#define cARROW "\176"

#if UI_DISPLAY_CHARSET == 0 // ASCII fallback
#define CHAR_RIGHT '-'
#define CHAR_SELECTOR '>'
#define CHAR_SELECTED '*'
#define STR_auml "ae"
#define STR_Auml "Ae"
#define STR_uuml "ue"
#define STR_Uuml "Ue"
#define STR_ouml "oe"
#define STR_Ouml "Oe"
#define STR_szlig "ss"
#endif

#if UI_DISPLAY_CHARSET == 1 // HD44870 charset with knji chars
#define CHAR_RIGHT 0x7e
#define CHAR_SELECTOR '>'
#define CHAR_SELECTED '*'
#define STR_auml "\xe1"
#define STR_Auml "Ae"
#define STR_uuml "\365"
#define STR_Uuml "Ue"
#define STR_ouml "\357"
#define STR_Ouml "Oe"
#define STR_szlig "\342"
#endif

#if UI_DISPLAY_CHARSET == 2 // Western charset
#define CHAR_RIGHT 0xbc
#define CHAR_SELECTOR 0xf6
#define CHAR_SELECTED '*'
#define STR_auml "\204"
#define STR_Auml "\216"
#define STR_uuml "\201"
#define STR_Uuml "\212"
#define STR_ouml "\204"
#define STR_Ouml "\211"
#define STR_szlig "160"
#endif


#if UI_DISPLAY_CHARSET==3 // U8glib
#define CHAR_RIGHT 187 //>>
#define CHAR_SELECTOR 255 //'>'
#define CHAR_SELECTED 254 //'*'
#define STR_auml "\344"
#define STR_Auml "\304"
#define STR_uuml "\374"
#define STR_Uuml "\334"
#define STR_ouml "\366"
#define STR_Ouml "\326"
#define STR_szlig "\337"
#endif

#define LANGUAGE_EN_ID 0
#define LANGUAGE_DE_ID 1
#define LANGUAGE_NL_ID 2
#define LANGUAGE_PT_ID 3
#define LANGAUGE_IT_ID 4
#define LANGUAGE_ES_ID 5
#define LANGUAGE_SE_ID 6
#define LANGUAGE_FR_ID 7
#define LANGUAGE_CZ_ID 8
#define LANGUAGE_PL_ID 9
#define LANGUAGE_TR_ID 10
#define LANGUAGE_FI_ID 11

#define NUM_LANGUAGES_KNOWN 12
#define NUM_TRANSLATED_WORDS 418

// For selectable translations we refer to each text by a id which gets
// defined here. The list starts at 0 and defines the position in the
// translation index.

#define UI_TEXT_ON_ID 0
#define UI_TEXT_OFF_ID 1
#define UI_TEXT_NA_ID 2
#define UI_TEXT_YES_ID 3
#define UI_TEXT_NO_ID 4
#define UI_TEXT_PRINT_POS_ID 5
#define UI_TEXT_PRINTING_ID 6
#define UI_TEXT_IDLE_ID 7
#define UI_TEXT_NOSDCARD_ID 8
#define UI_TEXT_ERROR_ID 9
#define UI_TEXT_BACK_ID 10
#define UI_TEXT_QUICK_SETTINGS_ID 11
#define UI_TEXT_ERRORMSG_ID 12
#define UI_TEXT_CONFIGURATION_ID 13
#define UI_TEXT_POSITION_ID 14
#define UI_TEXT_EXTRUDER_ID 15
#define UI_TEXT_SD_CARD_ID 16
#define UI_TEXT_DEBUGGING_ID 17
#define UI_TEXT_HOME_DELTA_ID 18
#define UI_TEXT_HOME_ALL_ID 19
#define UI_TEXT_HOME_X_ID 20
#define UI_TEXT_HOME_Y_ID 21
#define UI_TEXT_HOME_Z_ID 22
#define UI_TEXT_PREHEAT_PLA_ID 23
#define UI_TEXT_PREHEAT_ABS_ID 24
#define UI_TEXT_LIGHTS_ONOFF_ID 25
#define UI_TEXT_COOLDOWN_ID 26
#define UI_TEXT_SET_TO_ORIGIN_ID 27
#define UI_TEXT_DISABLE_STEPPER_ID 28
#define UI_TEXT_X_POSITION_ID 29
#define UI_TEXT_X_POS_FAST_ID 30
#define UI_TEXT_Y_POSITION_ID 31
#define UI_TEXT_Y_POS_FAST_ID 32
#define UI_TEXT_Z_POSITION_ID 33
#define UI_TEXT_Z_POS_FAST_ID 34
#define UI_TEXT_E_POSITION_ID 35
#define UI_TEXT_BED_TEMP_ID 36
#define UI_TEXT_EXTR0_TEMP_ID 37
#define UI_TEXT_EXTR1_TEMP_ID 38
#define UI_TEXT_EXTR2_TEMP_ID 39
#define UI_TEXT_EXTR0_OFF_ID 40
#define UI_TEXT_EXTR1_OFF_ID 41
#define UI_TEXT_EXTR2_OFF_ID 42
#define UI_TEXT_EXTR0_SELECT_ID 43
#define UI_TEXT_EXTR1_SELECT_ID 44
#define UI_TEXT_EXTR2_SELECT_ID 45
#define UI_TEXT_EXTR_ORIGIN_ID 46
#define UI_TEXT_PRINT_X_ID 47
#define UI_TEXT_PRINT_Y_ID 48
#define UI_TEXT_PRINT_Z_ID 49
#define UI_TEXT_PRINT_Z_DELTA_ID 50
#define UI_TEXT_MOVE_X_ID 51
#define UI_TEXT_MOVE_Y_ID 52
#define UI_TEXT_MOVE_Z_ID 53
#define UI_TEXT_MOVE_Z_DELTA_ID 54
#define UI_TEXT_JERK_ID 55
#define UI_TEXT_ZJERK_ID 56
#define UI_TEXT_ACCELERATION_ID 57
#define UI_TEXT_STORE_TO_EEPROM_ID 58
#define UI_TEXT_LOAD_EEPROM_ID 59
#define UI_TEXT_DBG_ECHO_ID 60
#define UI_TEXT_DBG_INFO_ID 61
#define UI_TEXT_DBG_ERROR_ID 62
#define UI_TEXT_DBG_DRYRUN_ID 63
#define UI_TEXT_OPS_OFF_ID 64
#define UI_TEXT_OPS_CLASSIC_ID 65
#define UI_TEXT_OPS_FAST_ID 66
#define UI_TEXT_OPS_RETRACT_ID 67
#define UI_TEXT_OPS_BACKSLASH_ID 68
#define UI_TEXT_OPS_MINDIST_ID 69
#define UI_TEXT_OPS_MOVE_AFTER_ID 70
#define UI_TEXT_ANTI_OOZE_ID 71
#define UI_TEXT_PRINT_FILE_ID 72
#define UI_TEXT_PAUSE_PRINT_ID 73
#define UI_TEXT_CONTINUE_PRINT_ID 74
#define UI_TEXT_UNMOUNT_CARD_ID 75
#define UI_TEXT_MOUNT_CARD_ID 76
#define UI_TEXT_DELETE_FILE_ID 77
#define UI_TEXT_FEEDRATE_ID 78
#define UI_TEXT_FEED_MAX_X_ID 79
#define UI_TEXT_FEED_MAX_Y_ID 80
#define UI_TEXT_FEED_MAX_Z_ID 81
#define UI_TEXT_FEED_MAX_Z_DELTA_ID 82
#define UI_TEXT_FEED_HOME_X_ID 83
#define UI_TEXT_FEED_HOME_Y_ID 84
#define UI_TEXT_FEED_HOME_Z_ID 85
#define UI_TEXT_FEED_HOME_Z_DELTA_ID 86
#define UI_TEXT_ACTION_XPOSITION4A_ID 87
#define UI_TEXT_ACTION_XPOSITION4B_ID 88
#define UI_TEXT_ACTION_XPOSITION4C_ID 89
#define UI_TEXT_ACTION_XPOSITION4D_ID 90
#define UI_TEXT_ACTION_YPOSITION4A_ID 91
#define UI_TEXT_ACTION_YPOSITION4B_ID 92
#define UI_TEXT_ACTION_YPOSITION4C_ID 93
#define UI_TEXT_ACTION_YPOSITION4D_ID 94
#define UI_TEXT_ACTION_ZPOSITION4A_ID 95
#define UI_TEXT_ACTION_ZPOSITION4B_ID 96
#define UI_TEXT_ACTION_ZPOSITION4C_ID 97
#define UI_TEXT_ACTION_ZPOSITION4D_ID 98
#define UI_TEXT_ACTION_XPOSITION_FAST4A_ID 99
#define UI_TEXT_ACTION_XPOSITION_FAST4B_ID 100
#define UI_TEXT_ACTION_XPOSITION_FAST4C_ID 101
#define UI_TEXT_ACTION_XPOSITION_FAST4D_ID 102
#define UI_TEXT_ACTION_YPOSITION_FAST4A_ID 103
#define UI_TEXT_ACTION_YPOSITION_FAST4B_ID 104
#define UI_TEXT_ACTION_YPOSITION_FAST4C_ID 105
#define UI_TEXT_ACTION_YPOSITION_FAST4D_ID 106
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_ID 107
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_ID 108
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_ID 109
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_ID 110
#define UI_TEXT_ACTION_EPOSITION_FAST2A_ID 111
#define UI_TEXT_ACTION_EPOSITION_FAST2B_ID 112
#define UI_TEXT_ACTION_XPOSITION2A_ID 113
#define UI_TEXT_ACTION_XPOSITION2B_ID 114
#define UI_TEXT_ACTION_YPOSITION2A_ID 115
#define UI_TEXT_ACTION_YPOSITION2B_ID 116
#define UI_TEXT_ACTION_ZPOSITION2A_ID 117
#define UI_TEXT_ACTION_ZPOSITION2B_ID 118
#define UI_TEXT_ACTION_XPOSITION_FAST2A_ID 119
#define UI_TEXT_ACTION_XPOSITION_FAST2B_ID 120
#define UI_TEXT_ACTION_YPOSITION_FAST2A_ID 121
#define UI_TEXT_ACTION_YPOSITION_FAST2B_ID 122
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_ID 123
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_ID 124
#define UI_TEXT_FANSPEED_ID 125
#define UI_TEXT_ACTION_FANSPEED_ID 126
#define UI_TEXT_FAN_OFF_ID 127
#define UI_TEXT_FAN_25_ID 128
#define UI_TEXT_FAN_50_ID 129
#define UI_TEXT_FAN_75_ID 130
#define UI_TEXT_FAN_FULL_ID 131
#define UI_TEXT_STEPPER_INACTIVE_ID 132
#define UI_TEXT_STEPPER_INACTIVE2A_ID 133
#define UI_TEXT_STEPPER_INACTIVE2B_ID 134
#define UI_TEXT_POWER_INACTIVE_ID 135
#define UI_TEXT_POWER_INACTIVE2A_ID 136
#define UI_TEXT_POWER_INACTIVE2B_ID 137
#define UI_TEXT_GENERAL_ID 138
#define UI_TEXT_BAUDRATE_ID 139
#define UI_TEXT_EXTR_STEPS_ID 140
#define UI_TEXT_EXTR_START_FEED_ID 141
#define UI_TEXT_EXTR_MAX_FEED_ID 142
#define UI_TEXT_EXTR_ACCEL_ID 143
#define UI_TEXT_EXTR_WATCH_ID 144
#define UI_TEXT_EXTR_ADVANCE_L_ID 145
#define UI_TEXT_EXTR_ADVANCE_K_ID 146
#define UI_TEXT_EXTR_MANAGER_ID 147
#define UI_TEXT_EXTR_PGAIN_ID 148
#define UI_TEXT_EXTR_DEADTIME_ID 149
#define UI_TEXT_EXTR_DMAX_DT_ID 150
#define UI_TEXT_EXTR_IGAIN_ID 151
#define UI_TEXT_EXTR_DGAIN_ID 152
#define UI_TEXT_EXTR_DMIN_ID 153
#define UI_TEXT_EXTR_DMAX_ID 154
#define UI_TEXT_EXTR_PMAX_ID 155
#define UI_TEXT_STRING_HM_BANGBANG_ID 156
#define UI_TEXT_STRING_HM_PID_ID 157
#define UI_TEXT_STRING_ACTION_ID 158
#define UI_TEXT_HEATING_EXTRUDER_ID 159
#define UI_TEXT_HEATING_BED_ID 160
#define UI_TEXT_KILLED_ID 161
#define UI_TEXT_STEPPER_DISABLED_ID 162
#define UI_TEXT_EEPROM_STOREDA_ID 163
#define UI_TEXT_EEPROM_STOREDB_ID 164
#define UI_TEXT_EEPROM_LOADEDA_ID 165
#define UI_TEXT_EEPROM_LOADEDB_ID 166
#define UI_TEXT_UPLOADING_ID 167
#define UI_TEXT_PAGE_BUFFER_ID 168
#define UI_TEXT_PAGE_EXTRUDER_ID 169
#define UI_TEXT_PAGE_EXTRUDER1_ID 170
#define UI_TEXT_PAGE_EXTRUDER2_ID 171
#define UI_TEXT_PAGE_EXTRUDER3_ID 172
#define UI_TEXT_PAGE_BED_ID 173
#define UI_TEXT_SPEED_MULTIPLY_ID 174
#define UI_TEXT_FLOW_MULTIPLY_ID 175
#define UI_TEXT_SHOW_MEASUREMENT_ID 176
#define UI_TEXT_RESET_MEASUREMENT_ID 177
#define UI_TEXT_SET_MEASURED_ORIGIN_ID 178
#define UI_TEXT_ZCALIB_ID 179
#define UI_TEXT_SET_P1_ID 180
#define UI_TEXT_SET_P2_ID 181
#define UI_TEXT_SET_P3_ID 182
#define UI_TEXT_CALCULATE_LEVELING_ID 183
#define UI_TEXT_LEVEL_ID 184
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_ID 185
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_ID 186
#define UI_TEXT_SD_REMOVED_ID 187
#define UI_TEXT_SD_INSERTED_ID 188
#define UI_TEXT_PRINTER_READY_ID 189
#define UI_TEXT_PRINTTIME_DAYS_ID 190
#define UI_TEXT_PRINTTIME_HOURS_ID 191
#define UI_TEXT_PRINTTIME_MINUTES_ID 192
#define UI_TEXT_PRINT_TIME_ID 193
#define UI_TEXT_PRINT_FILAMENT_ID 194
#define UI_TEXT_PRINTED_ID 195
#define UI_TEXT_POWER_ID 196
#define UI_TEXT_STRING_HM_DEADTIME_ID 197
#define UI_TEXT_STRING_HM_SLOWBANG_ID 198
#define UI_TEXT_STOP_PRINT_ID 199
#define UI_TEXT_Z_BABYSTEPPING_ID 200
#define UI_TEXT_CHANGE_FILAMENT_ID 201
#define UI_TEXT_WIZ_CH_FILAMENT1_ID 202
#define UI_TEXT_WIZ_CH_FILAMENT2_ID 203
#define UI_TEXT_WIZ_CH_FILAMENT3_ID 204
#define UI_TEXT_CLICK_DONE_ID 205
#define UI_TEXT_AUTOLEVEL_ONOFF_ID 206
#define UI_TEXT_SERVOPOS_ID 207
#define UI_TEXT_IGNORE_M106_ID 208
#define UI_TEXT_WIZ_REHEAT1_ID 209
#define UI_TEXT_WIZ_REHEAT2_ID 210
#define UI_TEXT_WIZ_WAITTEMP1_ID 211
#define UI_TEXT_WIZ_WAITTEMP2_ID 212
#define UI_TEXT_EXTRUDER_JAM_ID 213
#define UI_TEXT_STANDBY_ID 214
#define UI_TEXT_BED_COATING_ID 215
#define UI_TEXT_BED_COATING_SET1_ID 216
#define UI_TEXT_BED_COATING_SET2_ID 217
#define UI_TEXT_NOCOATING_ID 218
#define UI_TEXT_BUILDTAK_ID 219
#define UI_TEXT_KAPTON_ID 220
#define UI_TEXT_BLUETAPE_ID 221
#define UI_TEXT_PETTAPE_ID 222
#define UI_TEXT_GLUESTICK_ID 223
#define UI_TEXT_CUSTOM_ID 224
#define UI_TEXT_COATING_CUSTOM_ID 225
#define UI_TEXT_LANGUAGE_ID 226
#define UI_TEXT_MAINPAGE6_1_ID 227
#define UI_TEXT_MAINPAGE6_2_ID 228
#define UI_TEXT_MAINPAGE6_3_ID 229
#define UI_TEXT_MAINPAGE6_4_ID 230
#define UI_TEXT_MAINPAGE6_5_ID 231
#define UI_TEXT_MAINPAGE6_6_ID 232
#define UI_TEXT_MAINPAGE_TEMP_BED_ID 233
#define UI_TEXT_MAINPAGE_BED_ID 234
#define UI_TEXT_MAINPAGE_Z_BUF_ID 235
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_ID 236
#define UI_TEXT_MAINPAGE_XY_ID 237
#define UI_TEXT_PRINT_TIME_VALUE_ID 238
#define UI_TEXT_PRINT_FILAMENT_VALUE_ID 239
#define UI_TEXT_METER_PRINTED_ID 240
#define UI_TEXT_STATUS_ID 241
#define UI_TEXT_EMPTY_ID 242
#define UI_TEXT_TEMP_SET_ID 243
#define UI_TEXT_CURRENT_TEMP_ID 244
#define UI_TEXT_COATING_THICKNESS_ID 245
#define UI_TEXT_EXTR3_TEMP_ID 246
#define UI_TEXT_EXTR4_TEMP_ID 247
#define UI_TEXT_EXTR5_TEMP_ID 248
#define UI_TEXT_EXTR3_OFF_ID 249
#define UI_TEXT_EXTR4_OFF_ID 250
#define UI_TEXT_EXTR5_OFF_ID 251
#define UI_TEXT_EXTR3_SELECT_ID 252
#define UI_TEXT_EXTR4_SELECT_ID 253
#define UI_TEXT_EXTR5_SELECT_ID 254
#define UI_TEXT_DITTO_0_ID 255
#define UI_TEXT_DITTO_1_ID 256
#define UI_TEXT_DITTO_2_ID 257
#define UI_TEXT_DITTO_3_ID 258
#define UI_TEXT_ZPROBE_HEIGHT_ID 259
#define UI_TEXT_OFFSETS_ID 260
#define UI_TEXT_X_OFFSET_ID 261
#define UI_TEXT_Y_OFFSET_ID 262
#define UI_TEXT_Z_OFFSET_ID 263
#define UI_TEXT_LOAD_FILAMENT_ID 264
#define UI_TEXT_UNLOAD_FILAMENT_ID 265
#define UI_TEXT_FIL_SENSOR_ONOFF_ID 266
#define UI_TEXT_CLEANING_DRIPBOX_ID 267
#define UI_TEXT_CLEAN_DRIPBOX_ID 268
#define UI_TEXT_CLEAN_DRIPBOX_3_ID 269
#define UI_TEXT_EXTR_XOFF_ID 270
#define UI_TEXT_EXTR_YOFF_ID 271
#define UI_TEXT_LOAD_RIGHT_FILAMENT_ID 272
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_ID 273
#define UI_TEXT_LOAD_LEFT_FILAMENT_ID 274
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_ID 275
#define UI_TEXT_FEED_LOADING_FILAMENT_ID 276
#define UI_TEXT_FEED_UNLOADING_FILAMENT_ID 277
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_ID 278
#define UI_TEXT_BED_DOWN_ID 279
#define UI_TEXT_WIFI_ONOFF_ID 280
#define UI_TEXT_REDO_ACTION_ID 281
#define UI_TEXT_X_LENGTH_ID 282
#define UI_TEXT_Y_LENGTH_ID 283
#define UI_TEXT_Z_LENGTH_ID 284
#define UI_TEXT_EXT_TEMP_ABS_ID 285
#define UI_TEXT_EXT_TEMP_PLA_ID 286
#define UI_TEXT_BED_TEMP_ABS_ID 287
#define UI_TEXT_BED_TEMP_PLA_ID 288
#define UI_TEXT_ALERT_NO_FILAMENTA_ID 289
#define UI_TEXT_ALERT_NO_FILAMENTB_ID 290
#define UI_TEXT_ALERT_NO_FILAMENTC_ID 291
#define UI_TEXT_ALERT_NO_FILAMENTD_ID 292
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_ID 293
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_ID 294
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_ID 295
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_ID 296
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_ID 297
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_ID 298
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_ID 299
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_ID 300
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_ID 301
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_ID 302
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_ID 303
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_ID 304
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_ID 305
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_ID 306
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_ID 307
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_ID 308
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_ID 309
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_ID 310
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_ID 311
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_ID 312
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_ID 313
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_ID 314
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_ID 315
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_ID 316
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_ID 317
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_ID 318
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_ID 319
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_ID 320
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_ID 321
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_ID 322
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_ID 323
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_ID 324
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_ID 325
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_ID 326
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_ID 327
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_ID 328
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_ID 329
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_ID 330
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_ID 331
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_ID 332
#define UI_TEXT_X_MIN_SETTING_ID 333
#define UI_TEXT_Y_MIN_SETTING_ID 334
#define UI_TEXT_Z_MIN_SETTING_ID 335
#define UI_TEXT_TOP_COVER_OPEN_ID 336
#define UI_TEXT_TOP_SENSOR_ONOFF_ID 337
#define UI_TEXT_VERSION_MENU_ID 338
#define UI_TEXT_COOLDOWN_MENU_ID 339
#define UI_TEXT_WAIT_OK_ID 340
#define UI_TEXT_FAILED_ID 341
#define UI_TEXT_PLEASE_WAIT_ID 342
#define UI_TEXT_MANUAL_LEVEL_ID 343
#define UI_TEXT_WARNING_ID 344
#define UI_TEXT_EXTRUDER_COLD_ID 345
#define UI_TEXT_HEAT_EXTRUDER_ID 346
#define UI_TEXT_WRITE_DEBUG_ID 347
#define UI_TEXT_POWER_SAVE_MENU_ID 348
#define UI_TEXT_MAINTENANCE_ID 349
#define UI_TEXT_SETTINGS_ID 350
#define UI_TEXT_LOAD_UNLOAD_ID 351
#define UI_TEXT_BED_ID 352
#define UI_TEXT_BED_OFF_ID 353
#define UI_TEXT_LIGHT_OFF_AFTER_ID 354
#define UI_TEXT_LIGHT_OFF_AFTER2A_ID 355
#define UI_TEXT_LIGHT_OFF_AFTER2B_ID 356
#define UI_TEXT_WARNING_POS_X_UNKNOWN_ID 357
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_ID 358
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_ID 359
#define UI_TEXT_MOVING_ID 360
#define UI_TEXT_PREPARING_ID 361
#define UI_TEXT_KEEP_LIGHT_ON_ID 362
#define UI_TEXT_LOADING_FILAMENT_ID 363
#define UI_TEXT_UNLOADING_FILAMENT_ID 364
#define UI_TEXT_LOADUNLOAD_FILAMENT_ID 365
#define UI_TEXT_AUTOLEVEL_ID 366
#define UI_TEXT_Z_PROBE_FAILED_ID 367
#define UI_TEXT_SAVE_ID 368
#define UI_TEXT_ZMIN_ID 369
#define UI_TEXT_ZPROBING_ID 370
#define UI_TEXT_ZPOSITION_ID 371
#define UI_TEXT_DO_YOU_ID 372
#define UI_TEXT_CLEAN1_ID 373
#define UI_TEXT_CLEAN2_ID 374
#define UI_TEXT_DISPLAY_MODE_ID 375
#define UI_TEXT_EASY_MODE_ID 376
#define UI_TEXT_ADVANCED_MODE_ID 377
#define UI_TEXT_POWER_SAVE_ID 378
#define UI_TEXT_BADGE_LIGHT_ONOFF_ID 379
#define UI_TEXT_SOUND_ONOF_ID 380
#define UI_TEXT_YES_SELECTED_ID 381
#define UI_TEXT_NO_SELECTED_ID 382
#define UI_TEXT_PLEASE_CONFIRM_ID 383
#define UI_TEXT_CANCEL_ACTION_ID 384
#define UI_TEXT_CONTINUE_ACTION_ID 385
#define UI_TEXT_CANCELED_ID 386
#define UI_TEXT_CONFIRMED_ID 387
#define UI_TEXT_CLEAN_NOZZLE_ID 388
#define UI_TEXT_CLEANING_NOZZLE_ID 389
#define UI_TEXT_CLEAN_DRIPBOX_1_ID 390
#define UI_TEXT_CLEAN_DRIPBOX_2_ID 391
#define UI_TEXT_PUSH_FILAMENT_ID 392
#define UI_TEXT_WAIT_FILAMENT_ID 393
#define UI_TEXT_LOAD_FAILSAFE2_ID 394
#define UI_TEXT_LOAD_FAILSAFE_ID 395
#define UI_TEXT_HEATING_ID 396
#define UI_TEXT_WAIT_FOR_OK_ID 397
#define UI_TEXT_MAIN_LINE1_ID 398
#define UI_TEXT_MAIN_LINE2_ID 399
#define UI_TEXT_AUTOLEVEL_MATRIX_ID 400
#define UI_TEXT_P1_L1_ID 401
#define UI_TEXT_P1_L2_ID 402
#define UI_TEXT_P1_L3_ID 403
#define UI_TEXT_P1_L1_EASY_ID 404
#define UI_TEXT_P1_L2_EASY_ID 405
#define UI_TEXT_P1_L3_EASY_ID 406
#define UI_TEXT_DBG_ENDSTOP_ID 407 //     "EndStop:%dp"
#define UI_TEXT_MENU_SCANNER_ID 408
#define UI_TEXT_PREPARE_SCANNER_ID 409
#define UI_TEXT_LASER1_ONOFF_ID 410
#define UI_TEXT_LASER2_ONOFF_ID 411
#define UI_TEXT_LED1_ONOFF_ID 412
#define UI_TEXT_LED2_ONOFF_ID 413
#define UI_TEXT_CHANGE_TABLE_SPEED_ID 414
#define UI_TEXT_ROTATE_TABLE_ID 415
#define UI_TEXT_RELEASE_TABLE_ID 416
#define UI_TEXT_Z_MIN_CALCULATION_ID 417


// Universal definitions

#define UI_TEXT_SEL              cSEL
#define UI_TEXT_NOSEL            cUNSEL
#define UI_TEXT_CONFIRMATION     "%C1","%C2","%C3","%C4"

// At first all terms in english are defined. After that the selected language
// can overwrite the definition. That way new strings are at least in english
// available.

#define UI_TEXT_ON_EN               " On"
#define UI_TEXT_OFF_EN              "Off"
#define UI_TEXT_NA_EN               "N/A" // Output for not available
#define UI_TEXT_YES_EN              "Yes"
#define UI_TEXT_NO_EN               " No"
#define UI_TEXT_PRINT_POS_EN        "Printing.."
#define UI_TEXT_PRINTING_EN         "Printing"
#define UI_TEXT_IDLE_EN             "Idle"
#define UI_TEXT_NOSDCARD_EN         "No SD card"
#define UI_TEXT_ERROR_EN            "**** ERROR ****"
#define UI_TEXT_BACK_EN             "Back " cUP
#define UI_TEXT_QUICK_SETTINGS_EN   "Quick settings"
#define UI_TEXT_ERRORMSG_EN         "%oe"
#define UI_TEXT_CONFIGURATION_EN    "Configuration"
#define UI_TEXT_POSITION_EN         "Position"
#define UI_TEXT_EXTRUDER_EN         "Extruder"
#define UI_TEXT_SD_CARD_EN          "SD card"
#define UI_TEXT_DEBUGGING_EN        "Debugging"
#define UI_TEXT_HOME_DELTA_EN       "Home delta"
#define UI_TEXT_HOME_ALL_EN         "Home all"
#define UI_TEXT_HOME_X_EN           "Home X"
#define UI_TEXT_HOME_Y_EN           "Home Y"
#define UI_TEXT_HOME_Z_EN           "Home Z"
#define UI_TEXT_PREHEAT_PLA_EN      "Preheat PLA   %B1"
#define UI_TEXT_PREHEAT_ABS_EN      "Preheat ABS   %B2"
#define UI_TEXT_LIGHTS_ONOFF_EN     "Lights:     %lo"
#define UI_TEXT_COOLDOWN_EN         "Cooldown"
#define UI_TEXT_SET_TO_ORIGIN_EN    "Set to origin"
#define UI_TEXT_DISABLE_STEPPER_EN  "Disable stepper"
#define UI_TEXT_X_POSITION_EN       "X position"
#define UI_TEXT_X_POS_FAST_EN       "X pos. fast"
#define UI_TEXT_Y_POSITION_EN       "Y position"
#define UI_TEXT_Y_POS_FAST_EN       "Y pos. fast"
#define UI_TEXT_Z_POSITION_EN       "Z position"
#define UI_TEXT_Z_POS_FAST_EN       "Z pos. fast"
#define UI_TEXT_E_POSITION_EN       "Extr. position"
#define UI_TEXT_BED_TEMP_EN         "Bed :%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_EN       "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_EN       "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_EN       "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_EN       "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_EN        "SwitchOff Ext %B4"
#else
#define UI_TEXT_EXTR0_OFF_EN        "SwitchOff Ext1%B4"
#endif
#define UI_TEXT_EXTR1_OFF_EN        "SwitchOff Ext2%B5"
#define UI_TEXT_EXTR2_OFF_EN        "SwitchOff Ext3%B6"
#define UI_TEXT_EXTR0_SELECT_EN     "Select Extr.1 %X0"
#define UI_TEXT_EXTR1_SELECT_EN     "Select Extr.2 %X1"
#define UI_TEXT_EXTR2_SELECT_EN     "Select Extr.3 %X2"
#define UI_TEXT_EXTR_ORIGIN_EN      "Set Origin"
#define UI_TEXT_PRINT_X_EN "Print X:  %ax"
#define UI_TEXT_PRINT_Y_EN "Print Y:  %ay"
#define UI_TEXT_PRINT_Z_EN "Print Z:  %az"
#define UI_TEXT_PRINT_Z_DELTA_EN "Print:%az"
#define UI_TEXT_MOVE_X_EN "Move X:   %aX"
#define UI_TEXT_MOVE_Y_EN "Move Y:   %aY"
#define UI_TEXT_MOVE_Z_EN "Move Z:   %aZ"
#define UI_TEXT_MOVE_Z_DELTA_EN "Move:     %aZ"
#define UI_TEXT_JERK_EN "Jerk:     %aj"
#define UI_TEXT_ZJERK_EN "Z-Jerk:   %aJ"
#define UI_TEXT_ACCELERATION_EN "Acceleration"
#define UI_TEXT_STORE_TO_EEPROM_EN "Store \176 EEPROM"
#define UI_TEXT_LOAD_EEPROM_EN "Load \177 EEPROM"
#define UI_TEXT_DBG_ECHO_EN "Echo:       %do"
#define UI_TEXT_DBG_INFO_EN "Info:       %di"
#define UI_TEXT_DBG_ERROR_EN "Errors:     %de"
#define UI_TEXT_DBG_DRYRUN_EN "Dry run:    %dd"
#define UI_TEXT_DBG_ENDSTOP_EN      "EndStop:%dp"
#define UI_TEXT_OPS_OFF_EN "%O0 OPS Off"
#define UI_TEXT_OPS_CLASSIC_EN "%O1 OPS Classic"
#define UI_TEXT_OPS_FAST_EN "%O2 OPS Fast"
#define UI_TEXT_OPS_RETRACT_EN "Retract:    %Or"
#define UI_TEXT_OPS_BACKSLASH_EN "Backsl.:    %Ob"
#define UI_TEXT_OPS_MINDIST_EN "Min.dist:   %Od"
#define UI_TEXT_OPS_MOVE_AFTER_EN "Move after: %Oa"
#define UI_TEXT_ANTI_OOZE_EN "Anti Ooze"
#define UI_TEXT_PRINT_FILE_EN "Print file"
#define UI_TEXT_PAUSE_PRINT_EN "Pause Print"
#define UI_TEXT_CONTINUE_PRINT_EN "Continue Print"
#define UI_TEXT_UNMOUNT_CARD_EN "Unmount Card"
#define UI_TEXT_MOUNT_CARD_EN "Mount Card"
#define UI_TEXT_DELETE_FILE_EN "Delete file"
#define UI_TEXT_FEEDRATE_EN "Feedrate"
#define UI_TEXT_FEED_MAX_X_EN "Max X:    %fx"
#define UI_TEXT_FEED_MAX_Y_EN "Max Y:    %fy"
#define UI_TEXT_FEED_MAX_Z_EN "Max Z:    %fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_EN "Max:      %fz"
#define UI_TEXT_FEED_HOME_X_EN "Home X:   %fX"
#define UI_TEXT_FEED_HOME_Y_EN "Home Y:   %fY"
#define UI_TEXT_FEED_HOME_Z_EN "Home Z:   %fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_EN "Home:     %fZ"
#define UI_TEXT_ACTION_XPOSITION4A_EN "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_EN "Min endstop:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_EN "Max endstop:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_EN ""
#define UI_TEXT_ACTION_YPOSITION4A_EN "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_EN "Min endstop:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_EN "Max endstop:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_EN ""
#define UI_TEXT_ACTION_ZPOSITION4A_EN "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_EN "Min endstop:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_EN "Max endstop:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_EN ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_EN "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_EN "Min endstop:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_EN "Max endstop:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_EN ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_EN "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_EN "Min endstop:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_EN "Max endstop:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_EN ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_EN "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_EN "Min endstop:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_EN "Max endstop:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_EN ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_EN "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_EN "1 click = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_EN "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_EN "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_EN "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_EN "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_EN "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_EN "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_EN "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_EN "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_EN "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_EN "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_EN "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_EN "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_EN "Fan speed"
#define UI_TEXT_ACTION_FANSPEED_EN "Fan speed:%Fs%%%"
#define UI_TEXT_FAN_OFF_EN "Turn Fan Off"
#define UI_TEXT_FAN_25_EN "Set Fan 25%%%"
#define UI_TEXT_FAN_50_EN "Set Fan 50%%%"
#define UI_TEXT_FAN_75_EN "Set Fan 75%%%"
#define UI_TEXT_FAN_FULL_EN "Set Fan Full"
#define UI_TEXT_STEPPER_INACTIVE_EN "Steppers Inact"
#define UI_TEXT_STEPPER_INACTIVE2A_EN "Dis. After:%is"
#define UI_TEXT_STEPPER_INACTIVE2B_EN "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_EN "Max. Inact."
#define UI_TEXT_POWER_INACTIVE2A_EN "Dis. After:%ip"
#define UI_TEXT_POWER_INACTIVE2B_EN "[min] 0=Off"
#define UI_TEXT_GENERAL_EN "General"
#define UI_TEXT_BAUDRATE_EN "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_EN "Steps/MM: %Se"
#define UI_TEXT_EXTR_START_FEED_EN "Start FR: %Xf"
#define UI_TEXT_EXTR_MAX_FEED_EN "Max FR:   %XF"
#define UI_TEXT_EXTR_ACCEL_EN "Accel:    %XA"
#define UI_TEXT_EXTR_WATCH_EN "Stab.Time: %Xw"
#define UI_TEXT_EXTR_ADVANCE_L_EN "Advance L.: %Xl"
#define UI_TEXT_EXTR_ADVANCE_K_EN "Advance Q.: %Xa"
#define UI_TEXT_EXTR_MANAGER_EN "Ctrl:%Xh"
#define UI_TEXT_EXTR_PGAIN_EN "PID P:   %Xp"
#define UI_TEXT_EXTR_DEADTIME_EN "Deadtime:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_EN "Control PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_EN "PID I:  %Xi"
#define UI_TEXT_EXTR_DGAIN_EN "PID D:  %Xd"
#define UI_TEXT_EXTR_DMIN_EN "Drive Min:  %Xm"
#define UI_TEXT_EXTR_DMAX_EN "Drive Max:  %XM"
#define UI_TEXT_EXTR_PMAX_EN "PID Max:    %XD"
#define UI_TEXT_STRING_HM_BANGBANG_EN "  BangBang"
#define UI_TEXT_STRING_HM_PID_EN "       PID"
#define UI_TEXT_STRING_ACTION_EN "Action:%la"
#define UI_TEXT_HEATING_EXTRUDER_EN "Heating Extruder%Ne"
#define UI_TEXT_HEATING_BED_EN "Heating Bed"
#define UI_TEXT_KILLED_EN "Stopped"
#define UI_TEXT_STEPPER_DISABLED_EN "Stepper Disabled"
#define UI_TEXT_EEPROM_STOREDA_EN "Configuration"
#define UI_TEXT_EEPROM_STOREDB_EN "stored \176 EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_EN "Configuration"
#define UI_TEXT_EEPROM_LOADEDB_EN "loaded \177 EEPROM"
#define UI_TEXT_UPLOADING_EN "Uploading..."
#define UI_TEXT_PAGE_BUFFER_EN "Buffer:%oB"
#define UI_TEXT_PAGE_EXTRUDER_EN " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_EN "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_EN "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_EN "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_EN " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_EN "Speed Mul.:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_EN "Flow Mul.: %of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_EN "Show meas."
#define UI_TEXT_RESET_MEASUREMENT_EN "Reset meas."
#define UI_TEXT_SET_MEASURED_ORIGIN_EN "Set Z=0"
#define UI_TEXT_ZCALIB_EN "Z Calib."
#define UI_TEXT_SET_P1_EN "Set P1"
#define UI_TEXT_SET_P2_EN "Set P2"
#define UI_TEXT_SET_P3_EN "Set P3"
#define UI_TEXT_CALCULATE_LEVELING_EN "Calculate Leveling"
#define UI_TEXT_LEVEL_EN "Level delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_EN "Wait T.: %XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_EN "Wait U.:  %XU mm"
#define UI_TEXT_SD_REMOVED_EN "SD Card removed"
#define UI_TEXT_SD_INSERTED_EN "SD Card inserted"
#define UI_TEXT_PRINTER_READY_EN "Printer ready."
#define UI_TEXT_PRINTTIME_DAYS_EN " days  "
#define UI_TEXT_PRINTTIME_HOURS_EN ":"
#define UI_TEXT_PRINTTIME_MINUTES_EN ""
#define UI_TEXT_PRINT_TIME_EN "Printing time"
#define UI_TEXT_PRINT_FILAMENT_EN "Filament printed"
#define UI_TEXT_PRINTED_EN "printed"
#define UI_TEXT_POWER_EN "ATX power on/off"
#define UI_TEXT_STRING_HM_DEADTIME_EN " Dead Time"
#define UI_TEXT_STRING_HM_SLOWBANG_EN "  SlowBang"
#define UI_TEXT_STOP_PRINT_EN "Stop Print"
#define UI_TEXT_Z_BABYSTEPPING_EN "Z babyS:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_EN "Change filament"
#define UI_TEXT_WIZ_CH_FILAMENT1_EN "Change filament"
#define UI_TEXT_WIZ_CH_FILAMENT2_EN "Rotate to move"
#define UI_TEXT_WIZ_CH_FILAMENT3_EN "filament up/down"
#define UI_TEXT_CLICK_DONE_EN "Click when done"
#define UI_TEXT_AUTOLEVEL_ONOFF_EN "Autolevel:  %ll"
#define UI_TEXT_SERVOPOS_EN "Servo pos.: %oS"
#define UI_TEXT_IGNORE_M106_EN "Ignore M106 cmd %Fi"
#define UI_TEXT_WIZ_REHEAT1_EN "Click to reheat"
#define UI_TEXT_WIZ_REHEAT2_EN "extruders."
#define UI_TEXT_WIZ_WAITTEMP1_EN "Wait for target"
#define UI_TEXT_WIZ_WAITTEMP2_EN "temperatures ..."
#define UI_TEXT_EXTRUDER_JAM_EN "Extruder Jam"
#define UI_TEXT_STANDBY_EN "Standby"
#define UI_TEXT_BED_COATING_EN "Bed coating"
#define UI_TEXT_BED_COATING_SET1_EN "Bed coating set to"
#define UI_TEXT_BED_COATING_SET2_EN ""
#define UI_TEXT_NOCOATING_EN "No coating"
#define UI_TEXT_BUILDTAK_EN "BuildTak"
#define UI_TEXT_KAPTON_EN "Kapton"
#define UI_TEXT_BLUETAPE_EN "Blue paper tape"
#define UI_TEXT_PETTAPE_EN "Green PET tape"
#define UI_TEXT_GLUESTICK_EN "Glue stick"
#define UI_TEXT_CUSTOM_EN "Custom"
#define UI_TEXT_COATING_CUSTOM_EN "Custom: %BCmm"
#define UI_TEXT_LANGUAGE_EN "Language"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_EN "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_EN "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_EN "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_EN "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_EN "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_EN "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_EN "Copies: %ed    Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_EN "Flow:\xfd %of%%%   Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_EN "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_EN "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_EN "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_EN cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_EN "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_EN "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_EN "Mul: %om%%%  E:%x4"
#define UI_TEXT_MAINPAGE_XY_EN "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_EN "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_EN "%Uf m"
#define UI_TEXT_METER_PRINTED_EN "%Uf m " UI_TEXT_PRINTED_EN
#define UI_TEXT_STATUS_EN "%os"
#define UI_TEXT_EMPTY_EN ""
#define UI_TEXT_TEMP_SET_EN cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_EN cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_EN " %BCmm"
#define UI_TEXT_EXTR3_TEMP_EN "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_EN "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_EN "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_EN "Turn extr. 4 off"
#define UI_TEXT_EXTR4_OFF_EN "Turn extr. 5 off"
#define UI_TEXT_EXTR5_OFF_EN "Turn extr. 6 off"
#define UI_TEXT_EXTR3_SELECT_EN "%X3 Select extr. 4"
#define UI_TEXT_EXTR4_SELECT_EN "%X4 Select extr. 5"
#define UI_TEXT_EXTR5_SELECT_EN "%X5 Select extr. 6"
#define UI_TEXT_DITTO_0_EN "%D0 No copies"
#define UI_TEXT_DITTO_1_EN "%D1 1 copy"
#define UI_TEXT_DITTO_2_EN "%D2 2 copies"
#define UI_TEXT_DITTO_3_EN "%D3 3 copies"
#define UI_TEXT_ZPROBE_HEIGHT_EN "Z-probe height:%zh"
#define UI_TEXT_OFFSETS_EN "Set print offsets"
#define UI_TEXT_X_OFFSET_EN "X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_EN "Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_EN "Zoffset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_EN "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_EN "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_EN "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_EN "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_EN "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_EN "Dripbox"
#define UI_TEXT_EXTR_XOFF_EN "Xoffst:%Xxmm"
#define UI_TEXT_EXTR_YOFF_EN "Yoffst:%Xymm"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_EN "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_EN "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_EN "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_EN "Dripboxes"
#define UI_TEXT_EXTR_XOFF_EN "Xoffst:%Xxmm"
#define UI_TEXT_EXTR_YOFF_EN "Yoffst:%Xymm"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_EN "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_EN "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_EN "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_EN "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_EN "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_EN "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_EN "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_EN "Bed Down"
#define UI_TEXT_WIFI_ONOFF_EN "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_EN "Do it again ?"
#define UI_TEXT_X_LENGTH_EN "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_EN "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_EN "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_EN "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_EN "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_EN "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_EN "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_EN "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_EN "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_EN "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_EN "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_EN "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_EN "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_EN "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_EN "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_EN "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_EN "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_EN "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_EN "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_EN "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_EN "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_EN "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_EN "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_EN "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_EN "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_EN "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_EN "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_EN "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_EN "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_EN "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_EN "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_EN "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_EN "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_EN "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_EN "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_EN "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_EN "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_EN "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_EN "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_EN "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_EN "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_EN "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_EN "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_EN "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_EN "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_EN "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_EN "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_EN "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_EN "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_EN "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_EN "and plate."
#define UI_TEXT_X_MIN_SETTING_EN "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_EN "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_EN "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_EN "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_EN "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_EN "System Version"
#define UI_TEXT_COOLDOWN_MENU_EN "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_EN "Ok to continue"
#define UI_TEXT_FAILED_EN "Failed"
#define UI_TEXT_PLEASE_WAIT_EN "Please wait"
#define UI_TEXT_MANUAL_LEVEL_EN "Manual leveling"
#define UI_TEXT_WARNING_EN "Warning!"
#define UI_TEXT_EXTRUDER_COLD_EN "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_EN "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_EN "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_EN "Powersave"
#define UI_TEXT_MAINTENANCE_EN "Maintenance"
#define UI_TEXT_SETTINGS_EN "Settings"
#define UI_TEXT_LOAD_UNLOAD_EN "Load/Unload"
#define UI_TEXT_BED_EN "Bed"
#define UI_TEXT_BED_OFF_EN "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_EN "Light AutoOff"
#define UI_TEXT_LIGHT_OFF_AFTER2A_EN "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_EN "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_EN "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_EN "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_EN "Real Z unknown !"
#define UI_TEXT_MOVING_EN "Moving..."
#define UI_TEXT_PREPARING_EN "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_EN "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_EN "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_EN "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_EN "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_EN "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_EN "Z-probe failed"
#define UI_TEXT_SAVE_EN "Save to EEPROM ?"
#define UI_TEXT_ZMIN_EN "zMin : %zm"
#define UI_TEXT_ZPROBING_EN "Probing..."
#define UI_TEXT_ZPOSITION_EN "Go position..."
#define UI_TEXT_DO_YOU_EN "Do you want to:"
#define UI_TEXT_CLEAN1_EN "Clean nozzle and"
#define UI_TEXT_CLEAN2_EN "plates first ?"
#define UI_TEXT_DISPLAY_MODE_EN "Mode:%Md"
#define UI_TEXT_EASY_MODE_EN "      Easy"
#define UI_TEXT_ADVANCED_MODE_EN "  Advanced"
#define UI_TEXT_POWER_SAVE_EN "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_EN "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_EN "Sound:      %so"
#define UI_TEXT_YES_SELECTED_EN "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_EN "   Yes    [No] "
#define UI_TEXT_PLEASE_CONFIRM_EN "Please confirm"
#define UI_TEXT_CANCEL_ACTION_EN "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_EN "Continue"
#define UI_TEXT_CANCELED_EN "Action canceled!"
#define UI_TEXT_CONFIRMED_EN "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_EN "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_EN "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_EN "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_EN "And Clean"
#define UI_TEXT_PUSH_FILAMENT_EN "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_EN "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_EN "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_EN "Load Fail-Safe"
#define UI_TEXT_HEATING_EN "Heating..."
#define UI_TEXT_WAIT_FOR_OK_EN "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_EN  "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_EN  "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_EN "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_EN "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_EN "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_EN "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_EN "Z:%x2"
#define UI_TEXT_P1_L3_EN "Mul:%om%%%Buf:%oB"

#define UI_TEXT_P1_L1_EASY_EN "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_EN "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_EN "Z:%x2"

#else
#define UI_TEXT_P1_L1_EN "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EN "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_EN "Z:%x2"

#define UI_TEXT_P1_L1_EASY_EN "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_EN "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_EN "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_EN "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EN "Z:%x2"
#define UI_TEXT_P1_L3_EN "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_EN "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_EN "Z:%x2"
#define UI_TEXT_P1_L3_EASY_EN "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_EN "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EN "Z:%x2"
#define UI_TEXT_P1_L3_EN "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_EN "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_EN "Z:%x2"
#define UI_TEXT_P1_L3_EASY_EN "Buffer:%oB"
#endif
#endif
#define UI_TEXT_MENU_SCANNER_EN    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_EN    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_EN        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_EN       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_EN           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_EN          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_EN    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_EN     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_EN      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_EN      "Zmin evaluation"

// *************** German translation ****************

#define UI_TEXT_ON_DE               "An"
#define UI_TEXT_OFF_DE              "Aus"
#define UI_TEXT_NA_DE               "nv"
#define UI_TEXT_YES_DE              "Ja"
#define UI_TEXT_NO_DE               "Nein"
#define UI_TEXT_PRINT_POS_DE        "Drucke..."
#define UI_TEXT_PRINTING_DE         "Drucken"
#define UI_TEXT_IDLE_DE             "Leerlauf"
#define UI_TEXT_NOSDCARD_DE         "Keine SD Karte"
#define UI_TEXT_ERROR_DE            "**** FEHLER ****"
#define UI_TEXT_BACK_DE             "Zur" STR_uuml "ck " cUP
#define UI_TEXT_QUICK_SETTINGS_DE   "Schnelleinst."
#define UI_TEXT_ERRORMSG_DE         "%oe"
#define UI_TEXT_CONFIGURATION_DE    "Konfiguration"
#define UI_TEXT_POSITION_DE         "Position"
#define UI_TEXT_EXTRUDER_DE         "Extruder"
#define UI_TEXT_SD_CARD_DE          "SD Karte"
#define UI_TEXT_DEBUGGING_DE        "Debugging"
#define UI_TEXT_HOME_DELTA_DE       "Home Delta"
#define UI_TEXT_HOME_ALL_DE         "Home Alle"
#define UI_TEXT_HOME_X_DE           "Home X"
#define UI_TEXT_HOME_Y_DE           "Home Y"
#define UI_TEXT_HOME_Z_DE           "Home Z"
#define UI_TEXT_PREHEAT_PLA_DE      "Vorheizen PLA %B1"
#define UI_TEXT_PREHEAT_ABS_DE      "Vorheizen ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_DE     "Lampen: %lo"
#define UI_TEXT_COOLDOWN_DE         "Abk" STR_uuml "hlen"
#define UI_TEXT_SET_TO_ORIGIN_DE    "Setze Nullpunkt"
#define UI_TEXT_DISABLE_STEPPER_DE  "Motoren Aussch."
#define UI_TEXT_X_POSITION_DE       "X Position"
#define UI_TEXT_X_POS_FAST_DE       "X Pos. Schnell"
#define UI_TEXT_Y_POSITION_DE       "Y Position"
#define UI_TEXT_Y_POS_FAST_DE       "Y Pos. Schnell"
#define UI_TEXT_Z_POSITION_DE       "Z Position"
#define UI_TEXT_Z_POS_FAST_DE       "Z Pos. Schnell"
#define UI_TEXT_E_POSITION_DE       "Extr. Position"
#define UI_TEXT_BED_TEMP_DE         "Bed Temp:%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 2
#define UI_TEXT_EXTR0_TEMP_DE         "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_DE         "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_DE         "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_DE         "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 2
#define UI_TEXT_EXTR0_OFF_DE          "Extruder Aus %B4"
#else
#define UI_TEXT_EXTR0_OFF_DE          "Extruder 1 Aus%B4"
#endif
#define UI_TEXT_EXTR1_OFF_DE          "Extruder 2 Aus%B5"
#define UI_TEXT_EXTR2_OFF_DE          "Extruder 3 Aus%B6"
#define UI_TEXT_EXTR0_SELECT_DE     "%X0 W" STR_auml "hle Extr. 1"
#define UI_TEXT_EXTR1_SELECT_DE     "%X1 W" STR_auml "hle Extr. 2"
#define UI_TEXT_EXTR2_SELECT_DE     "%X2 W" STR_auml "hle Extr. 3"
#define UI_TEXT_EXTR_ORIGIN_DE      "Setze Nullpunkt"
#define UI_TEXT_PRINT_X_DE          "Drucken X:%ax"
#define UI_TEXT_PRINT_Y_DE          "Drucken Y:%ay"
#define UI_TEXT_PRINT_Z_DE          "Drucken Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_DE    "Drucken:%az"
#define UI_TEXT_MOVE_X_DE           "Bewegen X:%aX"
#define UI_TEXT_MOVE_Y_DE           "Bewegen Y:%aY"
#define UI_TEXT_MOVE_Z_DE           "Bewegen Z:%aZ"
#define UI_TEXT_MOVE_Z_DELTA_DE     "Bewegen:%aZ"
#define UI_TEXT_JERK_DE             "Ruck     :%aj"
#define UI_TEXT_ZJERK_DE            "Z-Ruck   :%aJ"
#define UI_TEXT_ACCELERATION_DE     "Beschleunigung"
#define UI_TEXT_STORE_TO_EEPROM_DE  "Sichere EEPROM"
#define UI_TEXT_LOAD_EEPROM_DE      "Lade vom EEPROM"
#define UI_TEXT_DBG_ECHO_DE         "Echo       :%do"
#define UI_TEXT_DBG_INFO_DE         "Info       :%di"
#define UI_TEXT_DBG_ERROR_DE        "Fehler     :%de"
#define UI_TEXT_DBG_DRYRUN_DE       "Trockenlauf:%dd"
#define UI_TEXT_DBG_ENDSTOP_DE      "EndStop    :%dp"
#define UI_TEXT_OPS_OFF_DE          "%O0 OPS Aus"
#define UI_TEXT_OPS_CLASSIC_DE      "%O1 OPS Klassisch"
#define UI_TEXT_OPS_FAST_DE         "%O2 OPS Schnell"
#define UI_TEXT_OPS_RETRACT_DE      "Einfahren :%Or"
#define UI_TEXT_OPS_BACKSLASH_DE    "Backsl.   :%Ob"
#define UI_TEXT_OPS_MINDIST_DE      "Min.Abst. :%Od"
#define UI_TEXT_OPS_MOVE_AFTER_DE   "Start nach:%Oa"
#define UI_TEXT_ANTI_OOZE_DE        "Anti Ooze"
#define UI_TEXT_PRINT_FILE_DE       "Drucke Datei"
#define UI_TEXT_PAUSE_PRINT_DE      "Druck Pausieren"
#define UI_TEXT_CONTINUE_PRINT_DE   "Druck Forts."
#define UI_TEXT_UNMOUNT_CARD_DE     "Unmount Karte"
#define UI_TEXT_MOUNT_CARD_DE       "Mount Karte"
#define UI_TEXT_DELETE_FILE_DE      "Datei l" STR_ouml "schen"
#define UI_TEXT_FEEDRATE_DE         "Feedrate"
#define UI_TEXT_FEED_MAX_X_DE       "Max X:%fx"
#define UI_TEXT_FEED_MAX_Y_DE       "Max Y:%fy"
#define UI_TEXT_FEED_MAX_Z_DE       "Max Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_DE "Max:%fz"
#define UI_TEXT_FEED_HOME_X_DE      "Home X:%fX"
#define UI_TEXT_FEED_HOME_Y_DE      "Home Y:%fY"
#define UI_TEXT_FEED_HOME_Z_DE      "Home Z:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_DE "Home:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_DE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_DE "Min Endstopp:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_DE "Max Endstopp:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_DE ""
#define UI_TEXT_ACTION_YPOSITION4A_DE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_DE "Min Endstopp:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_DE "Max Endstopp:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_DE ""
#define UI_TEXT_ACTION_ZPOSITION4A_DE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_DE "Min Endstopp:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_DE "Max Endstopp:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_DE ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_DE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_DE "Min Endstopp:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_DE "Max Endstopp:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_DE ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_DE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_DE "Min Endstopp:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_DE "Max Endstopp:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_DE ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_DE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_DE "Min Endstopp:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_DE "Max Endstopp:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_DE ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_DE "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_DE "1 klick = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_DE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_DE "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_DE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_DE "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_DE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_DE "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_DE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_DE "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_DE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_DE "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_DE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_DE "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_DE          "L" STR_uuml "fter"
#define UI_TEXT_ACTION_FANSPEED_DE   "L" STR_uuml "fter:%Fs%%%"
#define UI_TEXT_FAN_OFF_DE           "L" STR_uuml "fter Aus"
#define UI_TEXT_FAN_25_DE            "L" STR_uuml "fter auf 25%%%"
#define UI_TEXT_FAN_50_DE            "L" STR_uuml "fter auf 50%%%"
#define UI_TEXT_FAN_75_DE            "L" STR_uuml "fter auf 75%%%"
#define UI_TEXT_FAN_FULL_DE          "L" STR_uuml "fter Voll"
#define UI_TEXT_STEPPER_INACTIVE_DE  "Motor Inaktiv"
#define UI_TEXT_STEPPER_INACTIVE2A_DE "Aus nach: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_DE "[min] 0=Aus"
#define UI_TEXT_POWER_INACTIVE_DE    "Max. Inaktiv"
#define UI_TEXT_POWER_INACTIVE2A_DE   "Aus nach: %ip"
#define UI_TEXT_POWER_INACTIVE2B_DE   "[min] 0=Aus"
#define UI_TEXT_GENERAL_DE           "Allgemein"
#define UI_TEXT_BAUDRATE_DE          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_DE        "Schr/MM:%Se"
#define UI_TEXT_EXTR_START_FEED_DE   "Start FR:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_DE     "Max FR:%XF"
#define UI_TEXT_EXTR_ACCEL_DE        "Beschl.:%XA"
#define UI_TEXT_EXTR_WATCH_DE        "Stab.Zeit:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_DE    "Advance lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_DE    "Advance quad:%Xa"
#define UI_TEXT_EXTR_MANAGER_DE      "Regler:%Xh"
#define UI_TEXT_EXTR_PGAIN_DE        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_DE     "Totzeit:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_DE      "Controll-PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_DE        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_DE        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_DE         "Drive Min:%Xm"
#define UI_TEXT_EXTR_DMAX_DE         "Drive Max:%XM"
#define UI_TEXT_EXTR_PMAX_DE         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_DE "BangBang"
#define UI_TEXT_STRING_HM_PID_DE     "PID"
#define UI_TEXT_STRING_ACTION_DE     "Aktion:%la"
#define UI_TEXT_HEATING_EXTRUDER_DE  "Heize Extruder %Ne "
#define UI_TEXT_HEATING_BED_DE       "Heize Druckbett"
#define UI_TEXT_KILLED_DE            "Angehalten"
#define UI_TEXT_STEPPER_DISABLED_DE  "Motoren Aus"
#define UI_TEXT_EEPROM_STOREDA_DE     "Konfiguration"
#define UI_TEXT_EEPROM_STOREDB_DE     "gespeichert."
#define UI_TEXT_EEPROM_LOADEDA_DE     "Konfiguration"
#define UI_TEXT_EEPROM_LOADEDB_DE     "geladen."
#define UI_TEXT_UPLOADING_DE         "Hochladen..."
#define UI_TEXT_PAGE_BUFFER_DE       "Puffer:%oB"
#define UI_TEXT_PAGE_EXTRUDER_DE     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_DE     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_DE     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_DE     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_DE          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_DE    "Geschw.Mul:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_DE     "Flow Mul.:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_DE  "Zeige Messung"
#define UI_TEXT_RESET_MEASUREMENT_DE "Reset Messung"
#define UI_TEXT_SET_MEASURED_ORIGIN_DE "Setze Z=0"
#define UI_TEXT_ZCALIB_DE        "Z Kalib."
#define UI_TEXT_SET_P1_DE       "Setze P1"
#define UI_TEXT_SET_P2_DE       "Setze P2"
#define UI_TEXT_SET_P3_DE       "Setze P3"
#define UI_TEXT_CALCULATE_LEVELING_DE "Berechne Leveling"
#define UI_TEXT_LEVEL_DE        "Level delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_DE  "Wartetemp.%XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_DE "R" STR_uuml "ckz. um:%XU mm"
#define UI_TEXT_SD_REMOVED_DE       "SD Karte entfernt"
#define UI_TEXT_SD_INSERTED_DE      "SD Karte eingelegt"
#define UI_TEXT_PRINTER_READY_DE    "Drucker bereit."
#define UI_TEXT_PRINTTIME_DAYS_DE   " Tage "
#define UI_TEXT_PRINTTIME_HOURS_DE  ":"
#define UI_TEXT_PRINTTIME_MINUTES_DE ""
#define UI_TEXT_PRINT_TIME_DE     "Ges. Druckzeit"
#define UI_TEXT_PRINT_FILAMENT_DE "Filament gedruckt"
#define UI_TEXT_PRINTED_DE           "gedruckt"
#define UI_TEXT_POWER_DE            "ATX Netzteil an/aus"
#define UI_TEXT_STRING_HM_DEADTIME_DE     "Totzeit"
#define UI_TEXT_STRING_HM_SLOWBANG_DE     "Langs.BB"
#define UI_TEXT_STOP_PRINT_DE "Druck abbrechen"
#define UI_TEXT_Z_BABYSTEPPING_DE "Z Babyschr.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_DE "Filamentwechsel"
#define UI_TEXT_WIZ_CH_FILAMENT1_DE "Filamentwechsel:"
#define UI_TEXT_WIZ_CH_FILAMENT2_DE "Zum man. f" STR_ouml "rdern"
#define UI_TEXT_WIZ_CH_FILAMENT3_DE "Men" STR_uuml "knopf drehen"
#define UI_TEXT_CLICK_DONE_DE "Weiter mit Klick"
#define UI_TEXT_AUTOLEVEL_ONOFF_DE  "Autolevel: %ll"
#define UI_TEXT_SERVOPOS_DE "Servo Pos.: %oS"
#define UI_TEXT_IGNORE_M106_DE      "Ignoriere M106 %Fi"
#define UI_TEXT_WIZ_REHEAT1_DE "Klicke zum Aufw" STR_auml "rmen"
#define UI_TEXT_WIZ_REHEAT2_DE "der Extruder."
#define UI_TEXT_WIZ_WAITTEMP1_DE "Warte auf"
#define UI_TEXT_WIZ_WAITTEMP2_DE "Zieltemperatur..."
#define UI_TEXT_EXTRUDER_JAM_DE "Extruderstau"
#define UI_TEXT_STANDBY_DE "Standby"
#define UI_TEXT_BED_COATING_DE "Bettbeschichtung"
#define UI_TEXT_BED_COATING_SET1_DE "Bettbeschichtung:"
#define UI_TEXT_BED_COATING_SET2_DE ""
#define UI_TEXT_NOCOATING_DE "Unbeschichtet"
#define UI_TEXT_BUILDTAK_DE "BuildTak"
#define UI_TEXT_KAPTON_DE "Kapton"
#define UI_TEXT_BLUETAPE_DE "Blaues Kreppband"
#define UI_TEXT_PETTAPE_DE "Gr" STR_uuml "nes PET Band"
#define UI_TEXT_GLUESTICK_DE "Klebestift"
#define UI_TEXT_CUSTOM_DE "Individuell"
#define UI_TEXT_COATING_CUSTOM_DE "Indiv.:%BCmm"
#define UI_TEXT_LANGUAGE_DE "Sprache"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_DE "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_DE "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_DE "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_DE "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_DE "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_DE "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_DE "Kopien: %ed    Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_DE "Fluss:\xfd %of%%%  Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_DE "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_DE "Puf: %oB"
#define UI_TEXT_MAINPAGE6_6_DE "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_DE cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_DE "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_DE "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_DE "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_DE "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_DE "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_DE "%Uf m"
#define UI_TEXT_METER_PRINTED_DE "%Uf m " UI_TEXT_PRINTED_DE
#define UI_TEXT_STATUS_DE "%os"
#define UI_TEXT_EMPTY_DE ""
#define UI_TEXT_TEMP_SET_DE cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_DE cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_DE " %BCmm"
#define UI_TEXT_EXTR3_TEMP_DE "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_DE "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_DE "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_DE "Extruder 4 Aus"
#define UI_TEXT_EXTR4_OFF_DE "Extruder 5 Aus"
#define UI_TEXT_EXTR5_OFF_DE "Extruder 6 Aus"
#define UI_TEXT_EXTR3_SELECT_DE "%X3 W" STR_auml "hle Extr. 4"
#define UI_TEXT_EXTR4_SELECT_DE "%X4 W" STR_auml "hle Extr. 5"
#define UI_TEXT_EXTR5_SELECT_DE "%X5 W" STR_auml "hle Extr. 6"
#define UI_TEXT_DITTO_0_DE "%D0 Keine Kopien"
#define UI_TEXT_DITTO_1_DE "%D1 1 Kopie"
#define UI_TEXT_DITTO_2_DE "%D2 2 Kopien"
#define UI_TEXT_DITTO_3_DE "%D3 3 Kopien"
#define UI_TEXT_ZPROBE_HEIGHT_DE "Z-Probenh" STR_ouml "he:%zh"
#define UI_TEXT_OFFSETS_DE "Set print offsets"
#define UI_TEXT_X_OFFSET_DE "X-Offset:%T0mm"
#define UI_TEXT_Y_OFFSET_DE "Y-Offset:%T1mm"
#define UI_TEXT_Z_OFFSET_DE "Z-Offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_DE   "Filament laden"
#define UI_TEXT_UNLOAD_FILAMENT_DE   "Filament entladen"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_DE   "F" STR_uuml "ll.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_DE   "Abfallbeh" STR_auml  "lter leeren"
#define UI_TEXT_CLEAN_DRIPBOX_DE   "Leere Abfallbeh" STR_auml  "lter"
#define UI_TEXT_CLEAN_DRIPBOX_3_DE   "Abfallbeh" STR_auml  "lter"
#define UI_TEXT_EXTR_XOFF_DE   "X Verschiebung:%Xx"
#define UI_TEXT_EXTR_YOFF_DE   "Y Verschiebung:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_DE   "F" STR_uuml "ll.Sensoren:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_DE   "Leere Abfallbeh" STR_auml  "lter"
#define UI_TEXT_CLEANING_DRIPBOX_DE   "Abfallbeh" STR_auml  "lter leeren"
#define UI_TEXT_CLEAN_DRIPBOX_3_DE   "Abfallbeh" STR_auml  "lter"
#define UI_TEXT_EXTR_XOFF_DE   "X Verschiebung:%Xx"
#define UI_TEXT_EXTR_YOFF_DE   "Y Verschiebung:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_DE   "Lade rechts \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_DE   "Entlade rechts \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_DE   "Lade links \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_DE   "Entlade links \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_DE   "Lade Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_DE   "Entlade F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_DE   "Lade dist.: %fd"
#define UI_TEXT_BED_DOWN_DE   "Bett senken"
#define UI_TEXT_WIFI_ONOFF_DE   "WLAN:       %wo"
#define UI_TEXT_REDO_ACTION_DE   "Wiederholen ?"
#define UI_TEXT_X_LENGTH_DE   "X L" STR_auml  "nge:%Lxmm"
#define UI_TEXT_Y_LENGTH_DE   "Y L" STR_auml  "nge:%Lymm"
#define UI_TEXT_Z_LENGTH_DE   "Z L" STR_auml  "nge:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_DE   "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_DE   "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_DE   "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_DE   "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_DE   "Extruder%Ne hat kein"
#define UI_TEXT_ALERT_NO_FILAMENTB_DE   "Filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_DE   "weiter mit OK"
#define UI_TEXT_ALERT_NO_FILAMENTD_DE   "Filament wechseln"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_DE   "Nimm die 3 Muttern"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_DE   "am Boden von"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_DE   "der Plattform und"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_DE   "ziehe sie 4 Umdr. an"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_DE   "Ich werde den"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_DE   "Extruder an einige"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_DE   "Positionen zur"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_DE   "Einstellung bewegen."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_DE   "Auf jeder Position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_DE   "werden wir 2"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_DE   "Muttern gleichzeitg"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_DE   "drehen müssen."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_DE   "Die Düse ist im"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_DE   "richtigen Abstand, wenn"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_DE   "man einfach ein"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_DE   "Blatt Papier gerade"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_DE   "zwischen schieben"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_DE   "kann. Nimm ein"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_DE   "Blatt Papier um"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_DE   "fortzufahren."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_DE   "Stelle die vordere"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_DE   "Mutter so ein, damit das"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_DE   "Papier zwischen D" STR_uuml  "se"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_DE   "und Bett passt"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_DE   "Stelle die hinteren"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_DE   "Muttern, damit das"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_DE   "Papier zwischen D" STR_uuml  "se"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_DE   "und Bett passt"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_DE   "Stelle die hintere und"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_DE   "rechte Mutter so ein, dass"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_DE   "das Papier wieder"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_DE   "gerade passt."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_DE   "Stelle die vordere und"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_DE   "linke Mutter so ein, dass"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_DE   "das Papier wieder"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_DE   "gerade passt."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_DE   "Kontrolliere, ob das"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_DE   "Papier gerade zwischen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_DE   "Bett und D" STR_uuml  "s"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_DE   "passt."
#define UI_TEXT_X_MIN_SETTING_DE   "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_DE   "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_DE   "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_DE   "Deckel offen!"
#define UI_TEXT_TOP_SENSOR_ONOFF_DE   "Deckel Sensor: %st"
#define UI_TEXT_VERSION_MENU_DE   "System Version"
#define UI_TEXT_COOLDOWN_MENU_DE   "Abk" STR_uuml  "hlen     %B3"
#define UI_TEXT_WAIT_OK_DE   "weiter mit OK"
#define UI_TEXT_FAILED_DE   "Fehlgeschlagen"
#define UI_TEXT_PLEASE_WAIT_DE   "Bitte warten"
#define UI_TEXT_MANUAL_LEVEL_DE   "Manuelles Ausrichten"
#define UI_TEXT_WARNING_DE   "Achtung!"
#define UI_TEXT_EXTRUDER_COLD_DE   "E%Ne ist zu kalt!"
#define UI_TEXT_HEAT_EXTRUDER_DE   "Extruder heizen?"
#define UI_TEXT_WRITE_DEBUG_DE   "Fehler schreiben"
#define UI_TEXT_POWER_SAVE_MENU_DE   "Energie sparen"
#define UI_TEXT_MAINTENANCE_DE   "Wartung"
#define UI_TEXT_SETTINGS_DE   "Einstellungen"
#define UI_TEXT_LOAD_UNLOAD_DE   "Be-/Entladen"
#define UI_TEXT_BED_DE   "Bett"
#define UI_TEXT_BED_OFF_DE   "Bett ausschalten %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_DE   "Beleuchtung autom. aus   "
#define UI_TEXT_LIGHT_OFF_AFTER2A_DE   "aus nach:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_DE   "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_DE   "X-Pos. unbekannt!"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_DE   "Y-Pos. unbekannt!"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_DE   "Z-Pos. unbekannt!"
#define UI_TEXT_MOVING_DE   "Fahre..."
#define UI_TEXT_PREPARING_DE   "Bereite vor..."
#define UI_TEXT_KEEP_LIGHT_ON_DE   "Belechtung:%lk"
#define UI_TEXT_LOADING_FILAMENT_DE   "Laden \005"
#define UI_TEXT_UNLOADING_FILAMENT_DE   "Entladen \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_DE   "Be-/Entladen \005"
#define UI_TEXT_AUTOLEVEL_DE   "Autom. Bett kalibrieren"
#define UI_TEXT_Z_PROBE_FAILED_DE   "Z-Antasten fehlgeschlagen"
#define UI_TEXT_SAVE_DE   "Auf EEPROM speichern?"
#define UI_TEXT_ZMIN_DE   "zMin : %zm"
#define UI_TEXT_ZPROBING_DE   "Antasten..."
#define UI_TEXT_ZPOSITION_DE   "Positionieren..."
#define UI_TEXT_DO_YOU_DE   "Wollen Sie:"
#define UI_TEXT_CLEAN1_DE   "Bett und D" STR_uuml  "se zuerst"
#define UI_TEXT_CLEAN2_DE   "reinigen ?"
#define UI_TEXT_DISPLAY_MODE_DE   "Modus:%Md"
#define UI_TEXT_EASY_MODE_DE   "Einfach"
#define UI_TEXT_ADVANCED_MODE_DE   "Fortgeschritten"
#define UI_TEXT_POWER_SAVE_DE   "Energie sparen:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_DE   "Tastenbeleuchtung:%lb"
#define UI_TEXT_SOUND_ONOF_DE   "Ton:  %so"
#define UI_TEXT_YES_SELECTED_DE   "  [Ja]    Nein"
#define UI_TEXT_NO_SELECTED_DE   "   Ja    [Nein]"
#define UI_TEXT_PLEASE_CONFIRM_DE  "Bitte best" STR_auml  "tigen"
#define UI_TEXT_CANCEL_ACTION_DE   "Abbrechen:"
#define UI_TEXT_CONTINUE_ACTION_DE   "Weiter"
#define UI_TEXT_CANCELED_DE   "Aktion abgebrochen!"
#define UI_TEXT_CONFIRMED_DE   "Aktion best" STR_auml  "tigt!"
#define UI_TEXT_CLEAN_NOZZLE_DE   "D" STR_uuml  "se reinigen"
#define UI_TEXT_CLEANING_NOZZLE_DE   "Reinige D" STR_uuml  "se"
#define UI_TEXT_CLEAN_DRIPBOX_1_DE   "Bitte nimm"
#define UI_TEXT_CLEAN_DRIPBOX_2_DE   "und reinie"
#define UI_TEXT_PUSH_FILAMENT_DE   "Schiebe Filament"
#define UI_TEXT_WAIT_FILAMENT_DE   "Dr" STR_uuml  "cke OK zum starten"
#define UI_TEXT_LOAD_FAILSAFE2_DE   "nachdem laden ?"
#define UI_TEXT_LOAD_FAILSAFE_DE   "Lade Fail-Safe"
#define UI_TEXT_HEATING_DE   "Aufheizen..."
#define UI_TEXT_WAIT_FOR_OK_DE   "Ende mit OK"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_DE    "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_DE    "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_DE   "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_DE   "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_DE   "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci spezifisch, Seite 1-2-3, sind anders bei 1 oder 2 Extrudern,und bei Einfach- oder Expertenmodus
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_DE "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_DE "Z:%x2"
#define UI_TEXT_P1_L3_DE "Mul:%om Puf:%oB"

#define UI_TEXT_P1_L1_EASY_DE "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_DE "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_DE "Z:%x2"

#else
#define UI_TEXT_P1_L1_DE "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_DE "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_DE "Z:%x2"

#define UI_TEXT_P1_L1_EASY_DE "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_DE "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_DE "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_DE "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_DE "Z:%x2"
#define UI_TEXT_P1_L3_DE "Puffer:%oB"

#define UI_TEXT_P1_L1_EASY_DE "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_DE "Z:%x2"
#define UI_TEXT_P1_L3_EASY_DE "Puffer:%oB"
#else
#define UI_TEXT_P1_L1_DE "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_DE "Z:%x2"
#define UI_TEXT_P1_L3_DE "Puffer:%oB"

#define UI_TEXT_P1_L1_EASY_DE "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_DE "Z:%x2"
#define UI_TEXT_P1_L3_EASY_DE "Puffer:%oB"
#endif
#endif
#define UI_TEXT_MENU_SCANNER_DE    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_DE    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_DE        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_DE       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_DE           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_DE          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_DE    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_DE     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_DE      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_DE      "Zmin evaluation"

// Dutch translation

#define UI_TEXT_ON_NL               "Aan"
#define UI_TEXT_OFF_NL              "Uit"
#define UI_TEXT_NA_NL               "N/A" // Output for not available
#define UI_TEXT_YES_NL              "Ja"
#define UI_TEXT_NO_NL               "Nee"
#define UI_TEXT_PRINT_POS_NL        "Printen..."
#define UI_TEXT_PRINTING_NL         "Printen"
#define UI_TEXT_IDLE_NL             "Rust"
#define UI_TEXT_NOSDCARD_NL         "Geen SD Kaart"
#define UI_TEXT_ERROR_NL            "**** FOUT ****"
#define UI_TEXT_BACK_NL             "Terug " cUP
#define UI_TEXT_QUICK_SETTINGS_NL   "Snelle Instelling"
#define UI_TEXT_ERRORMSG_NL         "%oe"
#define UI_TEXT_CONFIGURATION_NL    "Configuratie"
#define UI_TEXT_POSITION_NL         "Positie"
#define UI_TEXT_EXTRUDER_NL         "Extruder"
#define UI_TEXT_SD_CARD_NL          "SD Kaart"
#define UI_TEXT_DEBUGGING_NL        "Debugging"
#define UI_TEXT_HOME_DELTA_NL       "Home Delta"
#define UI_TEXT_HOME_ALL_NL         "Home Alles"
#define UI_TEXT_HOME_X_NL           "Home X"
#define UI_TEXT_HOME_Y_NL           "Home Y"
#define UI_TEXT_HOME_Z_NL           "Home Z"
#define UI_TEXT_PREHEAT_PLA_NL      "Voorverwarmen PLA %B1"
#define UI_TEXT_PREHEAT_ABS_NL      "Voorverwarmen ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_NL     "Lichten:%lo"
#define UI_TEXT_COOLDOWN_NL         "Koelen"
#define UI_TEXT_SET_TO_ORIGIN_NL    "Zet oorsprong"
#define UI_TEXT_DISABLE_STEPPER_NL  "Zet motor uit"
#define UI_TEXT_X_POSITION_NL       "X Positie"
#define UI_TEXT_X_POS_FAST_NL       "X Pos. Snel"
#define UI_TEXT_Y_POSITION_NL       "Y Positie"
#define UI_TEXT_Y_POS_FAST_NL       "Y Pos. Snel"
#define UI_TEXT_Z_POSITION_NL       "Z Positie"
#define UI_TEXT_Z_POS_FAST_NL       "Z Pos. Snel"
#define UI_TEXT_E_POSITION_NL       "Extr. positie"
#define UI_TEXT_BED_TEMP_NL         "Bed Temp:%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_NL       "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_NL       "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_NL       "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_NL       "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_NL        "Extruder Uit %B4"
#else
#define UI_TEXT_EXTR0_OFF_NL        "Extruder 1 Uit %B4"
#endif
#define UI_TEXT_EXTR1_OFF_NL        "Extruder 2 Uit %B5"
#define UI_TEXT_EXTR2_OFF_NL        "Extruder 3 Uit %B6"
#define UI_TEXT_EXTR0_SELECT_NL     "%X0 Select Extr. 1"
#define UI_TEXT_EXTR1_SELECT_NL     "%X1 Select Extr. 2"
#define UI_TEXT_EXTR2_SELECT_NL     "%X2 Select Extr. 3"
#define UI_TEXT_EXTR_ORIGIN_NL      "Zet oorsprong"
#define UI_TEXT_PRINT_X_NL          "Print X:%ax"
#define UI_TEXT_PRINT_Y_NL          "Print Y:%ay"
#define UI_TEXT_PRINT_Z_NL          "Print Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_NL    "Print:%az"
#define UI_TEXT_MOVE_X_NL           "Beweeg X:%aX"
#define UI_TEXT_MOVE_Y_NL           "Beweeg Y:%aY"
#define UI_TEXT_MOVE_Z_NL           "Beweeg Z:%aZ"
#define UI_TEXT_MOVE_Z_DELTA_NL     "Beweeg:%aZ"
#define UI_TEXT_JERK_NL             "Ruk:%aj"
#define UI_TEXT_ZJERK_NL            "Z-Ruk:%aJ"
#define UI_TEXT_ACCELERATION_NL     "Acceleratie"
#define UI_TEXT_STORE_TO_EEPROM_NL  "Opslaan n. EEPROM"
#define UI_TEXT_LOAD_EEPROM_NL      "Ladd f. EEPROM"
#define UI_TEXT_DBG_ECHO_NL         "Echo     :%do"
#define UI_TEXT_DBG_INFO_NL         "Info     :%di"
#define UI_TEXT_DBG_ERROR_NL        "Fouten   :%de"
#define UI_TEXT_DBG_DRYRUN_NL       "Droogloop:%dd"
#define UI_TEXT_DBG_ENDSTOP_NL      "EndStop  :%dp"
#define UI_TEXT_OPS_OFF_NL          "%O0 OPS Uit"
#define UI_TEXT_OPS_CLASSIC_NL      "%O1 OPS Klassiek"
#define UI_TEXT_OPS_FAST_NL         "%O2 OPS Snel"
#define UI_TEXT_OPS_RETRACT_NL      "Terugtrekken:%Or"
#define UI_TEXT_OPS_BACKSLASH_NL    "Backsl. :%Ob"
#define UI_TEXT_OPS_MINDIST_NL      "Min. afstand:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_NL   "Beweeg na:%Oa"
#define UI_TEXT_ANTI_OOZE_NL        "Anti lekken"
#define UI_TEXT_PRINT_FILE_NL       "Print bestand"
#define UI_TEXT_PAUSE_PRINT_NL      "Pauzeer Print"
#define UI_TEXT_CONTINUE_PRINT_NL   "Zet print voort"
#define UI_TEXT_UNMOUNT_CARD_NL     "Ontkoppel Kaart"
#define UI_TEXT_MOUNT_CARD_NL       "Koppel Kaart"
#define UI_TEXT_DELETE_FILE_NL      "Verw. bestand"
#define UI_TEXT_FEEDRATE_NL         "Beweeg snelheid"
#define UI_TEXT_FEED_MAX_X_NL       "Max X:%fx"
#define UI_TEXT_FEED_MAX_Y_NL       "Max Y:%fy"
#define UI_TEXT_FEED_MAX_Z_NL       "Max Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_NL "Max:%fz"
#define UI_TEXT_FEED_HOME_X_NL      "Home X:%fX"
#define UI_TEXT_FEED_HOME_Y_NL      "Home Y:%fY"
#define UI_TEXT_FEED_HOME_Z_NL      "Home Z:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_NL "Home:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_NL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_NL "Min eindst.:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_NL "Max eindst.:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_NL ""
#define UI_TEXT_ACTION_YPOSITION4A_NL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_NL "Min eindst.:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_NL "Max eindst.:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_NL ""
#define UI_TEXT_ACTION_ZPOSITION4A_NL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_NL "Min eindst.:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_NL "Max eindst.:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_NL ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_NL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_NL "Min eindst.:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_NL "Max eindst.:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_NL ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_NL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_NL "Min eindst.:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_NL "Max eindst.:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_NL ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_NL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_NL "Min eindst.:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_NL "Max eindst.:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_NL ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_NL "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_NL "1 klik = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_NL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_NL "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_NL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_NL "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_NL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_NL "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_NL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_NL "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_NL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_NL "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_NL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_NL "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_NL          "Fan snelheid"
#define UI_TEXT_FAN_OFF_NL           "Zet Fan uit"
#define UI_TEXT_ACTION_FANSPEED_NL   "Fan snelheid:%Fs%%%"
#define UI_TEXT_FAN_25_NL            "Zet Fan 25%%%"
#define UI_TEXT_FAN_50_NL            "Zet Fan 50%%%"
#define UI_TEXT_FAN_75_NL            "Zet Fan 75%%%"
#define UI_TEXT_FAN_FULL_NL          "Zet Fan Vol aan"
#define UI_TEXT_STEPPER_INACTIVE_NL  "Motor Inactief"
#define UI_TEXT_STEPPER_INACTIVE2A_NL "Uit na: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_NL "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_NL    "Max. Inactief"
#define UI_TEXT_POWER_INACTIVE2A_NL   "Zet uit na: %ip"
#define UI_TEXT_POWER_INACTIVE2B_NL   "[min] 0=Off"
#define UI_TEXT_GENERAL_NL           "Algemeen"
#define UI_TEXT_BAUDRATE_NL          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_NL        "Stappen/MM:%Se"
#define UI_TEXT_EXTR_START_FEED_NL   "Start FR.:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_NL     "Max FR.:%XF"
#define UI_TEXT_EXTR_ACCEL_NL        "Accel:%XA"
#define UI_TEXT_EXTR_WATCH_NL        "Stab.Tijd:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_NL    "Advance lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_NL    "Advance quad:%Xa"
#define UI_TEXT_EXTR_MANAGER_NL      "Control:%Xh"
#define UI_TEXT_EXTR_PGAIN_NL        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_NL     "Dode tijd:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_NL      "Controle PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_NL        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_NL        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_NL         "Drive Min:%Xm"
#define UI_TEXT_EXTR_DMAX_NL         "Drive Max:%XM"
#define UI_TEXT_EXTR_PMAX_NL         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_NL "BangBang"
#define UI_TEXT_STRING_HM_PID_NL     "PID"
#define UI_TEXT_STRING_ACTION_NL     "Action:%la"
#define UI_TEXT_HEATING_EXTRUDER_NL  "Opwarmen Ext%Ne"
#define UI_TEXT_HEATING_BED_NL       "Opwarmen Bed"
#define UI_TEXT_KILLED_NL            "Uitgeschakeld"
#define UI_TEXT_STEPPER_DISABLED_NL  "Motor uitgezet"
#define UI_TEXT_EEPROM_STOREDA_NL     "Configuratie"
#define UI_TEXT_EEPROM_STOREDB_NL     "saved. in EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_NL     "Configuratie"
#define UI_TEXT_EEPROM_LOADEDB_NL     "loaded f. EEPROM"
#define UI_TEXT_UPLOADING_NL         "Uploaden..."
#define UI_TEXT_PAGE_BUFFER_NL       "Buffer:%oB"
#define UI_TEXT_PAGE_EXTRUDER_NL     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_NL     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_NL     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_NL     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_NL          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_NL    "Snelh. Mul.:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_NL     "Flow Mul.:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_NL  "Show meting"
#define UI_TEXT_RESET_MEASUREMENT_NL "Reset meting"
#define UI_TEXT_SET_MEASURED_ORIGIN_NL "Set Z=0"
#define UI_TEXT_ZCALIB_NL             "Z Calib."
#define UI_TEXT_SET_P1_NL            "Set P1"
#define UI_TEXT_SET_P2_NL            "Set P2"
#define UI_TEXT_SET_P3_NL            "Set P3"
#define UI_TEXT_CALCULATE_LEVELING_NL "Bereken Leveling"
#define UI_TEXT_LEVEL_NL             "Level delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_NL "Wacht Temp. %XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_NL "Wacht Retr.:%XU mm"
#define UI_TEXT_SD_REMOVED_NL       "SD-kaart verwijderd"
#define UI_TEXT_SD_INSERTED_NL      "SD-kaart geplaatst"
#define UI_TEXT_PRINTER_READY_NL    "Printer klaar."
#define UI_TEXT_PRINTTIME_DAYS_NL   " dagen "
#define UI_TEXT_PRINTTIME_HOURS_NL  ":"
#define UI_TEXT_PRINTTIME_MINUTES_NL ""
#define UI_TEXT_PRINT_TIME_NL     "Printing tijd"
#define UI_TEXT_PRINT_FILAMENT_NL "Filament gedrukt"
#define UI_TEXT_PRINTED_NL           "gedrukt"
#define UI_TEXT_POWER_NL            "ATX power aan/uit"
#define UI_TEXT_STRING_HM_DEADTIME_NL     "Dode tijd"
#define UI_TEXT_STRING_HM_SLOWBANG_NL     "SlowBang"
#define UI_TEXT_STOP_PRINT_NL "Stop Print"
#define UI_TEXT_Z_BABYSTEPPING_NL "Z Babystep.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_NL "Ruil filament"
#define UI_TEXT_WIZ_CH_FILAMENT1_NL "Ruil filament:"
#define UI_TEXT_WIZ_CH_FILAMENT2_NL "Draaien voor"
#define UI_TEXT_WIZ_CH_FILAMENT3_NL "filament op/omlaag"
#define UI_TEXT_CLICK_DONE_NL "Verder met klik"
#define UI_TEXT_AUTOLEVEL_ONOFF_NL  "Autolevel: %ll"
#define UI_TEXT_SERVOPOS_NL "Servo-pos.: %oS"
#define UI_TEXT_IGNORE_M106_NL      "Negeer M106 %Fi"
#define UI_TEXT_WIZ_REHEAT1_NL "Klik om extruder"
#define UI_TEXT_WIZ_REHEAT2_NL "verwarmen."
#define UI_TEXT_WIZ_WAITTEMP1_NL "Wachten op"
#define UI_TEXT_WIZ_WAITTEMP2_NL "doeltemperatuur..."
#define UI_TEXT_EXTRUDER_JAM_NL "Extruder jam"
#define UI_TEXT_STANDBY_NL "Standby"
#define UI_TEXT_BED_COATING_NL "Bed bedekking"
#define UI_TEXT_BED_COATING_SET1_NL "Bed bedekking set to"
#define UI_TEXT_BED_COATING_SET2_NL ""
#define UI_TEXT_NOCOATING_NL "Geen bedekking"
#define UI_TEXT_BUILDTAK_NL "BuildTak"
#define UI_TEXT_KAPTON_NL "Kapton"
#define UI_TEXT_BLUETAPE_NL "Blauw afplakband"
#define UI_TEXT_PETTAPE_NL "Groene PET Tape"
#define UI_TEXT_GLUESTICK_NL "Lijmstift"
#define UI_TEXT_CUSTOM_NL "Custom"
#define UI_TEXT_COATING_CUSTOM_NL "Custom:%BCmm"
#define UI_TEXT_LANGUAGE_NL "Taal"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_NL "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_NL "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_NL "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_NL "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_NL "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_NL "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_NL "Kopieen: %ed   Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_NL "Flow:\xfd %of%%%   Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_NL "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_NL "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_NL "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_NL cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_NL "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_NL "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_NL "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_NL "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_NL "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_NL "%Uf m"
#define UI_TEXT_METER_PRINTED_NL "%Uf m " UI_TEXT_PRINTED_NL
#define UI_TEXT_STATUS_NL "%os"
#define UI_TEXT_EMPTY_NL ""
#define UI_TEXT_TEMP_SET_NL cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_NL cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_NL " %BCmm"
#define UI_TEXT_EXTR3_TEMP_NL "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_NL "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_NL "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_NL "Extruder 4 Uit"
#define UI_TEXT_EXTR4_OFF_NL "Extruder 5 Uit"
#define UI_TEXT_EXTR5_OFF_NL "Extruder 6 Uit"
#define UI_TEXT_EXTR3_SELECT_NL "%X3 Select Extr. 4"
#define UI_TEXT_EXTR4_SELECT_NL "%X4 Select Extr. 5"
#define UI_TEXT_EXTR5_SELECT_NL "%X5 Select Extr. 6"
#define UI_TEXT_DITTO_0_NL "%D0 Geen Kopieen"
#define UI_TEXT_DITTO_1_NL "%D1 1 Kopie"
#define UI_TEXT_DITTO_2_NL "%D2 2 Kopieen"
#define UI_TEXT_DITTO_3_NL "%D3 3 Kopieen"
#define UI_TEXT_ZPROBE_HEIGHT_NL "z-probe hoogte:%zh"
#define UI_TEXT_OFFSETS_NL "Set print offsets"
#define UI_TEXT_X_OFFSET_NL "Set X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_NL "Set Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_NL "Set Z offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_NL "Laad Filament"
#define UI_TEXT_UNLOAD_FILAMENT_NL "Ontlaad Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_NL "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_NL "Reinigen van Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_NL "Reinig Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_NL "Dripbox"
#define UI_TEXT_EXTR_XOFF_NL "Xoffset:%Xx"
#define UI_TEXT_EXTR_YOFF_NL "Yoffset:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_NL "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_NL "Reinig Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_NL "Reinigen van Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_NL "Dripboxes"
#define UI_TEXT_EXTR_XOFF_NL "Xoffst:%Xx"
#define UI_TEXT_EXTR_YOFF_NL "Yoffst:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_NL "Laad Rechts \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_NL "Ontlaad Rechts \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_NL "Laad Links \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_NL "Ontlaad Links \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_NL "Laad Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_NL "Ontlaad F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_NL "Laad dist.: %fd"
#define UI_TEXT_BED_DOWN_NL "Bed Beneden"
#define UI_TEXT_WIFI_ONOFF_NL "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_NL "Opnieuw ?"
#define UI_TEXT_X_LENGTH_NL "X Lengte:%Lxmm"
#define UI_TEXT_Y_LENGTH_NL "Y Lengte:%Lymm"
#define UI_TEXT_Z_LENGTH_NL "Z Lengte:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_NL "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_NL "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_NL "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_NL "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_NL "Extruder%Ne heeft geen"
#define UI_TEXT_ALERT_NO_FILAMENTB_NL "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_NL "Druk Ok om fil."
#define UI_TEXT_ALERT_NO_FILAMENTD_NL "wisselen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_NL "Vind de 3 knoppen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_NL "op de onderzijde"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_NL "van het platform"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_NL "en draai 4x aan."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_NL "Ik ga de ext."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_NL "verplaatsen naar"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_NL "een aantal pos."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_NL "om aan te passen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_NL "In elke positie"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_NL " zullen 2 knoppen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_NL "samen worden"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_NL "aangepast."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_NL "Nozzle is op de"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_NL "juiste hoogte"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_NL "als er een blad"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_NL "glijden kan "
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_NL "tussen de nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_NL "en het bed. Neem"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_NL "een blad papier"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_NL "voort te gaan"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_NL "Pas de voorste"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_NL "knoppen aan tot"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_NL "het blad tussen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_NL "nozzle & bed kan"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_NL "Pas de achterste"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_NL "knoppen aan tot"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_NL "het blad tussen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_NL "nozzle & bed kan"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_NL "Pas achterste en"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_NL "rechter knop aan"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_NL "tot het blad er"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_NL "tussen glijdt."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_NL "Pas achterste en"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_NL "linker knop aan"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_NL "tot het blad er"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_NL "tussen glijdt."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_NL "Controleer dat"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_NL "het blad tussen"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_NL "nozzle en bed"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_NL "glijdt."
#define UI_TEXT_X_MIN_SETTING_NL "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_NL "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_NL "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_NL "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_NL "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_NL "System Versie"
#define UI_TEXT_COOLDOWN_MENU_NL "Afkoelen      %B3"
#define UI_TEXT_WAIT_OK_NL "Ok voor verder"
#define UI_TEXT_FAILED_NL "Gefaald"
#define UI_TEXT_PLEASE_WAIT_NL "Even geduld aub"
#define UI_TEXT_MANUAL_LEVEL_NL "Manueel richten"
#define UI_TEXT_WARNING_NL "Waarschuwing!"
#define UI_TEXT_EXTRUDER_COLD_NL "E%Ne is te koud!"
#define UI_TEXT_HEAT_EXTRUDER_NL "Ext. opwarmen?"
#define UI_TEXT_WRITE_DEBUG_NL "Schrijf Debug"
#define UI_TEXT_POWER_SAVE_MENU_NL "Powersave"
#define UI_TEXT_MAINTENANCE_NL "Onderhoud"
#define UI_TEXT_SETTINGS_NL "Instellingen"
#define UI_TEXT_LOAD_UNLOAD_NL "Laden/ontladen"
#define UI_TEXT_BED_NL "Bed"
#define UI_TEXT_BED_OFF_NL "Bed uitzetten %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_NL "Licht AutoUit "
#define UI_TEXT_LIGHT_OFF_AFTER2A_NL "Dis. Na:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_NL "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_NL "Echte X onbekend"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_NL "Echte Y onbekend"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_NL "Echte Z onbekend"
#define UI_TEXT_MOVING_NL "In beweging..."
#define UI_TEXT_PREPARING_NL "Voorbereiden..."
#define UI_TEXT_KEEP_LIGHT_ON_NL "Print met/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_NL "Laden \005"
#define UI_TEXT_UNLOADING_FILAMENT_NL "Ontladen \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_NL "Laden/Ontladen \005"
#define UI_TEXT_AUTOLEVEL_NL "Auto richten bed"
#define UI_TEXT_Z_PROBE_FAILED_NL "Z-probe gefaald"
#define UI_TEXT_SAVE_NL "Opslaan EEPROM?"
#define UI_TEXT_ZMIN_NL "zMin : %zm"
#define UI_TEXT_ZPROBING_NL "Onderzoeken..."
#define UI_TEXT_ZPOSITION_NL "Ga naar positie."
#define UI_TEXT_DO_YOU_NL "Wil je:"
#define UI_TEXT_CLEAN1_NL "Nozzle en bed"
#define UI_TEXT_CLEAN2_NL "eerst reinigen?"
#define UI_TEXT_DISPLAY_MODE_NL "Mode:%Md"
#define UI_TEXT_EASY_MODE_NL "Makkelijk"
#define UI_TEXT_ADVANCED_MODE_NL "Geevanveerd"
#define UI_TEXT_POWER_SAVE_NL "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_NL "Badge licht:%lb"
#define UI_TEXT_SOUND_ONOF_NL "Geluid:  %so"
#define UI_TEXT_YES_SELECTED_NL "  [Ja]    Nee"
#define UI_TEXT_NO_SELECTED_NL "   Ja    [Nee]"
#define UI_TEXT_PLEASE_CONFIRM_NL "Bevestig aub:"
#define UI_TEXT_CANCEL_ACTION_NL "Annuleer:"
#define UI_TEXT_CONTINUE_ACTION_NL "Verdergaan"
#define UI_TEXT_CANCELED_NL "Actie gestopt!"
#define UI_TEXT_CONFIRMED_NL "Actie bevestigd!"
#define UI_TEXT_CLEAN_NOZZLE_NL "Nozzle reinigen"
#define UI_TEXT_CLEANING_NOZZLE_NL "Reinigen Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_NL "Verwijden"
#define UI_TEXT_CLEAN_DRIPBOX_2_NL "En reinig"
#define UI_TEXT_PUSH_FILAMENT_NL "Duw Filament"
#define UI_TEXT_WAIT_FILAMENT_NL "Druk Ok vr start"
#define UI_TEXT_LOAD_FAILSAFE2_NL "Na het laden?"
#define UI_TEXT_LOAD_FAILSAFE_NL "Laad Fail-Safe"
#define UI_TEXT_HEATING_NL "Opwarmen..."
#define UI_TEXT_WAIT_FOR_OK_NL "Druk Ok vr einde"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_NL  "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_NL  "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_NL "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_NL "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_NL "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, deze zijn verschillend als je 1 of 2 extruders hebt, en of je in Makkelijke of Geevanceerde modus bent.
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_NL "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_NL "Z:%x2"
#define UI_TEXT_P1_L3_NL "Mul:%om Buf:%oB"

#define UI_TEXT_P1_L1_EASY_NL "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_NL "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_NL "Z:%x2"

#else
#define UI_TEXT_P1_L1_NL "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_NL "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_NL "Z:%x2"

#define UI_TEXT_P1_L1_EASY_NL "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_NL "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_NL "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_NL "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_NL "Z:%x2"
#define UI_TEXT_P1_L3_NL "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_NL "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_NL "Z:%x2"
#define UI_TEXT_P1_L3_EASY_NL "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_NL "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_NL "Z:%x2"
#define UI_TEXT_P1_L3_NL "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_NL "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_NL "Z:%x2"
#define UI_TEXT_P1_L3_EASY_NL "Buffer:%oB"
#endif
#endif
#define  UI_TEXT_MENU_SCANNER_NL    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_NL    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_NL        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_NL       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_NL           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_NL          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_NL    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_NL     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_NL      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_NL      "Zmin evaluation"





// *************** Brazilian portuguese translation ****************

#define UI_TEXT_ON_PT               "On"
#define UI_TEXT_OFF_PT              "Off"
#define UI_TEXT_NA_PT               "N/A" // Output for not available
#define UI_TEXT_YES_PT              "Sim"
#define UI_TEXT_NO_PT               "Não"
#define UI_TEXT_PRINT_POS_PT        "Imprimindo..."
#define UI_TEXT_PRINTING_PT         "Imprimindo"
#define UI_TEXT_IDLE_PT             "Ocioso"
#define UI_TEXT_NOSDCARD_PT         "Nenhum cartao SD"
#define UI_TEXT_ERROR_PT            "**** ERRO ****"
#define UI_TEXT_BACK_PT             "Voltar " cUP
#define UI_TEXT_QUICK_SETTINGS_PT   "Configuracoes Rapidas"
#define UI_TEXT_ERRORMSG_PT         "%oe"
#define UI_TEXT_CONFIGURATION_PT    "Configuracao"
#define UI_TEXT_POSITION_PT         "Posicao"
#define UI_TEXT_EXTRUDER_PT         "Extrusor"
#define UI_TEXT_SD_CARD_PT          "Cartao SD"
#define UI_TEXT_DEBUGGING_PT        "Depuracao"
#define UI_TEXT_HOME_DELTA_PT       "Home delta"
#define UI_TEXT_HOME_ALL_PT         "Home todos"
#define UI_TEXT_HOME_X_PT           "Home X"
#define UI_TEXT_HOME_Y_PT           "Home Y"
#define UI_TEXT_HOME_Z_PT           "Home Z"
#define UI_TEXT_PREHEAT_PLA_PT      "Pre-aquecer PLA %B1"
#define UI_TEXT_PREHEAT_ABS_PT      "Pre-aquecer ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_PT     "Luzes:%lo"
#define UI_TEXT_COOLDOWN_PT         "Resfriar"
#define UI_TEXT_SET_TO_ORIGIN_PT    "Definir como origem"
#define UI_TEXT_DISABLE_STEPPER_PT  "Desabilitar motor"
#define UI_TEXT_X_POSITION_PT       "Posicao X"
#define UI_TEXT_X_POS_FAST_PT       "Pos. Rapida X"
#define UI_TEXT_Y_POSITION_PT       "Posicao Y"
#define UI_TEXT_Y_POS_FAST_PT       "Pos. Rapida Y"
#define UI_TEXT_Z_POSITION_PT       "Posicao Z"
#define UI_TEXT_Z_POS_FAST_PT       "Pos. Rapida Z"
#define UI_TEXT_E_POSITION_PT       "Posicao Extrusor"
#define UI_TEXT_BED_TEMP_PT         "Tem.Cama:%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_PT       "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_PT       "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_PT       "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_PT       "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_PT        "Extr. Desligado%B4"
#else
#define UI_TEXT_EXTR0_OFF_PT        "Extr. 1 Desligado%B4"
#endif
#define UI_TEXT_EXTR1_OFF_PT        "Extr. 2 Desligado%B5"
#define UI_TEXT_EXTR2_OFF_PT        "Extr. 3 Desligado%B6"
#define UI_TEXT_EXTR0_SELECT_PT     "%X0 Sel. Extr. 1"
#define UI_TEXT_EXTR1_SELECT_PT     "%X1 Sel. Extr. 2"
#define UI_TEXT_EXTR2_SELECT_PT     "%X2 Sel. Extr. 3"
#define UI_TEXT_EXTR_ORIGIN_PT      "Definir Origem"
#define UI_TEXT_PRINT_X_PT          "Imprimir X:%ax"
#define UI_TEXT_PRINT_Y_PT          "Imprimir Y:%ay"
#define UI_TEXT_PRINT_Z_PT          "Imprimir Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_PT    "Imprimir:%az"
#define UI_TEXT_MOVE_X_PT           "Mover X:%aX"
#define UI_TEXT_MOVE_Y_PT           "Mover Y:%aY"
#define UI_TEXT_MOVE_Z_PT           "Mover Z:%aZ"
#define UI_TEXT_MOVE_Z_DELTA_PT     "Mover:%aZ"
#define UI_TEXT_JERK_PT             "Jerk:%aj"
#define UI_TEXT_ZJERK_PT            "Z-Jerk:%aJ"
#define UI_TEXT_ACCELERATION_PT     "Aceleracao"
#define UI_TEXT_STORE_TO_EEPROM_PT  "Armazenar na EEPROM"
#define UI_TEXT_LOAD_EEPROM_PT      "Carregar da EEPROM"
#define UI_TEXT_DBG_ECHO_PT         "Echo   :%do"
#define UI_TEXT_DBG_INFO_PT         "Info   :%di"
#define UI_TEXT_DBG_ERROR_PT        "Erros :%de"
#define UI_TEXT_DBG_DRYRUN_PT       "Dry run:%dd"
#define UI_TEXT_DBG_ENDSTOP_PT      "EndStop:%dp"
#define UI_TEXT_OPS_OFF_PT          "%O0 OPS Off"
#define UI_TEXT_OPS_CLASSIC_PT      "%O1 OPS Classic"
#define UI_TEXT_OPS_FAST_PT         "%O2 OPS Fast"
#define UI_TEXT_OPS_RETRACT_PT      "Retract :%Or"
#define UI_TEXT_OPS_BACKSLASH_PT    "Backsl. :%Ob"
#define UI_TEXT_OPS_MINDIST_PT      "Min.dist:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_PT   "Move after:%Oa"
#define UI_TEXT_ANTI_OOZE_PT        "Anti Ooze"
#define UI_TEXT_PRINT_FILE_PT       "Imprimir arquivo"
#define UI_TEXT_PAUSE_PRINT_PT      "Pausar Impressao"
#define UI_TEXT_CONTINUE_PRINT_PT   "Continuar Impressao"
#define UI_TEXT_UNMOUNT_CARD_PT     "Desmontar Cartao"
#define UI_TEXT_MOUNT_CARD_PT       "Montar Cartao"
#define UI_TEXT_DELETE_FILE_PT      "Deletar arquivo"
#define UI_TEXT_FEEDRATE_PT         "Feedrate"
#define UI_TEXT_FEED_MAX_X_PT       "Max X:%fx"
#define UI_TEXT_FEED_MAX_Y_PT       "Max Y:%fy"
#define UI_TEXT_FEED_MAX_Z_PT       "Max Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_PT "Max:%fz"
#define UI_TEXT_FEED_HOME_X_PT      "Home X:%fX"
#define UI_TEXT_FEED_HOME_Y_PT      "Home Y:%fY"
#define UI_TEXT_FEED_HOME_Z_PT      "Home Z:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_PT "Home:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_PT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_PT "Min endstop:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_PT "Max endstop:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_PT ""
#define UI_TEXT_ACTION_YPOSITION4A_PT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_PT "Min endstop:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_PT "Max endstop:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_PT ""
#define UI_TEXT_ACTION_ZPOSITION4A_PT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_PT "Min endstop:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_PT "Max endstop:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_PT ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_PT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_PT "Min endstop:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_PT "Max endstop:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_PT ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_PT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_PT "Min endstop:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_PT "Max endstop:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_PT ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_PT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_PT "Min endstop:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_PT "Max endstop:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_PT ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_PT "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_PT "1 click = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_PT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_PT "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_PT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_PT "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_PT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_PT "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_PT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_PT "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_PT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_PT "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_PT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_PT "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_PT          "Velocidade ventoinha"
#define UI_TEXT_ACTION_FANSPEED_PT   "Vel. vent.:%Fs%%%"
#define UI_TEXT_FAN_OFF_PT           "Desligar ventoinha"
#define UI_TEXT_FAN_25_PT            "Def. ventoinha 25%%%"
#define UI_TEXT_FAN_50_PT            "Def. ventoinha 50%%%"
#define UI_TEXT_FAN_75_PT            "Def. ventoinha 75%%%"
#define UI_TEXT_FAN_FULL_PT          "Def. ventoinha 100%%%"
#define UI_TEXT_STEPPER_INACTIVE_PT  "Motor Inativo"
#define UI_TEXT_STEPPER_INACTIVE2A_PT "Des. Depois: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_PT "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_PT    "Max. Inativo"
#define UI_TEXT_POWER_INACTIVE2A_PT   "Des. Depois: %ip"
#define UI_TEXT_POWER_INACTIVE2B_PT   "[min] 0=Off"
#define UI_TEXT_GENERAL_PT           "Geral"
#define UI_TEXT_BAUDRATE_PT          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_PT        "Passos/mm:%Se"
#define UI_TEXT_EXTR_START_FEED_PT   "Iniciar FR:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_PT     "Max FR:%XF"
#define UI_TEXT_EXTR_ACCEL_PT        "Accel:%XA"
#define UI_TEXT_EXTR_WATCH_PT        "Stab.Time:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_PT    "Advance lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_PT    "Advance quad:%Xa"
#define UI_TEXT_EXTR_MANAGER_PT      "Controle:%Xh"
#define UI_TEXT_EXTR_PGAIN_PT        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_PT     "Tempo morto:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_PT      "Controle PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_PT        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_PT        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_PT         "Drive Min:%Xm"
#define UI_TEXT_EXTR_DMAX_PT         "Drive Max:%XM"
#define UI_TEXT_EXTR_PMAX_PT         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_PT "BangBang"
#define UI_TEXT_STRING_HM_PID_PT     "PID"
#define UI_TEXT_STRING_ACTION_PT     "Acao:%la"
#define UI_TEXT_HEATING_EXTRUDER_PT  "Aquecendo Ext%Ne"
#define UI_TEXT_HEATING_BED_PT       "Aquecendo Cama"
#define UI_TEXT_KILLED_PT            "Killed"
#define UI_TEXT_STEPPER_DISABLED_PT  "Motor Desabilitado"
#define UI_TEXT_EEPROM_STOREDA_PT     "Configuracao"
#define UI_TEXT_EEPROM_STOREDB_PT     "armazenada na EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_PT     "Configuracao"
#define UI_TEXT_EEPROM_LOADEDB_PT     "carregada da EEPROM"
#define UI_TEXT_UPLOADING_PT         "Enviando..."
#define UI_TEXT_PAGE_BUFFER_PT       "Buffer:%oB"
#define UI_TEXT_PAGE_EXTRUDER_PT     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_PT     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_PT     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_PT     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_PT          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_PT    "Mult. Veloc.:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_PT     "Mult. Fluxo:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_PT  "Mostrar medicao"
#define UI_TEXT_RESET_MEASUREMENT_PT "Reset medicao"
#define UI_TEXT_SET_MEASURED_ORIGIN_PT "Set Z=0"
#define UI_TEXT_ZCALIB_PT             "Z calib."
#define UI_TEXT_SET_P1_PT            "Set P1"
#define UI_TEXT_SET_P2_PT            "Set P2"
#define UI_TEXT_SET_P3_PT            "Set P3"
#define UI_TEXT_CALCULATE_LEVELING_PT "Calcule nivelamento"
#define UI_TEXT_LEVEL_PT             "Nivel delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_PT  "Aguardar Temp.%XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_PT "Aguardar Unidades:%XUmm"
#define UI_TEXT_SD_REMOVED_PT       "Cartao SD removido"
#define UI_TEXT_SD_INSERTED_PT      "Cartao SD inserido"
#define UI_TEXT_PRINTER_READY_PT    "Impressora pronta."
#define UI_TEXT_PRINTTIME_DAYS_PT   " dias "
#define UI_TEXT_PRINTTIME_HOURS_PT  ":"
#define UI_TEXT_PRINTTIME_MINUTES_PT ""
#define UI_TEXT_PRINT_TIME_PT     "tempo de impressao"
#define UI_TEXT_PRINT_FILAMENT_PT "Filament impresso"
#define UI_TEXT_PRINTED_PT           "impresso"
#define UI_TEXT_POWER_PT            "ATX poder on/off"
#define UI_TEXT_STRING_HM_DEADTIME_PT     "Tempo morto"
#define UI_TEXT_STRING_HM_SLOWBANG_PT     "SlowBang"
#define UI_TEXT_STOP_PRINT_PT "Parar impressao"
#define UI_TEXT_Z_BABYSTEPPING_PT "Z babystep.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_PT "Alterar filamento"
#define UI_TEXT_WIZ_CH_FILAMENT1_PT "Alterar filamento"
#define UI_TEXT_WIZ_CH_FILAMENT2_PT "Gire para mover"
#define UI_TEXT_WIZ_CH_FILAMENT3_PT "filamento cima/baixo"
#define UI_TEXT_CLICK_DONE_PT "Clique quando feito"
#define UI_TEXT_AUTOLEVEL_ONOFF_PT  "Nivel auto: %ll"
#define UI_TEXT_SERVOPOS_PT "Pos. servo: %oS"
#define UI_TEXT_IGNORE_M106_PT      "Ignorar M106 %Fi"
#define UI_TEXT_WIZ_REHEAT1_PT "Clique para"
#define UI_TEXT_WIZ_REHEAT2_PT "aquecer extrusora."
#define UI_TEXT_WIZ_WAITTEMP1_PT "Aguardando a"
#define UI_TEXT_WIZ_WAITTEMP2_PT "temperatura alvo ..."
#define UI_TEXT_EXTRUDER_JAM_PT "Extrusora congest."
#define UI_TEXT_STANDBY_PT "Standby"
#define UI_TEXT_BED_COATING_PT "Revest. de leito"
#define UI_TEXT_BED_COATING_SET1_PT "Revest. de leito:"
#define UI_TEXT_BED_COATING_SET2_PT ""
#define UI_TEXT_NOCOATING_PT "Sem revestimento"
#define UI_TEXT_BUILDTAK_PT "BuildTak"
#define UI_TEXT_KAPTON_PT "Kapton"
#define UI_TEXT_BLUETAPE_PT "Fita crepe azul"
#define UI_TEXT_PETTAPE_PT "Fita verde PET"
#define UI_TEXT_GLUESTICK_PT "Cola bastao"
#define UI_TEXT_CUSTOM_PT "Personalizadas"
#define UI_TEXT_COATING_CUSTOM_PT "Person.:%oCmm"
#define UI_TEXT_LANGUAGE_PT "Idioma"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_PT "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_PT "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_PT "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_PT "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_PT "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_PT "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_PT "Copias: %ed    Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_PT "Fluxo:\xfd %of%%%  Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_PT "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_PT "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_PT "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_PT cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_PT "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_PT "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_PT "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_PT "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_PT "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_PT "%Uf m"
#define UI_TEXT_METER_PRINTED_PT "%Uf m " UI_TEXT_PRINTED_EN
#define UI_TEXT_STATUS_PT "%os"
#define UI_TEXT_EMPTY_PT ""
#define UI_TEXT_TEMP_SET_PT cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_PT cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_PT " %oCmm"
#define UI_TEXT_EXTR3_TEMP_PT "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_PT "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_PT "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_PT "Extr. 4 Desligado"
#define UI_TEXT_EXTR4_OFF_PT "Extr. 5 Desligado"
#define UI_TEXT_EXTR5_OFF_PT "Extr. 6 Desligado"
#define UI_TEXT_EXTR3_SELECT_PT "%X3 Sel. Extr. 4"
#define UI_TEXT_EXTR4_SELECT_PT "%X4 Sel. Extr. 5"
#define UI_TEXT_EXTR5_SELECT_PT "%X5 Sel. Extr. 6"
#define UI_TEXT_DITTO_0_PT "%D0 Nenhuma Copia"
#define UI_TEXT_DITTO_1_PT "%D1 1 Copia"
#define UI_TEXT_DITTO_2_PT "%D2 2 Copias"
#define UI_TEXT_DITTO_3_PT "%D3 3 Copias"
#define UI_TEXT_ZPROBE_HEIGHT_PT "Altura Z-Probe:%zh"
#define UI_TEXT_OFFSETS_PT "Set print offsets"
#define UI_TEXT_X_OFFSET_PT "Set X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_PT "Set Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_PT "Set Z offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_PT "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_PT "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_PT "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_PT "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_PT "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_PT "Dripbox"
#define UI_TEXT_EXTR_XOFF_PT "Xoffset:%Xx"
#define UI_TEXT_EXTR_YOFF_PT "Yoffset:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_PT "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_PT "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_PT "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_PT "Dripboxes"
#define UI_TEXT_EXTR_XOFF_PT "Xoffst:%Xx"
#define UI_TEXT_EXTR_YOFF_PT "Yoffst:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_PT "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_PT "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_PT "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_PT "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_PT "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_PT "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_PT "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_PT "Bed Down"
#define UI_TEXT_WIFI_ONOFF_PT "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_PT "Do it again ?"
#define UI_TEXT_X_LENGTH_PT "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_PT "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_PT "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_PT "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_PT "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_PT "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_PT "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_PT "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_PT "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_PT "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_PT "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_PT "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_PT "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_PT "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_PT "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_PT "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_PT "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_PT "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_PT "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_PT "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_PT "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_PT "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_PT "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_PT "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_PT "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_PT "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_PT "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_PT "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_PT "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_PT "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_PT "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_PT "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_PT "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_PT "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_PT "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_PT "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_PT "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_PT "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_PT "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_PT "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_PT "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_PT "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_PT "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_PT "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_PT "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_PT "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_PT "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_PT "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_PT "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_PT "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_PT "and plate."
#define UI_TEXT_X_MIN_SETTING_PT "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_PT "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_PT "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_PT "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_PT "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_PT "System Version"
#define UI_TEXT_COOLDOWN_MENU_PT "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_PT "Ok to continue"
#define UI_TEXT_FAILED_PT "Failed"
#define UI_TEXT_PLEASE_WAIT_PT "Please wait"
#define UI_TEXT_MANUAL_LEVEL_PT "Manual leveling"
#define UI_TEXT_WARNING_PT "Warning!"
#define UI_TEXT_EXTRUDER_COLD_PT "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_PT "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_PT "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_PT "Powersave"
#define UI_TEXT_MAINTENANCE_PT "Maintenance"
#define UI_TEXT_SETTINGS_PT "Settings"
#define UI_TEXT_LOAD_UNLOAD_PT "Load/Unload"
#define UI_TEXT_BED_PT "Bed"
#define UI_TEXT_BED_OFF_PT "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_PT "Light AutoOff "
#define UI_TEXT_LIGHT_OFF_AFTER2A_PT "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_PT "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_PT "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_PT "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_PT "Real Z unknown !"
#define UI_TEXT_MOVING_PT "Moving..."
#define UI_TEXT_PREPARING_PT "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_PT "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_PT "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_PT "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_PT "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_PT "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_PT "Z-probe failed"
#define UI_TEXT_SAVE_PT "Save to EEPROM ?"
#define UI_TEXT_ZMIN_PT "zMin : %zm"
#define UI_TEXT_ZPROBING_PT "Probing..."
#define UI_TEXT_ZPOSITION_PT "Go position..."
#define UI_TEXT_DO_YOU_PT "Do you want to:"
#define UI_TEXT_CLEAN1_PT "Clean nozzle and"
#define UI_TEXT_CLEAN2_PT "plates first ?"
#define UI_TEXT_DISPLAY_MODE_PT "Mode:%Md"
#define UI_TEXT_EASY_MODE_PT "Easy"
#define UI_TEXT_ADVANCED_MODE_PT "Advanced"
#define UI_TEXT_POWER_SAVE_PT "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_PT "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_PT "Sound:  %so"
#define UI_TEXT_YES_SELECTED_PT "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_PT "   Yes    [No]"
#define UI_TEXT_PLEASE_CONFIRM_PT "Please confirm"
#define UI_TEXT_CANCEL_ACTION_PT "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_PT "Continue"
#define UI_TEXT_CANCELED_PT "Action canceled!"
#define UI_TEXT_CONFIRMED_PT "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_PT "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_PT "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_PT "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_PT "And Clean"
#define UI_TEXT_PUSH_FILAMENT_PT "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_PT "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_PT "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_PT "Load Fail-Safe"
#define UI_TEXT_HEATING_PT "Heating..."
#define UI_TEXT_WAIT_FOR_OK_PT "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_PT  "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_PT  "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_PT "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_PT "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_PT "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_PT "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_PT "Z:%x2"
#define UI_TEXT_P1_L3_PT "Mul:%om Buf:%oB"

#define UI_TEXT_P1_L1_EASY_PT "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_PT "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_PT "Z:%x2"

#else
#define UI_TEXT_P1_L1_PT "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_PT "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_PT "Z:%x2"

#define UI_TEXT_P1_L1_EASY_PT "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_PT "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_PT "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_PT "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_PT "Z:%x2"
#define UI_TEXT_P1_L3_PT "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_PT "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_PT "Z:%x2"
#define UI_TEXT_P1_L3_EASY_PT "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_PT "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_PT "Z:%x2"
#define UI_TEXT_P1_L3_PT "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_PT "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_PT "Z:%x2"
#define UI_TEXT_P1_L3_EASY_PT "Buffer:%oB"
#endif
#endif
#define UI_TEXT_DBG_ENDSTOP_PT      "EndStop:%dp"
#define UI_TEXT_MENU_SCANNER_PT    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_PT    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_PT        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_PT       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_PT           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_PT          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_PT    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_PT     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_PT      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_PT      "Zmin evaluation"



// *************** Italian translation ****************

#define UI_TEXT_ON_IT               "On"
#define UI_TEXT_OFF_IT              "Off"
#define UI_TEXT_NA_IT               "N/A" // Output for not available
#define UI_TEXT_YES_IT              "Si"
#define UI_TEXT_NO_IT               "No"
#define UI_TEXT_PRINT_POS_IT        "Stampa..."
#define UI_TEXT_PRINTING_IT         "Stampa"
#define UI_TEXT_IDLE_IT             "Pausa"
#define UI_TEXT_NOSDCARD_IT         "No Scheda SD"
#define UI_TEXT_ERROR_IT            "**** ERRORE ****"
#define UI_TEXT_BACK_IT             "Indietro " cUP
#define UI_TEXT_QUICK_SETTINGS_IT   "Impostazioni veloci"
#define UI_TEXT_ERRORMSG_IT         "%oe"
#define UI_TEXT_CONFIGURATION_IT    "Configurazione"
#define UI_TEXT_POSITION_IT         "Posizione"
#define UI_TEXT_EXTRUDER_IT         "Estrusore"
#define UI_TEXT_SD_CARD_IT          "Scheda SD"
#define UI_TEXT_DEBUGGING_IT        "Sviluppo"
#define UI_TEXT_HOME_DELTA_IT       "Origine Delta"
#define UI_TEXT_HOME_ALL_IT         "Origine Tutti"
#define UI_TEXT_HOME_X_IT           "Origine X"
#define UI_TEXT_HOME_Y_IT           "Origine Y"
#define UI_TEXT_HOME_Z_IT           "Origine Z"
#define UI_TEXT_PREHEAT_PLA_IT      "Presicaldamento PLA %B1"
#define UI_TEXT_PREHEAT_ABS_IT      "Presicaldamento ABS%B2"
#define UI_TEXT_LIGHTS_ONOFF_IT     "Luci:%lo"
#define UI_TEXT_COOLDOWN_IT         "Raffreddamento"
#define UI_TEXT_SET_TO_ORIGIN_IT    "Imposta come Origine"
#define UI_TEXT_DISABLE_STEPPER_IT  "Disabilita Stepper"
#define UI_TEXT_X_POSITION_IT       "Posizione X"
#define UI_TEXT_X_POS_FAST_IT       "Pos. X Veloce"
#define UI_TEXT_Y_POSITION_IT       "Posizione Y"
#define UI_TEXT_Y_POS_FAST_IT       "Pos. Y Veloce"
#define UI_TEXT_Z_POSITION_IT       "Posizione Z"
#define UI_TEXT_Z_POS_FAST_IT       "Pos. Z Veloce"
#define UI_TEXT_E_POSITION_IT       "Posizione Estrusore"
#define UI_TEXT_BED_TEMP_IT         "Temp.Piatto:%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_IT         "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_IT         "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_IT         "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_IT         "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_IT          "Est. Spento  %B4"
#else
#define UI_TEXT_EXTR0_OFF_IT          "Est. 1 Spento %B4"
#endif
#define UI_TEXT_EXTR1_OFF_IT          "Est. 2 Spento%B5"
#define UI_TEXT_EXTR2_OFF_IT          "Est. 3 Spento%B6"
#define UI_TEXT_EXTR0_SELECT_IT     "%X0 Seleziona Estr. 1"
#define UI_TEXT_EXTR1_SELECT_IT     "%X1 Seleziona Estr. 2"
#define UI_TEXT_EXTR2_SELECT_IT     "%X2 Seleziona Estr. 3"
#define UI_TEXT_EXTR_ORIGIN_IT      "Imposta Origine"
#define UI_TEXT_PRINT_X_IT          "Stampa X:%ax"
#define UI_TEXT_PRINT_Y_IT          "Stampa Y:%ay"
#define UI_TEXT_PRINT_Z_IT          "Stampa Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_IT    "Stampa:%az"
#define UI_TEXT_MOVE_X_IT           "Movimento X:%aX"
#define UI_TEXT_MOVE_Y_IT           "Movimento Y:%aY"
#define UI_TEXT_MOVE_Z_IT           "Movimento Z:%aZ"
#define UI_TEXT_MOVE_Z_DELTA_IT     "Movimento:%aZ"
#define UI_TEXT_JERK_IT             "Scatto:%aj"
#define UI_TEXT_ZJERK_IT            "Scatto-Z:%aJ"
#define UI_TEXT_ACCELERATION_IT     "Accelerazione"
#define UI_TEXT_STORE_TO_EEPROM_IT  "Salva in EEPROM"
#define UI_TEXT_LOAD_EEPROM_IT      "Carica da EEPROM"
#define UI_TEXT_DBG_ECHO_IT         "Eco   :%do"
#define UI_TEXT_DBG_INFO_IT         "Info   :%di"
#define UI_TEXT_DBG_ERROR_IT        "Errori :%de"
#define UI_TEXT_DBG_DRYRUN_IT       "Simulazione:%dd"
#define UI_TEXT_DBG_ENDSTOP_IT      "EndStop:%dp"
#define UI_TEXT_OPS_OFF_IT          "%O0 OPS Spento"
#define UI_TEXT_OPS_CLASSIC_IT      "%O1 OPS Classico"
#define UI_TEXT_OPS_FAST_IT         "%O2 OPS Veloce"
#define UI_TEXT_OPS_RETRACT_IT      "Ritiro :%Or"
#define UI_TEXT_OPS_BACKSLASH_IT    "Gioco barra:%Ob"
#define UI_TEXT_OPS_MINDIST_IT      "Distanza Min.:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_IT   "Movimento dopo:%Oa"
#define UI_TEXT_ANTI_OOZE_IT        "Anti goccia"
#define UI_TEXT_PRINT_FILE_IT       "Stampa file"
#define UI_TEXT_PAUSE_PRINT_IT      "Pausa Stampa"
#define UI_TEXT_CONTINUE_PRINT_IT   "Continua Stampa"
#define UI_TEXT_UNMOUNT_CARD_IT     "Scarica Scheda"
#define UI_TEXT_MOUNT_CARD_IT       "Carica Scheda"
#define UI_TEXT_DELETE_FILE_IT      "Cancella file"
#define UI_TEXT_FEEDRATE_IT         "Velocita'"
#define UI_TEXT_FEED_MAX_X_IT       "Massimo X:%fx"
#define UI_TEXT_FEED_MAX_Y_IT       "Massimo Y:%fy"
#define UI_TEXT_FEED_MAX_Z_IT       "Massimo Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_IT "Massimo:%fz"
#define UI_TEXT_FEED_HOME_X_IT      "Origine X:%fX"
#define UI_TEXT_FEED_HOME_Y_IT      "Origine Y:%fY"
#define UI_TEXT_FEED_HOME_Z_IT      "Origine Z:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_IT "Origine:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_IT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_IT "Min finec.:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_IT "Max finec.:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_IT ""
#define UI_TEXT_ACTION_YPOSITION4A_IT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_IT "Min finec.:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_IT "Max finec.:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_IT ""
#define UI_TEXT_ACTION_ZPOSITION4A_IT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_IT "Min finec.:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_IT "Max finec.:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_IT ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_IT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_IT "Min finec.:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_IT "Max finec.:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_IT ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_IT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_IT "Min finec.:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_IT "Max finec.:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_IT ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_IT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_IT "Min finec.:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_IT "Max finec.:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_IT ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_IT "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_IT "1 scatto = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_IT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_IT "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_IT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_IT "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_IT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_IT "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_IT "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_IT "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_IT "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_IT "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_IT "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_IT "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_IT          "Velocita Ventola"
#define UI_TEXT_ACTION_FANSPEED_IT   "Vel. Ventola:%Fs%%%"
#define UI_TEXT_FAN_OFF_IT           "Spegnimento Ventola"
#define UI_TEXT_FAN_25_IT            "Ventola al 25%%%"
#define UI_TEXT_FAN_50_IT            "Ventola al%%%"
#define UI_TEXT_FAN_75_IT            "Ventola al%%%"
#define UI_TEXT_FAN_FULL_IT          "Ventola al massimo"
#define UI_TEXT_STEPPER_INACTIVE_IT  "Stepper Inattivi"
#define UI_TEXT_STEPPER_INACTIVE2A_IT "Dis. dopo: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_IT "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_IT    "Max. Inattivita'"
#define UI_TEXT_POWER_INACTIVE2A_IT   "Dis. After: %ip"
#define UI_TEXT_POWER_INACTIVE2B_IT   "[min] 0=Off"
#define UI_TEXT_GENERAL_IT           "Generale"
#define UI_TEXT_BAUDRATE_IT          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_IT        "Passi/mm:%Se"
#define UI_TEXT_EXTR_START_FEED_IT   "Velocita' Avvio:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_IT     "Velocita' Max:%XF"
#define UI_TEXT_EXTR_ACCEL_IT        "Accel:%XA"
#define UI_TEXT_EXTR_WATCH_IT        "Tempo Stab.:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_IT    "Avanzamento lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_IT    "Avanzamento espon:%Xa"
#define UI_TEXT_EXTR_MANAGER_IT      "Controllo:%Xh"
#define UI_TEXT_EXTR_PGAIN_IT        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_IT     "Tempo morto:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_IT      "Controllo PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_IT        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_IT        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_IT         "Drive Min:%Xm"
#define UI_TEXT_EXTR_DMAX_IT         "Drive Max:%XM"
#define UI_TEXT_EXTR_PMAX_IT         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_IT "BangBang"
#define UI_TEXT_STRING_HM_PID_IT     "PID"
#define UI_TEXT_STRING_ACTION_IT     "Azione:%la"
#define UI_TEXT_HEATING_EXTRUDER_IT  "Riscald. Est%Ne"
#define UI_TEXT_HEATING_BED_IT       "Riscald. Piatto"
#define UI_TEXT_KILLED_IT            "Abortito"
#define UI_TEXT_STEPPER_DISABLED_IT  "Stepper Disabilitato"
#define UI_TEXT_EEPROM_STOREDA_IT     "Configurazione"
#define UI_TEXT_EEPROM_STOREDB_IT     "Salvata in EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_IT     "Configurazione"
#define UI_TEXT_EEPROM_LOADEDB_IT     "Caricata da EEPROM"
#define UI_TEXT_UPLOADING_IT         "Caricamento..."
#define UI_TEXT_PAGE_BUFFER_IT       "Tampone:%oB"
#define UI_TEXT_PAGE_EXTRUDER_IT     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_IT     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_IT     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_IT     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_IT          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_IT    "Molt. Velocita':%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_IT     "Molt. Flusso:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_IT  "Mostra di misura"
#define UI_TEXT_RESET_MEASUREMENT_IT "Ripristino di misura"
#define UI_TEXT_SET_MEASURED_ORIGIN_IT "Set Z=0"
#define UI_TEXT_ZCALIB_IT             "Z Calib."
#define UI_TEXT_SET_P1_IT            "Impostato P1"
#define UI_TEXT_SET_P2_IT            "Impostato P2"
#define UI_TEXT_SET_P3_IT            "Impostato P3"
#define UI_TEXT_CALCULATE_LEVELING_IT "Calcol. livellamento"
#define UI_TEXT_LEVEL_IT             "Livello delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_IT  "Attesa Temp.%XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_IT "Attesa Unita':%XUmm"
#define UI_TEXT_SD_REMOVED_IT       "SD Card rimosso"
#define UI_TEXT_SD_INSERTED_IT      "SD Card inserita"
#define UI_TEXT_PRINTER_READY_IT    "Stampante pronta."
#define UI_TEXT_PRINTTIME_DAYS_IT   " giorni "
#define UI_TEXT_PRINTTIME_HOURS_IT  ":"
#define UI_TEXT_PRINTTIME_MINUTES_IT ""
#define UI_TEXT_PRINT_TIME_IT     "Tempo di stampa"
#define UI_TEXT_PRINT_FILAMENT_IT "Filament stampata"
#define UI_TEXT_PRINTED_IT           "stampato"
#define UI_TEXT_POWER_IT            "ATX on/off"
#define UI_TEXT_STRING_HM_DEADTIME_IT     "Tempo morto"
#define UI_TEXT_STRING_HM_SLOWBANG_IT     "SlowBang"
#define UI_TEXT_STOP_PRINT_IT "Arresto Stampa"
#define UI_TEXT_Z_BABYSTEPPING_IT "Z Babystep.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_IT "Cambia filamento"
#define UI_TEXT_WIZ_CH_FILAMENT1_IT "Cambia filamento"
#define UI_TEXT_WIZ_CH_FILAMENT2_IT "Ruotare per spostare"
#define UI_TEXT_WIZ_CH_FILAMENT3_IT "filamento su/giu"
#define UI_TEXT_CLICK_DONE_IT "Clicca quando fatto"
#define UI_TEXT_AUTOLEVEL_ONOFF_IT  "Autoliv.: %ll"
#define UI_TEXT_SERVOPOS_IT "Pos. servo: %oS"
#define UI_TEXT_IGNORE_M106_IT      "Ignora M106 cmd %Fi"
#define UI_TEXT_WIZ_REHEAT1_IT "Clicca per"
#define UI_TEXT_WIZ_REHEAT2_IT "riscaldare estrusori"
#define UI_TEXT_WIZ_WAITTEMP1_IT "Attendere che temp."
#define UI_TEXT_WIZ_WAITTEMP2_IT "di destinazione ..."
#define UI_TEXT_EXTRUDER_JAM_IT "Stoccaggio estrusore"
#define UI_TEXT_STANDBY_IT "Stand-by"
#define UI_TEXT_BED_COATING_IT "Rivestimento letto"
#define UI_TEXT_BED_COATING_SET1_IT "Rivestimento letto:"
#define UI_TEXT_BED_COATING_SET2_IT ""
#define UI_TEXT_NOCOATING_IT "Non rivestito"
#define UI_TEXT_BUILDTAK_IT "BuildTak"
#define UI_TEXT_KAPTON_IT "Kapton"
#define UI_TEXT_BLUETAPE_IT "Blu nastro adesivo"
#define UI_TEXT_PETTAPE_IT "Verde PET nastro"
#define UI_TEXT_GLUESTICK_IT "Colla stick"
#define UI_TEXT_CUSTOM_IT "Usanza"
#define UI_TEXT_COATING_CUSTOM_IT "Usanza:%oCmm"
#define UI_TEXT_LANGUAGE_IT "Lingua"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_IT "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_IT "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_IT "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_IT "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_IT "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_IT "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_IT "Copie: %ed     Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_IT "Flusso:\xfd %of%%% Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_IT "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_IT "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_IT "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_IT cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_IT "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_IT "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_IT "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_IT "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_IT "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_IT "%Uf m"
#define UI_TEXT_METER_PRINTED_IT "%Uf m " UI_TEXT_PRINTED_IT
#define UI_TEXT_STATUS_IT "%os"
#define UI_TEXT_EMPTY_IT ""
#define UI_TEXT_TEMP_SET_IT cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_IT cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_IT " %oCmm"
#define UI_TEXT_EXTR3_TEMP_IT "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_IT "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_IT "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_IT "Estrusore 4 Spento"
#define UI_TEXT_EXTR4_OFF_IT "Estrusore 5 Spento"
#define UI_TEXT_EXTR5_OFF_IT "Estrusore 6 Spento"
#define UI_TEXT_EXTR3_SELECT_IT "%X3 Seleziona Estr. 4"
#define UI_TEXT_EXTR4_SELECT_IT "%X4 Seleziona Estr. 5"
#define UI_TEXT_EXTR5_SELECT_IT "%X5 Seleziona Estr. 6"
#define UI_TEXT_DITTO_0_IT "%D0 Nessuna Copia"
#define UI_TEXT_DITTO_1_IT "%D1 1 Copia"
#define UI_TEXT_DITTO_2_IT "%D2 2 Copie"
#define UI_TEXT_DITTO_3_IT "%D3 3 Copie"
#define UI_TEXT_ZPROBE_HEIGHT_IT "Altezza Z-Probe:%zh"
#define UI_TEXT_OFFSETS_IT "Set print offsets"
#define UI_TEXT_X_OFFSET_IT "Set X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_IT "Set Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_IT "Set Z offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_IT   "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_IT   "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_IT   "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_IT   "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_IT   "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_IT   "Dripbox"
#define UI_TEXT_EXTR_XOFF_IT   "Xoffset:%Xx"
#define UI_TEXT_EXTR_YOFF_IT   "Yoffset:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_IT   "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_IT   "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_IT   "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_IT   "Dripboxes"
#define UI_TEXT_EXTR_XOFF_IT   "Xoffst:%Xx"
#define UI_TEXT_EXTR_YOFF_IT   "Yoffst:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_IT   "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_IT   "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_IT   "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_IT   "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_IT   "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_IT   "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_IT   "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_IT   "Bed Down"
#define UI_TEXT_WIFI_ONOFF_IT   "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_IT   "Do it again ?"
#define UI_TEXT_X_LENGTH_IT   "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_IT   "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_IT   "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_IT   "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_IT   "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_IT   "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_IT   "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_IT   "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_IT   "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_IT   "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_IT   "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_IT   "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_IT   "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_IT   "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_IT   "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_IT   "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_IT   "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_IT   "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_IT   "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_IT   "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_IT   "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_IT   "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_IT   "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_IT   "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_IT   "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_IT   "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_IT   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_IT   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_IT   "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_IT   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_IT   "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_IT   "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_IT   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_IT   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_IT   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_IT   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_IT   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_IT   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_IT   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_IT   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_IT   "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_IT   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_IT   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_IT   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_IT   "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_IT   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_IT   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_IT   "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_IT   "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_IT   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_IT   "and plate."
#define UI_TEXT_X_MIN_SETTING_IT   "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_IT   "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_IT   "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_IT   "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_IT   "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_IT   "System Version"
#define UI_TEXT_COOLDOWN_MENU_IT   "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_IT   "Ok to continue"
#define UI_TEXT_FAILED_IT   "Failed"
#define UI_TEXT_PLEASE_WAIT_IT   "Please wait"
#define UI_TEXT_MANUAL_LEVEL_IT   "Manual leveling"
#define UI_TEXT_WARNING_IT   "Warning!"
#define UI_TEXT_EXTRUDER_COLD_IT   "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_IT   "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_IT   "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_IT   "Powersave"
#define UI_TEXT_MAINTENANCE_IT   "Maintenance"
#define UI_TEXT_SETTINGS_IT   "Settings"
#define UI_TEXT_LOAD_UNLOAD_IT   "Load/Unload"
#define UI_TEXT_BED_IT   "Bed"
#define UI_TEXT_BED_OFF_IT   "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_IT   "Light AutoOff "
#define UI_TEXT_LIGHT_OFF_AFTER2A_IT   "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_IT   "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_IT   "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_IT   "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_IT   "Real Z unknown !"
#define UI_TEXT_MOVING_IT   "Moving..."
#define UI_TEXT_PREPARING_IT   "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_IT   "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_IT   "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_IT   "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_IT   "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_IT   "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_IT   "Z-probe failed"
#define UI_TEXT_SAVE_IT   "Save to EEPROM ?"
#define UI_TEXT_ZMIN_IT   "zMin : %zm"
#define UI_TEXT_ZPROBING_IT   "Probing..."
#define UI_TEXT_ZPOSITION_IT   "Go position..."
#define UI_TEXT_DO_YOU_IT   "Do you want to:"
#define UI_TEXT_CLEAN1_IT   "Clean nozzle and"
#define UI_TEXT_CLEAN2_IT   "plates first ?"
#define UI_TEXT_DISPLAY_MODE_IT   "Mode:%Md"
#define UI_TEXT_EASY_MODE_IT   "Easy"
#define UI_TEXT_ADVANCED_MODE_IT   "Advanced"
#define UI_TEXT_POWER_SAVE_IT   "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_IT   "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_IT   "Sound:  %so"
#define UI_TEXT_YES_SELECTED_IT   "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_IT   "   Yes    [No]"
#define UI_TEXT_PLEASE_CONFIRM_IT   "Please confirm"
#define UI_TEXT_CANCEL_ACTION_IT   "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_IT   "Continue"
#define UI_TEXT_CANCELED_IT   "Action canceled!"
#define UI_TEXT_CONFIRMED_IT   "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_IT   "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_IT   "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_IT   "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_IT   "And Clean"
#define UI_TEXT_PUSH_FILAMENT_IT   "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_IT   "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_IT   "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_IT   "Load Fail-Safe"
#define UI_TEXT_HEATING_IT   "Heating..."
#define UI_TEXT_WAIT_FOR_OK_IT   "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_IT    "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_IT    "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_IT   "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_IT   "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_IT   "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_IT "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_IT "Z:%x2"
#define UI_TEXT_P1_L3_IT "Mul:%om Buf:%oB"

#define UI_TEXT_P1_L1_EASY_IT "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_IT "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_IT "Z:%x2"

#else
#define UI_TEXT_P1_L1_IT "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_IT "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_IT "Z:%x2"

#define UI_TEXT_P1_L1_EASY_IT "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_IT "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_IT "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_IT "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_IT "Z:%x2"
#define UI_TEXT_P1_L3_IT "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_IT "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_IT "Z:%x2"
#define UI_TEXT_P1_L3_EASY_IT "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_IT "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_IT "Z:%x2"
#define UI_TEXT_P1_L3_IT "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_IT "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_IT "Z:%x2"
#define UI_TEXT_P1_L3_EASY_IT "Buffer:%oB"
#endif
#endif
#define UI_TEXT_MENU_SCANNER_IT    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_IT    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_IT        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_IT       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_IT           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_IT          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_IT    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_IT     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_IT      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_IT      "Zmin evaluation"



// Spanish translation

#define UI_TEXT_ON_ES               "On"
#define UI_TEXT_OFF_ES              "Off"
#define UI_TEXT_NA_ES               "N/A" // Output for not available
#define UI_TEXT_YES_ES              "Si"
#define UI_TEXT_NO_ES               "No"
#define UI_TEXT_PRINT_POS_ES        "Imprimiendo..."
#define UI_TEXT_PRINTING_ES         "Imprimiendo"
#define UI_TEXT_IDLE_ES             "Idle"
#define UI_TEXT_NOSDCARD_ES         "Sin tarjeta SD"
#define UI_TEXT_ERROR_ES            "**** ERROR ****"
#define UI_TEXT_BACK_ES             "Atras " cUP
#define UI_TEXT_QUICK_SETTINGS_ES   "Configuracion Rapida"
#define UI_TEXT_ERRORMSG_ES         "%oe"
#define UI_TEXT_CONFIGURATION_ES    "Configuracion"
#define UI_TEXT_POSITION_ES         "Posicion"
#define UI_TEXT_EXTRUDER_ES         "Extrusor"
#define UI_TEXT_SD_CARD_ES          "Tarjeta SD"
#define UI_TEXT_DEBUGGING_ES        "Debugging"
#define UI_TEXT_HOME_DELTA_ES       "Delta Home"
#define UI_TEXT_HOME_ALL_ES         "Todo Home"
#define UI_TEXT_HOME_X_ES           "X Home"
#define UI_TEXT_HOME_Y_ES           "Y Home"
#define UI_TEXT_HOME_Z_ES           "Z Home"
#define UI_TEXT_PREHEAT_PLA_ES      "Precalentar PLA %B1"
#define UI_TEXT_PREHEAT_ABS_ES      "Precalentar ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_ES     "Luces:%lo"
#define UI_TEXT_COOLDOWN_ES         "Enfriar"
#define UI_TEXT_SET_TO_ORIGIN_ES    "Fija a origen"
#define UI_TEXT_DISABLE_STEPPER_ES  "Desactiva motor"
#define UI_TEXT_X_POSITION_ES       "Posicion X"
#define UI_TEXT_X_POS_FAST_ES       "Pos. Rapida X"
#define UI_TEXT_Y_POSITION_ES       "Posicion Y"
#define UI_TEXT_Y_POS_FAST_ES       "Pos. Rapida Y"
#define UI_TEXT_Z_POSITION_ES       "Posicion Z"
#define UI_TEXT_Z_POS_FAST_ES       "Pos. Rapida Z"
#define UI_TEXT_E_POSITION_ES       "Extr. Posicion"
#define UI_TEXT_BED_TEMP_ES         "Temp.Cama:%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_ES         "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_ES         "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_ES         "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_ES         "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_ES          "Extrusor Off  %B4"
#else
#define UI_TEXT_EXTR0_OFF_ES          "Extrusor 1 Off %B4"
#endif
#define UI_TEXT_EXTR1_OFF_ES          "Extrusor 2 Off%B5"
#define UI_TEXT_EXTR2_OFF_ES          "Extrusor 3 Off %B6"
#define UI_TEXT_EXTR0_SELECT_ES     "%X0 Select Extr. 1"
#define UI_TEXT_EXTR1_SELECT_ES     "%X1 Select Extr. 2"
#define UI_TEXT_EXTR2_SELECT_ES     "%X2 Select Extr. 3"
#define UI_TEXT_EXTR_ORIGIN_ES      "Fija Originen"
#define UI_TEXT_PRINT_X_ES          "Print X:%ax"
#define UI_TEXT_PRINT_Y_ES          "Print Y:%ay"
#define UI_TEXT_PRINT_Z_ES          "Print Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_ES    "Print:%az"
#define UI_TEXT_MOVE_X_ES           "Mueve X:%aX"
#define UI_TEXT_MOVE_Y_ES           "Mueve Y:%aY"
#define UI_TEXT_MOVE_Z_ES           "Mueve Z:%aZ"
#define UI_TEXT_MOVE_Z_DELTA_ES     "Mueve:%aZ"
#define UI_TEXT_JERK_ES             "Jerk:%aj"
#define UI_TEXT_ZJERK_ES            "Z-Jerk:%aJ"
#define UI_TEXT_ACCELERATION_ES     "Aceleracion"
#define UI_TEXT_STORE_TO_EEPROM_ES  "Almacena en EEPROM"
#define UI_TEXT_LOAD_EEPROM_ES      "Carga de EEPROM"
#define UI_TEXT_DBG_ECHO_ES         "Echo   :%do"
#define UI_TEXT_DBG_INFO_ES         "Info   :%di"
#define UI_TEXT_DBG_ERROR_ES        "Errors :%de"
#define UI_TEXT_DBG_DRYRUN_ES       "Ejecucion vacio:%dd"
#define UI_TEXT_DBG_ENDSTOP_ES      "EndStop:%dp"
#define UI_TEXT_OPS_OFF_ES          "%O0 OPS Off"
#define UI_TEXT_OPS_CLASSIC_ES      "%O1 OPS Classica"
#define UI_TEXT_OPS_FAST_ES         "%O2 OPS Rapida"
#define UI_TEXT_OPS_RETRACT_ES      "Retraccion :%Or"
#define UI_TEXT_OPS_BACKSLASH_ES    "Backsl. :%Ob"
#define UI_TEXT_OPS_MINDIST_ES      "Min.dist:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_ES   "Move after:%Oa"
#define UI_TEXT_ANTI_OOZE_ES        "Anti Ooze"
#define UI_TEXT_PRINT_FILE_ES       "Imprimiendo fichero"
#define UI_TEXT_PAUSE_PRINT_ES      "Pausando impresion"
#define UI_TEXT_CONTINUE_PRINT_ES   "Continuando impresion"
#define UI_TEXT_UNMOUNT_CARD_ES     "Desmontando Tarjeta"
#define UI_TEXT_MOUNT_CARD_ES       "Montando Card"
#define UI_TEXT_DELETE_FILE_ES      "Borrando fichero"
#define UI_TEXT_FEEDRATE_ES         "Feedrate"
#define UI_TEXT_FEED_MAX_X_ES       "X Max:%fx"
#define UI_TEXT_FEED_MAX_Y_ES       "Y Max:%fy"
#define UI_TEXT_FEED_MAX_Z_ES       "Z Max:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_ES "Max:%fz"
#define UI_TEXT_FEED_HOME_X_ES      "X Home:%fX"
#define UI_TEXT_FEED_HOME_Y_ES      "Y Home:%fY"
#define UI_TEXT_FEED_HOME_Z_ES      "Z Home:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_ES "Home:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_ES "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_ES "Fin Carrera Min:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_ES "Max:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_ES ""
#define UI_TEXT_ACTION_YPOSITION4A_ES "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_ES "Fin Carrera Min:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_ES "Max:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_ES ""
#define UI_TEXT_ACTION_ZPOSITION4A_ES "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_ES "Fin Carrera Min:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_ES "Max:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_ES ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_ES "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_ES "Fin Carrera Min:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_ES "Max:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_ES ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_ES "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_ES "Fin Carrera Min:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_ES "Max:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_ES ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_ES "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_ES "Fin Carrera Min:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_ES "Max:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_ES ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_ES "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_ES "1 click = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_ES "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_ES "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_ES "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_ES "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_ES "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_ES "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_ES "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_ES "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_ES "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_ES "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_ES "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_ES "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_ES          "Velocida ventilador"
#define UI_TEXT_ACTION_FANSPEED_ES   "Vel. vent.:%Fs%%%"
#define UI_TEXT_FAN_OFF_ES           "Apaga ventilador"
#define UI_TEXT_FAN_25_ES            "Ventilador al 25%%%"
#define UI_TEXT_FAN_50_ES            "Ventilador al 50%%%"
#define UI_TEXT_FAN_75_ES            "Ventilador al 75%%%"
#define UI_TEXT_FAN_FULL_ES          "Ventilador al 100%%%"
#define UI_TEXT_STEPPER_INACTIVE_ES  "Motor Inactivo"
#define UI_TEXT_STEPPER_INACTIVE2A_ES "Dis. Despues: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_ES "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_ES    "Max. Inactivo"
#define UI_TEXT_POWER_INACTIVE2A_ES   "Dis. Despues: %ip"
#define UI_TEXT_POWER_INACTIVE2B_ES   "[min] 0=Off"
#define UI_TEXT_GENERAL_ES           "General"
#define UI_TEXT_BAUDRATE_ES          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_ES        "Pasos/MM:%Se"
#define UI_TEXT_EXTR_START_FEED_ES   "Start FR:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_ES     "Max FR:%XF"
#define UI_TEXT_EXTR_ACCEL_ES        "Acel:%XA"
#define UI_TEXT_EXTR_WATCH_ES        "Tiempo Estab.:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_ES    "Advance lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_ES    "Advance quad:%Xa"
#define UI_TEXT_EXTR_MANAGER_ES      "Control:%Xh"
#define UI_TEXT_EXTR_PGAIN_ES        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_ES     "Tiempo muerto:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_ES      "PWM control:%XM"
#define UI_TEXT_EXTR_IGAIN_ES        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_ES        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_ES         "Drive Min:%Xm"
#define UI_TEXT_EXTR_DMAX_ES         "Drive Max:%XM"
#define UI_TEXT_EXTR_PMAX_ES         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_ES "BangBang"
#define UI_TEXT_STRING_HM_PID_ES     "PID"
#define UI_TEXT_STRING_ACTION_ES     "Accion:%la"
#define UI_TEXT_HEATING_EXTRUDER_ES  "Calentando Extr %Ne"
#define UI_TEXT_HEATING_BED_ES       "Calentando Cama"
#define UI_TEXT_KILLED_ES            "Aborta"
#define UI_TEXT_STEPPER_DISABLED_ES  "Deshabilita motor"
#define UI_TEXT_EEPROM_STOREDA_ES     "Config."
#define UI_TEXT_EEPROM_STOREDB_ES     "almacenada en EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_ES     "Config."
#define UI_TEXT_EEPROM_LOADEDB_ES     "cargada de EEPROM"
#define UI_TEXT_UPLOADING_ES         "Actualizando..."
#define UI_TEXT_PAGE_BUFFER_ES       "Buffer:%oB"
#define UI_TEXT_PAGE_EXTRUDER_ES     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_ES     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_ES     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_ES     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_ES          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_ES    "Mult. Velocidad.:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_ES     "Mult. Flujo:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_ES  "Mostrar medicion"
#define UI_TEXT_RESET_MEASUREMENT_ES "Resetear medicion"
#define UI_TEXT_SET_MEASURED_ORIGIN_ES "Set Z=0"
#define UI_TEXT_ZCALIB_ES             "Z calib."
#define UI_TEXT_SET_P1_ES            "Set P1"
#define UI_TEXT_SET_P2_ES            "Set P2"
#define UI_TEXT_SET_P3_ES            "Set P3"
#define UI_TEXT_CALCULATE_LEVELING_ES "Calcula nivelacion"
#define UI_TEXT_LEVEL_ES             "Nivel delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_ES  "Esperando Temp.%XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_ES "Esperando Unidad:%XUmm"
#define UI_TEXT_SD_REMOVED_ES       "Tarjeta SD retira"
#define UI_TEXT_SD_INSERTED_ES      "Tarjeta SD insertada"
#define UI_TEXT_PRINTER_READY_ES    "Impresora lista."
#define UI_TEXT_PRINTTIME_DAYS_ES   " dias "
#define UI_TEXT_PRINTTIME_HOURS_ES  ":"
#define UI_TEXT_PRINTTIME_MINUTES_ES ""
#define UI_TEXT_PRINT_TIME_ES     "tiempo de impresion"
#define UI_TEXT_PRINT_FILAMENT_ES "Filamento impresa"
#define UI_TEXT_PRINTED_ES           "impreso"
#define UI_TEXT_POWER_ES            "Energie ATX on/off"
#define UI_TEXT_STRING_HM_DEADTIME_ES     "Tiempo muerto"
#define UI_TEXT_STRING_HM_SLOWBANG_ES     "SlowBang"
#define UI_TEXT_STOP_PRINT_ES "Detener impresion"
#define UI_TEXT_Z_BABYSTEPPING_ES "Z Babystep.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_ES "Cambio filamento"
#define UI_TEXT_WIZ_CH_FILAMENT1_ES "Cambio filamento"
#define UI_TEXT_WIZ_CH_FILAMENT2_ES "Gire para mover"
#define UI_TEXT_WIZ_CH_FILAMENT3_ES "fil. arriba/abajo"
#define UI_TEXT_CLICK_DONE_ES "Clic cuando se hace"
#define UI_TEXT_AUTOLEVEL_ONOFF_ES  "Autolevel: %ll"
#define UI_TEXT_SERVOPOS_ES "Pos. servo: %oS"
#define UI_TEXT_IGNORE_M106_ES      "Ignorar M106 cmd %Fi"
#define UI_TEXT_WIZ_REHEAT1_ES "Haga clic para"
#define UI_TEXT_WIZ_REHEAT2_ES "recalentar extr."
#define UI_TEXT_WIZ_WAITTEMP1_ES "Espere a temp."
#define UI_TEXT_WIZ_WAITTEMP2_ES "objetivo ..."
#define UI_TEXT_EXTRUDER_JAM_ES "Atasco extrusora"
#define UI_TEXT_STANDBY_ES "Standby"
#define UI_TEXT_BED_COATING_ES "Recubrimiento cama"
#define UI_TEXT_BED_COATING_SET1_ES "Rec. cama ajustado a"
#define UI_TEXT_BED_COATING_SET2_ES ""
#define UI_TEXT_NOCOATING_ES "Sin recubrimiento"
#define UI_TEXT_BUILDTAK_ES "BuildTak"
#define UI_TEXT_KAPTON_ES "Kapton"
#define UI_TEXT_BLUETAPE_ES "Cinta adhesiva azul"
#define UI_TEXT_PETTAPE_ES "Verde PET cinta"
#define UI_TEXT_GLUESTICK_ES "Barra de pegamento"
#define UI_TEXT_CUSTOM_ES "Especial"
#define UI_TEXT_COATING_CUSTOM_ES "Especial:%BCmm"
#define UI_TEXT_LANGUAGE_ES "Idioma"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_ES "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_ES "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_ES "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_ES "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_ES "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_ES "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_ES "Copias: %ed    Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_ES "Flujo:\xfd %of%%%  Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_ES "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_ES "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_ES "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_ES cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_ES "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_ES "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_ES "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_ES "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_ES "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_ES "%Uf m"
#define UI_TEXT_METER_PRINTED_ES "%Uf m " UI_TEXT_PRINTED_ES
#define UI_TEXT_STATUS_ES "%os"
#define UI_TEXT_EMPTY_ES ""
#define UI_TEXT_TEMP_SET_ES cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_ES cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_ES " %BCmm"
#define UI_TEXT_EXTR3_TEMP_ES "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_ES "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_ES "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_ES "Extrusor 4 Off"
#define UI_TEXT_EXTR4_OFF_ES "Extrusor 5 Off"
#define UI_TEXT_EXTR5_OFF_ES "Extrusor 6 Off"
#define UI_TEXT_EXTR3_SELECT_ES "%X3 Select Extr. 4"
#define UI_TEXT_EXTR4_SELECT_ES "%X4 Select Extr. 5"
#define UI_TEXT_EXTR5_SELECT_ES "%X5 Select Extr. 6"
#define UI_TEXT_DITTO_0_ES "%D0 No Hay Copias"
#define UI_TEXT_DITTO_1_ES "%D1 1 Copia"
#define UI_TEXT_DITTO_2_ES "%D2 2 Copias"
#define UI_TEXT_DITTO_3_ES "%D3 3 Copias"
#define UI_TEXT_ZPROBE_HEIGHT_ES "Altura Z-Probe:%zh"
#define UI_TEXT_OFFSETS_ES "Set print offsets"
#define UI_TEXT_X_OFFSET_ES "Set X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_ES "Set Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_ES "Set Z offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_ES   "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_ES   "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_ES   "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_ES   "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_ES   "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_ES   "Dripbox"
#define UI_TEXT_EXTR_XOFF_ES   "Xoffset:%Xx"
#define UI_TEXT_EXTR_YOFF_ES   "Yoffset:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_ES   "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_ES   "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_ES   "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_ES   "Dripboxes"
#define UI_TEXT_EXTR_XOFF_ES   "Xoffst:%Xx"
#define UI_TEXT_EXTR_YOFF_ES   "Yoffst:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_ES   "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_ES   "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_ES   "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_ES   "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_ES   "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_ES   "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_ES   "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_ES   "Bed Down"
#define UI_TEXT_WIFI_ONOFF_ES   "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_ES   "Do it again ?"
#define UI_TEXT_X_LENGTH_ES   "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_ES   "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_ES   "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_ES   "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_ES   "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_ES   "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_ES   "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_ES   "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_ES   "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_ES   "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_ES   "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_ES   "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_ES   "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_ES   "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_ES   "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_ES   "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_ES   "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_ES   "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_ES   "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_ES   "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_ES   "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_ES   "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_ES   "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_ES   "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_ES   "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_ES   "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_ES   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_ES   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_ES   "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_ES   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_ES   "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_ES   "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_ES   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_ES   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_ES   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_ES   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_ES   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_ES   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_ES   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_ES   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_ES   "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_ES   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_ES   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_ES   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_ES   "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_ES   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_ES   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_ES   "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_ES   "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_ES   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_ES   "and plate."
#define UI_TEXT_X_MIN_SETTING_ES   "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_ES   "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_ES   "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_ES   "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_ES   "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_ES   "System Version"
#define UI_TEXT_COOLDOWN_MENU_ES   "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_ES   "Ok to continue"
#define UI_TEXT_FAILED_ES   "Failed"
#define UI_TEXT_PLEASE_WAIT_ES   "Please wait"
#define UI_TEXT_MANUAL_LEVEL_ES   "Manual leveling"
#define UI_TEXT_WARNING_ES   "Warning!"
#define UI_TEXT_EXTRUDER_COLD_ES   "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_ES   "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_ES   "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_ES   "Powersave"
#define UI_TEXT_MAINTENANCE_ES   "Maintenance"
#define UI_TEXT_SETTINGS_ES   "Settings"
#define UI_TEXT_LOAD_UNLOAD_ES   "Load/Unload"
#define UI_TEXT_BED_ES   "Bed"
#define UI_TEXT_BED_OFF_ES   "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_ES   "Light AutoOff "
#define UI_TEXT_LIGHT_OFF_AFTER2A_ES   "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_ES   "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_ES   "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_ES   "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_ES   "Real Z unknown !"
#define UI_TEXT_MOVING_ES   "Moving..."
#define UI_TEXT_PREPARING_ES   "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_ES   "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_ES   "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_ES   "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_ES   "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_ES   "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_ES   "Z-probe failed"
#define UI_TEXT_SAVE_ES   "Save to EEPROM ?"
#define UI_TEXT_ZMIN_ES   "zMin : %zm"
#define UI_TEXT_ZPROBING_ES   "Probing..."
#define UI_TEXT_ZPOSITION_ES   "Go position..."
#define UI_TEXT_DO_YOU_ES   "Do you want to:"
#define UI_TEXT_CLEAN1_ES   "Clean nozzle and"
#define UI_TEXT_CLEAN2_ES   "plates first ?"
#define UI_TEXT_DISPLAY_MODE_ES   "Mode:%Md"
#define UI_TEXT_EASY_MODE_ES   "Easy"
#define UI_TEXT_ADVANCED_MODE_ES   "Advanced"
#define UI_TEXT_POWER_SAVE_ES   "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_ES   "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_ES   "Sound:  %so"
#define UI_TEXT_YES_SELECTED_ES   "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_ES   "   Yes    [No]"
#define UI_TEXT_PLEASE_CONFIRM_ES   "Please confirm"
#define UI_TEXT_CANCEL_ACTION_ES   "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_ES   "Continue"
#define UI_TEXT_CANCELED_ES   "Action canceled!"
#define UI_TEXT_CONFIRMED_ES   "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_ES   "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_ES   "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_ES   "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_ES   "And Clean"
#define UI_TEXT_PUSH_FILAMENT_ES   "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_ES   "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_ES   "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_ES   "Load Fail-Safe"
#define UI_TEXT_HEATING_ES   "Heating..."
#define UI_TEXT_WAIT_FOR_OK_ES   "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_ES    "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_ES    "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_ES   "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_ES   "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_ES   "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_ES "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_ES "Z:%x2"
#define UI_TEXT_P1_L3_ES "Mul:%om Buf:%oB"

#define UI_TEXT_P1_L1_EASY_ES "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_ES "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_ES "Z:%x2"

#else
#define UI_TEXT_P1_L1_ES "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_ES "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_ES "Z:%x2"

#define UI_TEXT_P1_L1_EASY_ES "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_ES "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_ES "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_ES "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_ES "Z:%x2"
#define UI_TEXT_P1_L3_ES "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_ES "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_ES "Z:%x2"
#define UI_TEXT_P1_L3_EASY_ES "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_ES "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_ES "Z:%x2"
#define UI_TEXT_P1_L3_ES "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_ES "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_ES "Z:%x2"
#define UI_TEXT_P1_L3_EASY_ES "Buffer:%oB"
#endif
#endif
#define  UI_TEXT_MENU_SCANNER_ES    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_ES    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_ES        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_ES       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_ES           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_ES          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_ES    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_ES     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_ES      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_ES      "Zmin evaluation"



// *************** Swedish translation ****************
// By Daniel Tedenljung 2013-08-21

#define UI_TEXT_ON_SE               "På"
#define UI_TEXT_OFF_SE              "Av"
#define UI_TEXT_NA_SE               "N/A" // Output for not available
#define UI_TEXT_YES_SE              "Ja"
#define UI_TEXT_NO_SE               "Nej"
#define UI_TEXT_PRINT_POS_SE        "Skriver ut..."
#define UI_TEXT_PRINTING_SE         "Skriver"
#define UI_TEXT_IDLE_SE             "Vilar"
#define UI_TEXT_NOSDCARD_SE         "Inget SD-kort"
#define UI_TEXT_ERROR_SE            "**** FEL ****"
#define UI_TEXT_BACK_SE             "Tillbaka "
#define UI_TEXT_QUICK_SETTINGS_SE   "Snabb Inställnigar"
#define UI_TEXT_ERRORMSG_SE         "%oe"
#define UI_TEXT_CONFIGURATION_SE    "Konfiguration"
#define UI_TEXT_POSITION_SE         "Position"
#define UI_TEXT_EXTRUDER_SE         "Extruder"
#define UI_TEXT_SD_CARD_SE          "SD-kort"
#define UI_TEXT_DEBUGGING_SE        "Debugging"
#define UI_TEXT_HOME_DELTA_SE       "Hem delta"
#define UI_TEXT_HOME_ALL_SE         "Hem alla"
#define UI_TEXT_HOME_X_SE           "Hem X"
#define UI_TEXT_HOME_Y_SE           "Hem Y"
#define UI_TEXT_HOME_Z_SE           "Hem Z"
#define UI_TEXT_PREHEAT_PLA_SE      "Förvärm PLA %B1"
#define UI_TEXT_PREHEAT_ABS_SE      "Förvärm ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_SE     "Belysning:     %lo"
#define UI_TEXT_COOLDOWN_SE         "Kyl ner"
#define UI_TEXT_SET_TO_ORIGIN_SE    "Sätt som origo"
#define UI_TEXT_DISABLE_STEPPER_SE  "Stäng av stegmotor"
#define UI_TEXT_X_POSITION_SE       "X-position"
#define UI_TEXT_X_POS_FAST_SE       "X-pos. snabb"
#define UI_TEXT_Y_POSITION_SE       "Y-position"
#define UI_TEXT_Y_POS_FAST_SE       "Y-pos. snabb"
#define UI_TEXT_Z_POSITION_SE       "Z-position"
#define UI_TEXT_Z_POS_FAST_SE       "Z-pos. snabb"
#define UI_TEXT_E_POSITION_SE       "Extr.-position"
#define UI_TEXT_BED_TEMP_SE         "Bädd-temp:%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_SE         "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_SE         "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_SE         "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_SE         "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_SE          "Extruder av %B4"
#else
#define UI_TEXT_EXTR0_OFF_SE          "Extruder1av%B4"
#endif
#define UI_TEXT_EXTR1_OFF_SE          "Extruder2av%B5"
#define UI_TEXT_EXTR2_OFF_SE          "Extruder3 av%B6"
#define UI_TEXT_EXTR0_SELECT_SE     "Välj Extr.1 %X0"
#define UI_TEXT_EXTR1_SELECT_SE     "Välj Extr.2 %X1"
#define UI_TEXT_EXTR2_SELECT_SE     "Välj Extr.3 %X2"
#define UI_TEXT_EXTR_ORIGIN_SE      "Sätt Origo"
#define UI_TEXT_PRINT_X_SE          "Skriv X: %ax"
#define UI_TEXT_PRINT_Y_SE          "Skriv Y: %ay"
#define UI_TEXT_PRINT_Z_SE          "Skriv Z: %az"
#define UI_TEXT_PRINT_Z_DELTA_SE    "Skriv: %az"
#define UI_TEXT_MOVE_X_SE           "Flytta. X:   %aX"
#define UI_TEXT_MOVE_Y_SE           "Flytta. Y:   %aY"
#define UI_TEXT_MOVE_Z_SE           "Flytta. Z:   %aZ"
#define UI_TEXT_MOVE_Z_DELTA_SE     "Flytta.:   %aZ"
#define UI_TEXT_JERK_SE             "Ryck:   %aj"
#define UI_TEXT_ZJERK_SE            "Z-ryck:   %aJ"
#define UI_TEXT_ACCELERATION_SE     "Acceleration"
#define UI_TEXT_STORE_TO_EEPROM_SE  "Spara till EEPROM"
#define UI_TEXT_LOAD_EEPROM_SE      "Ladda f. EEPROM"
#define UI_TEXT_DBG_ECHO_SE         "Eko:    %do"
#define UI_TEXT_DBG_INFO_SE         "Info:   %di"
#define UI_TEXT_DBG_ERROR_SE        "Felkoder:   %de"
#define UI_TEXT_DBG_DRYRUN_SE       "Torrkörning:   %dd"
#define UI_TEXT_OPS_OFF_SE          "%O1 OPS av"
#define UI_TEXT_OPS_CLASSIC_SE      "%O2 OPS Klassisk"
#define UI_TEXT_OPS_FAST_SE         "%O3 OPS Snabb"
#define UI_TEXT_OPS_RETRACT_SE      "Backa: %Or"
#define UI_TEXT_OPS_BACKSLASH_SE    "Backsl. :%Ob"
#define UI_TEXT_OPS_MINDIST_SE      "Min.dist: %Od"
#define UI_TEXT_OPS_MOVE_AFTER_SE   "Flytta efter:%Oa"
#define UI_TEXT_ANTI_OOZE_SE        "Antikladd"
#define UI_TEXT_PRINT_FILE_SE       "Skriv ut fil"
#define UI_TEXT_PAUSE_PRINT_SE      "Pausa utskrift"
#define UI_TEXT_CONTINUE_PRINT_SE   "Fortsätt utskrift"
#define UI_TEXT_UNMOUNT_CARD_SE     "Mata ut kort"
#define UI_TEXT_MOUNT_CARD_SE       "Anslut kort"
#define UI_TEXT_DELETE_FILE_SE      "Radera fil"
#define UI_TEXT_FEEDRATE_SE         "Matnings Fart"
#define UI_TEXT_FEED_MAX_X_SE       "Max X:   %fx"
#define UI_TEXT_FEED_MAX_Y_SE       "Max Y:   %fy"
#define UI_TEXT_FEED_MAX_Z_SE       "Max Z:   %fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_SE "Max:   %fz"
#define UI_TEXT_FEED_HOME_X_SE      "Ref X:   %fX"
#define UI_TEXT_FEED_HOME_Y_SE      "Ref Y:   %fY"
#define UI_TEXT_FEED_HOME_Z_SE      "Ref Z:   %fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_SE "Ref:   %fZ"
#define UI_TEXT_ACTION_XPOSITION4A_SE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_SE "Min Ändläge:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_SE "Max Ändläge:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_SE ""
#define UI_TEXT_ACTION_YPOSITION4A_SE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_SE "Min Ändläge:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_SE "Max  Ändläge:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_SE ""
#define UI_TEXT_ACTION_ZPOSITION4A_SE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_SE "Min  Ändläge:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_SE "Max  Ändläge:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_SE ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_SE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_SE "Min Ändläge:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_SE "Max Ändläge:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_SE ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_SE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_SE "Min Ändläge:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_SE "Max Ändläge:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_SE ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_SE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_SE "Min Ändläge:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_SE "Max Ändläge:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_SE ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_SE "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_SE "1 klick = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_SE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_SE "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_SE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_SE "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_SE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_SE "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_SE "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_SE "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_SE "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_SE "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_SE "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_SE "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_SE "Fläkt hast."
#define UI_TEXT_ACTION_FANSPEED_SE   "Fläkt hast.:%Fs%%%"
#define UI_TEXT_FAN_OFF_SE           "Stäng av fläkt"
#define UI_TEXT_FAN_25_SE            "Fläkt 25%%%"
#define UI_TEXT_FAN_50_SE            "Fläkt 50%%%"
#define UI_TEXT_FAN_75_SE            "Fläkt 75%%%"
#define UI_TEXT_FAN_FULL_SE          "Full Fläkt"
#define UI_TEXT_STEPPER_INACTIVE_SE  "Stegmotorer inakt."
#define UI_TEXT_STEPPER_INACTIVE2A_SE "Inakt. efter: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_SE "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_SE    "Max. inaktiv"
#define UI_TEXT_POWER_INACTIVE2A_SE   "Inakt. efter: %ip"
#define UI_TEXT_POWER_INACTIVE2B_SE   "[min] 0=Off"
#define UI_TEXT_GENERAL_SE           "Generella"
#define UI_TEXT_BAUDRATE_SE          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_SE        "Steg/MM:%Se"
#define UI_TEXT_EXTR_START_FEED_SE   "Start FR: %Xf"
#define UI_TEXT_EXTR_MAX_FEED_SE     "Max FR: %XF"
#define UI_TEXT_EXTR_ACCEL_SE        "Accel:   %XA"
#define UI_TEXT_EXTR_WATCH_SE        "Stab. tid: %Xw"
#define UI_TEXT_EXTR_ADVANCE_L_SE    "Avancera L: %Xl"
#define UI_TEXT_EXTR_ADVANCE_K_SE    "Avancera Q: %Xa"
#define UI_TEXT_EXTR_MANAGER_SE      "Justera:%Xh"
#define UI_TEXT_EXTR_PGAIN_SE        "PID P:   %Xp"
#define UI_TEXT_EXTR_DEADTIME_SE     "Dödtid:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_SE      "Justera PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_SE        "PID I:  %Xi"
#define UI_TEXT_EXTR_DGAIN_SE        "PID D:  %Xd"
#define UI_TEXT_EXTR_DMIN_SE         "Drive min:   %Xm"
#define UI_TEXT_EXTR_DMAX_SE         "Drive max:   %XM"
#define UI_TEXT_EXTR_PMAX_SE         "PID max:    %XD"
#define UI_TEXT_STRING_HM_BANGBANG_SE " BangBang"
#define UI_TEXT_STRING_HM_PID_SE     "PID"
#define UI_TEXT_STRING_ACTION_SE     "Funktion: %la"
#define UI_TEXT_HEATING_EXTRUDER_SE  "Värmer Extruder%Ne"
#define UI_TEXT_HEATING_BED_SE       "Värmer Bädd"
#define UI_TEXT_KILLED_SE            "Stoppad"
#define UI_TEXT_STEPPER_DISABLED_SE  "Stegmotorer av"
#define UI_TEXT_EEPROM_STOREDA_SE     "Konfiguration"
#define UI_TEXT_EEPROM_STOREDB_SE     "Sparad i EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_SE     "Konfiguration"
#define UI_TEXT_EEPROM_LOADEDB_SE     "Laddat fr. EEPROM"
#define UI_TEXT_UPLOADING_SE         "Uppladdning..."
#define UI_TEXT_PAGE_BUFFER_SE       "Buffer:%oB"
#define UI_TEXT_PAGE_EXTRUDER_SE     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_SE     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_SE     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_SE     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_SE          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_SE    "Hastighet Mul.:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_SE     "Flöde Mul.:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_SE  "Visa mätning"
#define UI_TEXT_RESET_MEASUREMENT_SE "Återställ mätning"
#define UI_TEXT_SET_MEASURED_ORIGIN_SE "Ställ Z=0"
#define UI_TEXT_ZCALIB_SE             "Z Kalib."
#define UI_TEXT_SET_P1_SE            "Ställ  P1"
#define UI_TEXT_SET_P2_SE            "Ställ  P2"
#define UI_TEXT_SET_P3_SE            "Ställ  P3"
#define UI_TEXT_CALCULATE_LEVELING_SE "Beräkna Nivellering"
#define UI_TEXT_LEVEL_SE             "Nivellering delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_SE  "Avvakta  Temp.: %XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_SE "Avvakta pos.: %XUmm"
#define UI_TEXT_SD_REMOVED_SE       "SD-kort tas bort"
#define UI_TEXT_SD_INSERTED_SE      "SD-kort isatt"
#define UI_TEXT_PRINTER_READY_SE    "Utskrift klar."
#define UI_TEXT_PRINTTIME_DAYS_SE   " Dagar "
#define UI_TEXT_PRINTTIME_HOURS_SE  ":"
#define UI_TEXT_PRINTTIME_MINUTES_SE ":"
#define UI_TEXT_PRINT_TIME_SE     "Filamentstid"
#define UI_TEXT_PRINT_FILAMENT_SE "Filament utskrift"
#define UI_TEXT_PRINTED_SE           "Utskrivet"
#define UI_TEXT_POWER_SE            "ATX Ström PÅ/AV"
#define UI_TEXT_STRING_HM_DEADTIME_SE     "Dödtid"
#define UI_TEXT_STRING_HM_SLOWBANG_SE     "SlowBang"
#define UI_TEXT_STOP_PRINT_SE "Stoppa Utskrift"
#define UI_TEXT_Z_BABYSTEPPING_SE "Z babystep.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_SE "Byta Filamentet"
#define UI_TEXT_WIZ_CH_FILAMENT1_SE "Byta Filamentet"
#define UI_TEXT_WIZ_CH_FILAMENT2_SE "Rotera för att flytta"
#define UI_TEXT_WIZ_CH_FILAMENT3_SE "Filamentet UPP/NER"
#define UI_TEXT_CLICK_DONE_SE "Klicka när du är klar"
#define UI_TEXT_AUTOLEVEL_ONOFF_SE  "Autonivellera:  %ll"
#define UI_TEXT_SERVOPOS_SE "Servoläge: %oS"
#define UI_TEXT_IGNORE_M106_SE      "Ignorera M106 cmd %Fi"
#define UI_TEXT_WIZ_REHEAT1_SE "Klicka för att Återvärma"
#define UI_TEXT_WIZ_REHEAT2_SE "Extruders."
#define UI_TEXT_WIZ_WAITTEMP1_SE "Avaktata för mål"
#define UI_TEXT_WIZ_WAITTEMP2_SE "Måltemperaturer ..."
#define UI_TEXT_EXTRUDER_JAM_SE "Extruder Kloggat"
#define UI_TEXT_STANDBY_SE "Standby"
#define UI_TEXT_BED_COATING_SE "Bäddbeläggning"
#define UI_TEXT_BED_COATING_SET1_SE "Beläggning ställd till"
#define UI_TEXT_BED_COATING_SET2_SE ""
#define UI_TEXT_NOCOATING_SE "Ingen Beläggning"
#define UI_TEXT_BUILDTAK_SE "BuildTak"
#define UI_TEXT_KAPTON_SE "Kapton"
#define UI_TEXT_BLUETAPE_SE "Blå maskerings Tejp"
#define UI_TEXT_PETTAPE_SE "Grön PET Tejp"
#define UI_TEXT_GLUESTICK_SE "Limstift"
#define UI_TEXT_CUSTOM_SE "Anpassad"
#define UI_TEXT_COATING_CUSTOM_SE "Anpassad: %BCmm"
#define UI_TEXT_LANGUAGE_SE "Språk"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_SE "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_SE "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_SE "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_SE "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_SE "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_SE "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_SE "Kopior: %ed    Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_SE "Fl" STR_ouml "de:\xfd %of%%%  Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_SE "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_SE "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_SE "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_SE cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_SE "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_SE "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_SE "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_SE "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_SE "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_SE "%Uf m"
#define UI_TEXT_METER_PRINTED_SE "%Uf m " UI_TEXT_PRINTED_SE
#define UI_TEXT_STATUS_SE "%os"
#define UI_TEXT_EMPTY_SE ""
#define UI_TEXT_TEMP_SET_SE cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_SE cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_SE " %BCmm"
#define UI_TEXT_EXTR3_TEMP_SE "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_SE "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_SE "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_SE "Extruder 4 av"
#define UI_TEXT_EXTR4_OFF_SE "Extruder 5 av"
#define UI_TEXT_EXTR5_OFF_SE "Extruder 6 av"
#define UI_TEXT_EXTR3_SELECT_SE "%X3 V" STR_auml "lj Extr. 4"
#define UI_TEXT_EXTR4_SELECT_SE "%X4 V" STR_auml "lj Extr. 5"
#define UI_TEXT_EXTR5_SELECT_SE "%X5 V" STR_auml "lj Extr. 6"
#define UI_TEXT_DITTO_0_SE "%D0 Inga Kopior"
#define UI_TEXT_DITTO_1_SE "%D1 1 Kopia"
#define UI_TEXT_DITTO_2_SE "%D2 2 Kopior"
#define UI_TEXT_DITTO_3_SE "%D3 3 Kopior"
#define UI_TEXT_ZPROBE_HEIGHT_SE "Z-prob höjd:%zh"
#define UI_TEXT_OFFSETS_SE "Ställ utskrift offsets"
#define UI_TEXT_X_OFFSET_SE "Ställ X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_SE "Ställ Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_SE "Ställ Z offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_SE   "Ladda Filament"
#define UI_TEXT_UNLOAD_FILAMENT_SE   "Ladda ur Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_SE   "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_SE   "Rengöra Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_SE   "Töm Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_SE   "Dripbox"
#define UI_TEXT_EXTR_XOFF_SE   "Xoffset:%Xx"
#define UI_TEXT_EXTR_YOFF_SE   "Yoffset:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_SE   "Fil.Sensorer:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_SE   "Töm Dripboxar"
#define UI_TEXT_CLEANING_DRIPBOX_SE   "Tömmer Dripboxar"
#define UI_TEXT_CLEAN_DRIPBOX_3_SE   "Dripboxes"
#define UI_TEXT_EXTR_XOFF_SE   "Xoffst:%Xx"
#define UI_TEXT_EXTR_YOFF_SE   "Yoffst:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_SE   "Ladda i Höger \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_SE   "Ladda ur Höger \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_SE   "Ladda i Vänster \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_SE   "Ladda ur Vänster \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_SE   "Ladda Fiament.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_SE   "Ladda ur Filament.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_SE   "Ladda Dist.anse: %fd"
#define UI_TEXT_BED_DOWN_SE   "Bädd Ner"
#define UI_TEXT_WIFI_ONOFF_SE   "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_SE   "Gör om ?"
#define UI_TEXT_X_LENGTH_SE   "X Längd:%Lxmm"
#define UI_TEXT_Y_LENGTH_SE   "Y Längd:%Lymm"
#define UI_TEXT_Z_LENGTH_SE   "Z Längd:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_SE   "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_SE   "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_SE   "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_SE   "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_SE   "Extruder%Ne har inget"
#define UI_TEXT_ALERT_NO_FILAMENTB_SE   "Filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_SE   "Tryck Ok för att gå till"
#define UI_TEXT_ALERT_NO_FILAMENTD_SE   "Byt filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_SE   "Finn 3 vred"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_SE   "på botten av"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_SE   "plattformen och"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_SE   "spännn 4 varv."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_SE   "Jag kommer"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_SE   "flytta extr. till"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_SE   "olika positioner"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_SE   "för justering."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_SE   "Vid varje position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_SE   "vi måste"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_SE   "justera 2 vred"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_SE   "samtidigt."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_SE   "Munstycket är på"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_SE   "rätt höjd om"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_SE   "du kan få in ett blad"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_SE   "standard A4-papper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_SE   "mellan munstycket"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_SE   "och bädden. Ta"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_SE   "ett A-4 papper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_SE   "för fortsättning."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_SE   "Justera främre"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_SE   "vred (en) så papper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_SE   "bara går emellan"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_SE   "munstycke och bädd"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_SE   "Justera bakre"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_SE   "vred (en) så papper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_SE   "bara går emellan"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_SE   "munstycke och bädd"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_SE   "Justera bakre"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_SE   "och högra vredet"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_SE   "till pappret knappt"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_SE   "glider."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_SE   "Justera bakre"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_SE   "och vänstra vredet"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_SE   "till pappret knappt"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_SE   "glider."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_SE   "Kontrollera igen "
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_SE   "att pappret knappt"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_SE   "mellan munstycke"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_SE   "och bädd."
#define UI_TEXT_X_MIN_SETTING_SE   "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_SE   "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_SE   "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_SE   "Övre lucka öpen!"
#define UI_TEXT_TOP_SENSOR_ONOFF_SE   "Övre Sensor: %st"
#define UI_TEXT_VERSION_MENU_SE   "System Version"
#define UI_TEXT_COOLDOWN_MENU_SE   "Nedkylning      %B3"
#define UI_TEXT_WAIT_OK_SE   "Ok att fortsätta"
#define UI_TEXT_FAILED_SE   "Misslyckad"
#define UI_TEXT_PLEASE_WAIT_SE   "Vänligen Vänta"
#define UI_TEXT_MANUAL_LEVEL_SE   "Manuell nivellering"
#define UI_TEXT_WARNING_SE   "Varning!"
#define UI_TEXT_EXTRUDER_COLD_SE   "Extruder kall!"
#define UI_TEXT_HEAT_EXTRUDER_SE   "Värm extruder ?"
#define UI_TEXT_WRITE_DEBUG_SE   "Skriv Felsök"
#define UI_TEXT_POWER_SAVE_MENU_SE   "Energispar"
#define UI_TEXT_MAINTENANCE_SE   "Underhåll"
#define UI_TEXT_SETTINGS_SE   "Inställningar"
#define UI_TEXT_LOAD_UNLOAD_SE   "Ladda/Ladda Ur"
#define UI_TEXT_BED_SE   "Bädd"
#define UI_TEXT_BED_OFF_SE   "Stäng av bädd %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_SE   "Lampa AutoAv "
#define UI_TEXT_LIGHT_OFF_AFTER2A_SE   "Släck Efter:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_SE   "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_SE   "Faktisk X okänd !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_SE   "Faktisk Y okänd !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_SE   "Faktisk Z okänd !"
#define UI_TEXT_MOVING_SE   "Flyttar..."
#define UI_TEXT_PREPARING_SE   "Förbereder..."
#define UI_TEXT_KEEP_LIGHT_ON_SE   "Skriv med /LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_SE   "Laddar \005"
#define UI_TEXT_UNLOADING_FILAMENT_SE   "Laddar ur \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_SE   "Ladda/Ladda Ur \005"
#define UI_TEXT_AUTOLEVEL_SE   "Auto nivellera bädd"
#define UI_TEXT_Z_PROBE_FAILED_SE   "Z-test misslyckat"
#define UI_TEXT_SAVE_SE   "Spara till EEPROM ?"
#define UI_TEXT_ZMIN_SE   "zMin : %zm"
#define UI_TEXT_ZPROBING_SE   "Testar..."
#define UI_TEXT_ZPOSITION_SE   "Gå till position..."
#define UI_TEXT_DO_YOU_SE   "Vill du:"
#define UI_TEXT_CLEAN1_SE   "Rengöra munstycke"
#define UI_TEXT_CLEAN2_SE   "och bädd först ?"
#define UI_TEXT_DISPLAY_MODE_SE   "Läge:%Md"
#define UI_TEXT_EASY_MODE_SE   "Enkelt"
#define UI_TEXT_ADVANCED_MODE_SE   "Avancerat"
#define UI_TEXT_POWER_SAVE_SE   "Energispar:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_SE   "Emblem lampa:%lb"
#define UI_TEXT_SOUND_ONOF_SE   "Ljud:  %so"
#define UI_TEXT_YES_SELECTED_SE   "  [Ja]    Nej"
#define UI_TEXT_NO_SELECTED_SE   "   Ja    [Nej]"
#define UI_TEXT_PLEASE_CONFIRM_SE   "Vänl. Bekräfta"
#define UI_TEXT_CANCEL_ACTION_SE   "Avbryt:"
#define UI_TEXT_CONTINUE_ACTION_SE   "Fortsätt"
#define UI_TEXT_CANCELED_SE   "Funktion stoppad!"
#define UI_TEXT_CONFIRMED_SE   "Funktion konfirmerad!"
#define UI_TEXT_CLEAN_NOZZLE_SE   "Rengöra Munstycke"
#define UI_TEXT_CLEANING_NOZZLE_SE   "Rengör Munstycke"
#define UI_TEXT_CLEAN_DRIPBOX_1_SE   "Vänligen ta"
#define UI_TEXT_CLEAN_DRIPBOX_2_SE   "Och Rengör"
#define UI_TEXT_PUSH_FILAMENT_SE   "Tryck Filament"
#define UI_TEXT_WAIT_FILAMENT_SE   "Tryck Ok för start"
#define UI_TEXT_LOAD_FAILSAFE2_SE   "efter Laddning ?"
#define UI_TEXT_LOAD_FAILSAFE_SE   "Ladda Felsäkert"
#define UI_TEXT_HEATING_SE   "Värmer..."
#define UI_TEXT_WAIT_FOR_OK_SE   "Tryck Ok för avslut"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_SE    "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_SE    "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_SE   "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_SE   "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_SE   "Nivellering Matrix"
#if HAVE_HEATED_BED
///Davinci Specifika,Sidor 1-2-3,dom är olika beroende på om du har  1 eller 2 extruders, och om du är i Enkelt eller Avancerat Läge
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_SE "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_SE "Z:%x2"
#define UI_TEXT_P1_L3_SE "Mul:%om Buf:%oB"

#define UI_TEXT_P1_L1_EASY_SE "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_SE "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_SE "Z:%x2"

#else
#define UI_TEXT_P1_L1_SE "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_SE "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_SE "Z:%x2"

#define UI_TEXT_P1_L1_EASY_SE "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_SE "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_SE "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_SE "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_SE "Z:%x2"
#define UI_TEXT_P1_L3_SE "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_SE "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_SE "Z:%x2"
#define UI_TEXT_P1_L3_EASY_SE "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_SE "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_SE "Z:%x2"
#define UI_TEXT_P1_L3_SE "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_SE "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_SE "Z:%x2"
#define UI_TEXT_P1_L3_EASY_SE "Buffer:%oB"
#endif
#endif
#define UI_TEXT_DBG_ENDSTOP_SE      "EndStop:%dp"
#define UI_TEXT_MENU_SCANNER_SE    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_SE    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_SE        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_SE       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_SE           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_SE          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_SE    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_SE     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_SE      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_SE      "Zmin evaluation"




// *************** French translation ****************

#define UI_TEXT_ON_FR               "Oui"
#define UI_TEXT_OFF_FR              "Non"
#define UI_TEXT_NA_FR               "N/A" // Output for not available
#define UI_TEXT_YES_FR              "Oui"
#define UI_TEXT_NO_FR               "Non"
#define UI_TEXT_PRINT_POS_FR        "Impression"
#define UI_TEXT_PRINTING_FR         "Impression"
#define UI_TEXT_IDLE_FR             "Au repos"
#define UI_TEXT_NOSDCARD_FR         "Pas de carte SD"
#define UI_TEXT_ERROR_FR            "**** ERREUR ****"
#define UI_TEXT_BACK_FR             "Retour \001"
#define UI_TEXT_QUICK_SETTINGS_FR   "Reglages rapides"
#define UI_TEXT_ERRORMSG_FR         "%oe"
#define UI_TEXT_CONFIGURATION_FR    "Configuration"
#define UI_TEXT_POSITION_FR         "Position"
#define UI_TEXT_EXTRUDER_FR         "Extrudeur"
#define UI_TEXT_SD_CARD_FR          "Carte SD"
#define UI_TEXT_DEBUGGING_FR        "Deboguer"
#define UI_TEXT_HOME_DELTA_FR       "Init. orig. Delta"
#define UI_TEXT_HOME_ALL_FR         "Initialise axes"
#define UI_TEXT_HOME_X_FR           "Initialise X"
#define UI_TEXT_HOME_Y_FR           "Initialise Y"
#define UI_TEXT_HOME_Z_FR           "Initialise Z"
#define UI_TEXT_PREHEAT_PLA_FR      "Prechauf. PLA %B1"
#define UI_TEXT_PREHEAT_ABS_FR      "Prechauf. ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_FR     "Eclairage : %lo"
#define UI_TEXT_COOLDOWN_FR         "Refroidir"
#define UI_TEXT_SET_TO_ORIGIN_FR    "R.A.Z. origine"
#define UI_TEXT_DISABLE_STEPPER_FR  "Arret moteurs"
#define UI_TEXT_X_POSITION_FR       "Position X"
#define UI_TEXT_X_POS_FAST_FR       "Pos. Rapide X"
#define UI_TEXT_Y_POSITION_FR       "Position Y"
#define UI_TEXT_Y_POS_FAST_FR       "Pos. Rapide Y"
#define UI_TEXT_Z_POSITION_FR       "Position Z"
#define UI_TEXT_Z_POS_FAST_FR       "Pos. Rapide Z"
#define UI_TEXT_E_POSITION_FR       "Position Extr."
#define UI_TEXT_BED_TEMP_FR         "Plat.:%eb/%Eb\002C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_FR       "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_FR       "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_FR       "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_FR       "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_FR        "Arret Ext    %B4"
#else
#define UI_TEXT_EXTR0_OFF_FR        "Arret Ext1   %B4"
#endif
#define UI_TEXT_EXTR1_OFF_FR        "Arret Ext2   %B5"
#define UI_TEXT_EXTR2_OFF_FR        "Arret Ext3   %B6"
#define UI_TEXT_EXTR0_SELECT_FR     "Select. Ext1 %X0"
#define UI_TEXT_EXTR1_SELECT_FR     "Select. Ext2 %X1"
#define UI_TEXT_EXTR2_SELECT_FR     "Select. Ext3 %X2"
#define UI_TEXT_EXTR_ORIGIN_FR      "R.A.Z. origine"
#define UI_TEXT_PRINT_X_FR          "Imprim. X:%ax"
#define UI_TEXT_PRINT_Y_FR          "Imprim. Y:%ay"
#define UI_TEXT_PRINT_Z_FR          "Imprim. Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_FR    "Imprim.:%az"
#define UI_TEXT_MOVE_X_FR           "Deplac. X:%aX"
#define UI_TEXT_MOVE_Y_FR           "Deplac. Y:%aY"
#define UI_TEXT_MOVE_Z_FR           "Deplac. Z:%aZ"
#define UI_TEXT_MOVE_Z_DELTA_FR     "Deplac.:%aZ"
#define UI_TEXT_JERK_FR             "Jerk:     %aj"
#define UI_TEXT_ZJERK_FR            "Z-Jerk:   %aJ"
#define UI_TEXT_ACCELERATION_FR     "Acceleration"
#define UI_TEXT_STORE_TO_EEPROM_FR  "Enreg. \176 EEPROM"
#define UI_TEXT_LOAD_EEPROM_FR       "Charg. \177 EEPROM"
#define UI_TEXT_DBG_ECHO_FR         "Echo       :%do"
#define UI_TEXT_DBG_INFO_FR         "Information:%di"
#define UI_TEXT_DBG_ERROR_FR        "Erreur :    %de"
#define UI_TEXT_DBG_DRYRUN_FR       "Mode test:  %dd"
#define UI_TEXT_DBG_ENDSTOP_FR      "Findecourse:%dp"
#define UI_TEXT_OPS_OFF_FR          "%O0 OPS Off"
#define UI_TEXT_OPS_CLASSIC_FR      "%O1 OPS classiq."
#define UI_TEXT_OPS_FAST_FR         "%O2 OPS rapide"
#define UI_TEXT_OPS_RETRACT_FR      "Retract. :%Or"
#define UI_TEXT_OPS_BACKSLASH_FR    "Backsl. :%Ob"
#define UI_TEXT_OPS_MINDIST_FR      "Min.dist:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_FR   "Deplac. apres:%Oa"
#define UI_TEXT_ANTI_OOZE_FR        "Anti Ooze"
#define UI_TEXT_PRINT_FILE_FR       "Imprim. fichier"
#define UI_TEXT_PAUSE_PRINT_FR      "Pause impress."
#define UI_TEXT_CONTINUE_PRINT_FR   "Continuer impress."
#define UI_TEXT_UNMOUNT_CARD_FR     "Retirer carte"
#define UI_TEXT_MOUNT_CARD_FR       "Inserer carte"
#define UI_TEXT_DELETE_FILE_FR      "Supp. fichier"
#define UI_TEXT_FEEDRATE_FR         "Vitesse"
#define UI_TEXT_FEED_MAX_X_FR       "Max X:    %fx"
#define UI_TEXT_FEED_MAX_Y_FR       "Max Y:    %fy"
#define UI_TEXT_FEED_MAX_Z_FR       "Max Z:    %fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_FR "Max:%fz"
#define UI_TEXT_FEED_HOME_X_FR      "Init. X:  %fX"
#define UI_TEXT_FEED_HOME_Y_FR      "Init. Y:  %fY"
#define UI_TEXT_FEED_HOME_Z_FR      "Init. Z:  %fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_FR "Init.:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_FR "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_FR "Min butee:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_FR "Max butee:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_FR ""
#define UI_TEXT_ACTION_YPOSITION4A_FR "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_FR "Min butee:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_FR "Max butee:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_FR ""
#define UI_TEXT_ACTION_ZPOSITION4A_FR "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_FR "Min butee:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_FR "Max butee:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_FR ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_FR "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_FR "Min butee:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_FR "Max butee:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_FR ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_FR "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_FR "Min butee:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_FR "Max butee:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_FR ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_FR "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_FR "Min butee:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_FR "Max butee:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_FR ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_FR "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_FR "1 clic = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_FR "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_FR "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_FR "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_FR "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_FR "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_FR "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_FR "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_FR "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_FR "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_FR "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_FR "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_FR "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_FR          "Vit. ventil."
#define UI_TEXT_ACTION_FANSPEED_FR   "Vit. Ventil.:%Fs%%%"
#define UI_TEXT_FAN_OFF_FR           "Arret Ventil."
#define UI_TEXT_FAN_25_FR            "Ventil. 25%%%"
#define UI_TEXT_FAN_50_FR            "Ventil. 50%%%"
#define UI_TEXT_FAN_75_FR            "Ventil. 75%%%"
#define UI_TEXT_FAN_FULL_FR          "Ventil. Max"
#define UI_TEXT_STEPPER_INACTIVE_FR  "Arret moteurs"
#define UI_TEXT_STEPPER_INACTIVE2A_FR "Arret apres: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_FR "[min] 0=Non"
#define UI_TEXT_POWER_INACTIVE_FR    "Arret total"
#define UI_TEXT_POWER_INACTIVE2A_FR   "Arret apres: %ip"
#define UI_TEXT_POWER_INACTIVE2B_FR    "[min] 0=Non"
#define UI_TEXT_GENERAL_FR           "General"
#define UI_TEXT_BAUDRATE_FR          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_FR        "Pas/mm:   %Se"
#define UI_TEXT_EXTR_START_FEED_FR   "Debit min:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_FR     "Debit max:%XF"
#define UI_TEXT_EXTR_ACCEL_FR        "Accel:    %XA"
#define UI_TEXT_EXTR_WATCH_FR        "Stab.Temps:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_FR    "Avance L.:  %Xl"
#define UI_TEXT_EXTR_ADVANCE_K_FR    "Avance Q.:%Xa"
#define UI_TEXT_EXTR_MANAGER_FR      "Controle:%Xh"
#define UI_TEXT_EXTR_PGAIN_FR        "PID P:  %Xp"
#define UI_TEXT_EXTR_DEADTIME_FR     "T. mort: %Xp"
#define UI_TEXT_EXTR_DMAX_DT_FR      "ControlePWM:%XM"
#define UI_TEXT_EXTR_IGAIN_FR        "PID I:  %Xi"
#define UI_TEXT_EXTR_DGAIN_FR        "PID D:  %Xd"
#define UI_TEXT_EXTR_DMIN_FR         "Drive Min:%Xm"
#define UI_TEXT_EXTR_DMAX_FR         "Drive Max:%XM"
#define UI_TEXT_EXTR_PMAX_FR         "PID Max:    %XD"
#define UI_TEXT_STRING_HM_BANGBANG_FR "  Bang"
#define UI_TEXT_STRING_HM_PID_FR     "   PID"
#define UI_TEXT_STRING_ACTION_FR     "Action:%la"
#define UI_TEXT_HEATING_EXTRUDER_FR  "Chauffe extrud.%Ne"
#define UI_TEXT_HEATING_BED_FR       "Chauffe plat."
#define UI_TEXT_KILLED_FR            "Arretee"
#define UI_TEXT_STEPPER_DISABLED_FR  "Moteurs arretes"
#define UI_TEXT_EEPROM_STOREDA_FR     "Configuration"
#define UI_TEXT_EEPROM_STOREDB_FR    "Enrg. \176 EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_FR     "Configuration"
#define UI_TEXT_EEPROM_LOADEDB_FR     "Charg. \177 EEPROM"
#define UI_TEXT_UPLOADING_FR         "Telechargement.."
#define UI_TEXT_PAGE_BUFFER_FR       "Tampon:%oB"
#define UI_TEXT_PAGE_EXTRUDER_FR     " E:%ec/%Ec\002C\176%oC"
#define UI_TEXT_PAGE_EXTRUDER1_FR     "E1:%e0/%E0\002C\176%o0"
#define UI_TEXT_PAGE_EXTRUDER2_FR     "E2:%e1/%E1\002C\176%o1"
#define UI_TEXT_PAGE_EXTRUDER3_FR     "E3:%e2/%E2\002C\176%o2"
#define UI_TEXT_PAGE_BED_FR          " B:%eb/%Eb\002C\176%ob"
#define UI_TEXT_SPEED_MULTIPLY_FR    "Vitesse :  %om%%%"
#define UI_TEXT_FLOW_MULTIPLY_FR     "Debit :    %of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_FR  "Montrer mesure"
#define UI_TEXT_RESET_MEASUREMENT_FR "R.A.Z mesure"
#define UI_TEXT_SET_MEASURED_ORIGIN_FR "Regler Z=0"
#define UI_TEXT_ZCALIB_FR             "Z Calib."
#define UI_TEXT_SET_P1_FR            "Regler P1"
#define UI_TEXT_SET_P2_FR            "Regler P2"
#define UI_TEXT_SET_P3_FR            "Regler P3"
#define UI_TEXT_CALCULATE_LEVELING_FR "Calibration"
#define UI_TEXT_LEVEL_FR             "Niveau delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_FR  "Att.temp:%XT\002C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_FR "Att. Units:%XUmm"
#define UI_TEXT_SD_REMOVED_FR       "Carte SD retiree"
#define UI_TEXT_SD_INSERTED_FR      "Carte SD inseree"
#define UI_TEXT_PRINTER_READY_FR    "Imprimante prete"
#define UI_TEXT_PRINTTIME_DAYS_FR      " jours "
#define UI_TEXT_PRINTTIME_HOURS_FR     ":"
#define UI_TEXT_PRINTTIME_MINUTES_FR   ""
#define UI_TEXT_PRINT_TIME_FR          "Temps impression"
#define UI_TEXT_PRINT_FILAMENT_FR      "Filament imprime"
#define UI_TEXT_PRINTED_FR           "Imprime"
#define UI_TEXT_POWER_FR               "Alimentation"
#define UI_TEXT_STRING_HM_DEADTIME_FR  "   PWM"
#define UI_TEXT_STRING_HM_SLOWBANG_FR  "   TOR"
#define UI_TEXT_STOP_PRINT_FR          "Arret impress."
#define UI_TEXT_Z_BABYSTEPPING_FR "Z PaP. :%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_FR "Changement filament"
#define UI_TEXT_WIZ_CH_FILAMENT1_FR "Changement filament"
#define UI_TEXT_WIZ_CH_FILAMENT2_FR "Tournez deplacer"
#define UI_TEXT_WIZ_CH_FILAMENT3_FR "Filament haut/bas"
#define UI_TEXT_CLICK_DONE_FR "Continuer avec Clic"
#define UI_TEXT_AUTOLEVEL_ONOFF_FR  "Calib. Auto:%ll"
#define UI_TEXT_SERVOPOS_FR "Pos. Servo: %oS"
#define UI_TEXT_IGNORE_M106_FR      "Ignorer M106 %Fi"
#define UI_TEXT_WIZ_REHEAT1_FR "Cliquez pour"
#define UI_TEXT_WIZ_REHEAT2_FR "chauffer extrud."
#define UI_TEXT_WIZ_WAITTEMP1_FR "Attendez temp."
#define UI_TEXT_WIZ_WAITTEMP2_FR "desirees ..."
#define UI_TEXT_EXTRUDER_JAM_FR "Ext. bloque"
#define UI_TEXT_STANDBY_FR "Attente"
#define UI_TEXT_BED_COATING_FR "Revetement"
#define UI_TEXT_BED_COATING_SET1_FR "Revetement :"
#define UI_TEXT_BED_COATING_SET2_FR ""
#define UI_TEXT_NOCOATING_FR "Aucun"
#define UI_TEXT_BUILDTAK_FR "BuildTak"
#define UI_TEXT_KAPTON_FR "Kapton"
#define UI_TEXT_BLUETAPE_FR "Blue Tape"
#define UI_TEXT_PETTAPE_FR "Ruban vert PET"
#define UI_TEXT_GLUESTICK_FR "Baton de Colle"
#define UI_TEXT_CUSTOM_FR "Special"
#define UI_TEXT_COATING_CUSTOM_FR "Special:%BCmm"
#define UI_TEXT_LANGUAGE_FR "Langue"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_FR "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_FR "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_FR "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_FR "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_FR "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_FR "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_FR "Copies: %ed    Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_FR "Debit:\xfd %of%%%   Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_FR "Vitesse: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_FR "Tampon: %oB"
#define UI_TEXT_MAINPAGE6_6_FR "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_FR cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_FR "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_FR "Z:%x2  Debit: %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_FR "Vitesse: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_FR "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_FR "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_FR "%Uf m"
#define UI_TEXT_METER_PRINTED_FR "%Uf m " UI_TEXT_PRINTED_FR
#define UI_TEXT_STATUS_FR "%os"
#define UI_TEXT_EMPTY_FR ""
#define UI_TEXT_TEMP_SET_FR cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_FR cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_FR " %BCmm"
#define UI_TEXT_EXTR3_TEMP_FR "Temp. 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_FR "Temp. 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_FR "Temp. 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_FR "Extr. 4 eteint"
#define UI_TEXT_EXTR4_OFF_FR "Extr. 5 eteint"
#define UI_TEXT_EXTR5_OFF_FR "Extr. 6 eteint"
#define UI_TEXT_EXTR3_SELECT_FR "%X3 Select. Extr. 4"
#define UI_TEXT_EXTR4_SELECT_FR "%X4 Select. Extr. 5"
#define UI_TEXT_EXTR5_SELECT_FR "%X5 Select. Extr. 6"
#define UI_TEXT_DITTO_0_FR "%D0 Aucune copie"
#define UI_TEXT_DITTO_1_FR "%D1 1 Copie"
#define UI_TEXT_DITTO_2_FR "%D2 2 Copies"
#define UI_TEXT_DITTO_3_FR "%D3 3 Copies"
#define UI_TEXT_ZPROBE_HEIGHT_FR "Sonde Z:%zh"
#define UI_TEXT_OFFSETS_FR "Decalage Imp."
#define UI_TEXT_X_OFFSET_FR "Decal. X:%T0mm"
#define UI_TEXT_Y_OFFSET_FR "Decal. Y:%T1mm"
#define UI_TEXT_Z_OFFSET_FR "Decal.Z:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_FR     "Inser. Filament"
#define UI_TEXT_UNLOAD_FILAMENT_FR    "Retir. Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_FR    "Detect.fil: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_FR    "Nettoyage reser"
#define UI_TEXT_CLEAN_DRIPBOX_FR    "Nettoyage reser"
#define UI_TEXT_CLEAN_DRIPBOX_3_FR    "le reservoir"
#define UI_TEXT_EXTR_XOFF_FR    "Deca.X:%Xx"
#define UI_TEXT_EXTR_YOFF_FR    "Deca.Y:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_FR   "Detect.fil: %sf"
#define UI_TEXT_CLEAN_DRIPBOX_FR    "Nettoyage reser"
#define UI_TEXT_CLEANING_DRIPBOX_FR    "Nettoyage reser"
#define UI_TEXT_CLEAN_DRIPBOX_3_FR    "les reservoirs"
#define UI_TEXT_EXTR_XOFF_FR    "Decal. X:%Xx"
#define UI_TEXT_EXTR_YOFF_FR    "Decal. Y:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_FR     "Inserer \005droit "
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_FR     "Retirer \005droit "
#define UI_TEXT_LOAD_LEFT_FILAMENT_FR   "Inserer \005gauche"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_FR    "Retirer \005gauche"
#define UI_TEXT_FEED_LOADING_FILAMENT_FR    "Inserer F:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_FR    "Retirer F:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_FR    "Dist.inser: %fd"
#define UI_TEXT_BED_DOWN_FR    "Plateau en bas"
#define UI_TEXT_WIFI_ONOFF_FR    "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_FR    "Recommencer ?"
#define UI_TEXT_X_LENGTH_FR    "Dist X:  %Lxmm"
#define UI_TEXT_Y_LENGTH_FR   "Dist Y:  %Lymm"
#define UI_TEXT_Z_LENGTH_FR    "Dist Z:  %Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_FR    "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_FR    "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_FR    "T\002PlatABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_FR    "T\002PlatPLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_FR    "L'extrudeur%Ne n'a"
#define UI_TEXT_ALERT_NO_FILAMENTB_FR   "pas de filament!"
#define UI_TEXT_ALERT_NO_FILAMENTC_FR   "Appuyez OK pour"
#define UI_TEXT_ALERT_NO_FILAMENTD_FR    "charger filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_FR    "Tournez 4X les"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_FR    "3 ecrous de"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_FR    "nivellement"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_FR    "sous le lit."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_FR     "Je vais deplacer"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_FR    "l'extrudeur sur"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_FR    "3 positions pour"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_FR    "les reglages"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_FR   "Nous allons sur"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_FR    "chaque positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_FR    "regler 2 ecrous"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_FR    "en meme temps."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_FR    "La buse est a la"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_FR    "bonne hauteur si"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_FR    "une feuille peut"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_FR    "glisser"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_FR    "entre la buse et"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_FR    "le plateau."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_FR    "Prenez une"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_FR    "feuille."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_FR    "Ajustez les"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_FR    "ecrous avants"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_FR    "pour que le"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_FR    "papier glisse."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_FR    "Ajustez les"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_FR    "ecrous arrieres"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_FR    "pour que le"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_FR   "papier glisse."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_FR    "Ajustez les"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_FR    "ecrous de droite"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_FR   "pour que le"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_FR    "papier glisse."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_FR    "Ajustez les"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_FR   "ecrous de gauche"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_FR    "pour que le"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_FR   "papier glisse."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_FR    "Le papier doit"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_FR    "glisser entre"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_FR    "la buse et"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_FR    "le plateau."
#define UI_TEXT_X_MIN_SETTING_FR    "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_FR    "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_FR    "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_FR    "Capot ouvert!"
#define UI_TEXT_TOP_SENSOR_ONOFF_FR   "Capt.capot: %st"
#define UI_TEXT_VERSION_MENU_FR    "Version systeme"
#define UI_TEXT_COOLDOWN_MENU_FR    "Arret chauffe %B3"
#define UI_TEXT_WAIT_OK_FR    "Ok continuer"
#define UI_TEXT_FAILED_FR    "Echoue"
#define UI_TEXT_PLEASE_WAIT_FR    "Attendre..."
#define UI_TEXT_MANUAL_LEVEL_FR    "Calib. Manuelle"
#define UI_TEXT_WARNING_FR    "Attention!!!"
#define UI_TEXT_EXTRUDER_COLD_FR    "E%Ne trop froid!"
#define UI_TEXT_HEAT_EXTRUDER_FR     "Chauffer extru.?"
#define UI_TEXT_WRITE_DEBUG_FR    "Ecrire debug"
#define UI_TEXT_POWER_SAVE_MENU_FR    "Mode eco"
#define UI_TEXT_MAINTENANCE_FR    "Maintenance"
#define UI_TEXT_SETTINGS_FR    "Parametres"
#define UI_TEXT_LOAD_UNLOAD_FR    "Insere/Retire"
#define UI_TEXT_BED_FR    "Plateau"
#define UI_TEXT_BED_OFF_FR    "Plat. eteint %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_FR    "LED eteint    "
#define UI_TEXT_LIGHT_OFF_AFTER2A_FR  "Arret apres:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_FR    "[min] 0=Non"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_FR    "Pos X inconnue !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_FR    "Pos Y inconnue !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_FR    "Pos Z inconnue !"
#define UI_TEXT_MOVING_FR    "Deplacement..."
#define UI_TEXT_PREPARING_FR    "Preparation..."
#define UI_TEXT_KEEP_LIGHT_ON_FR    "LED Impres.:%lk"
#define UI_TEXT_LOADING_FILAMENT_FR    "Insere fil. \005"
#define UI_TEXT_UNLOADING_FILAMENT_FR    "Retire fil. \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_FR    "Insere/Retire"
#define UI_TEXT_AUTOLEVEL_FR    "Auto calib."
#define UI_TEXT_Z_PROBE_FAILED_FR    "Echec calibr."
#define UI_TEXT_SAVE_FR    "Enr.  EEPROM ?"
#define UI_TEXT_ZMIN_FR    "zMin : %zm"
#define UI_TEXT_ZPROBING_FR    "Calibration..."
#define UI_TEXT_ZPOSITION_FR     "En mouvement..."
#define UI_TEXT_DO_YOU_FR    "Voulez-vous:"
#define UI_TEXT_CLEAN1_FR    "Nettoyer la buse"
#define UI_TEXT_CLEAN2_FR   "et les contacts?"
#define UI_TEXT_DISPLAY_MODE_FR    "Mode:%Md"
#define UI_TEXT_EASY_MODE_FR     "   Basique"
#define UI_TEXT_ADVANCED_MODE_FR    "    Expert"
#define UI_TEXT_POWER_SAVE_FR    "Mode Eco:  %Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_FR    "Badge :%lb"
#define UI_TEXT_SOUND_ONOF_FR    "Son:        %so"
#define UI_TEXT_YES_SELECTED_FR    "  [Oui]    Non"
#define UI_TEXT_NO_SELECTED_FR    "   Oui    [Non]"
#define UI_TEXT_PLEASE_CONFIRM_FR    "Confimer SVP"
#define UI_TEXT_CANCEL_ACTION_FR    "Annuler:"
#define UI_TEXT_CONTINUE_ACTION_FR    "Continuer"
#define UI_TEXT_CANCELED_FR    "Action annulee!"
#define UI_TEXT_CONFIRMED_FR    "Action confirmee!"
#define UI_TEXT_CLEAN_NOZZLE_FR    "Nettoyage buse"
#define UI_TEXT_CLEANING_NOZZLE_FR    "Nettoyage buse"
#define UI_TEXT_CLEAN_DRIPBOX_1_FR    "Vous pouvez"
#define UI_TEXT_CLEAN_DRIPBOX_2_FR    "nettoyer"
#define UI_TEXT_PUSH_FILAMENT_FR    "Charger filament"
#define UI_TEXT_WAIT_FILAMENT_FR    "Ok pour demarrer"
#define UI_TEXT_LOAD_FAILSAFE2_FR    "Apres chargement"
#define UI_TEXT_LOAD_FAILSAFE_FR    "R.A.Z. EEPROM "
#define UI_TEXT_HEATING_FR    "Chauffage..."
#define UI_TEXT_WAIT_FOR_OK_FR    "OK pour quitter"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_FR    "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_FR    "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_FR    "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_FR    "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_FR    "Calibration data"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_FR "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_FR "Z:%x2"
#define UI_TEXT_P1_L3_FR "Vit:%om%%%Tmp:%oB"

#define UI_TEXT_P1_L1_EASY_FR "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_FR "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_FR "Z:%x2"

#else
#define UI_TEXT_P1_L1_FR "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_FR "\007%eB/%EbVit:%om%%%"
#define UI_TEXT_P1_L3_FR "Z:%x2"

#define UI_TEXT_P1_L1_EASY_FR "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_FR "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_FR "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_FR "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_FR "Z:%x2"
#define UI_TEXT_P1_L3_FR "Tampon:%oB"

#define UI_TEXT_P1_L1_EASY_FR "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_FR "Z:%x2"
#define UI_TEXT_P1_L3_EASY_FR "Tampon:%oB"
#else
#define UI_TEXT_P1_L1_FR "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_FR "Z:%x2"
#define UI_TEXT_P1_L3_FR "Tampon:%oB"

#define UI_TEXT_P1_L1_EASY_FR "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_FR "Z:%x2"
#define UI_TEXT_P1_L3_EASY_FR "Tampon:%oB"
#endif
#endif
#define  UI_TEXT_MENU_SCANNER_FR    "Numeriser"
#define UI_TEXT_PREPARE_SCANNER_FR    "Prepare table"
#define UI_TEXT_LASER1_ONOFF_FR        "Laser droit:%PA"
#define UI_TEXT_LASER2_ONOFF_FR       "Lasergauche:%PB"
#define UI_TEXT_LED1_ONOFF_FR           "LED  droite:%P1"
#define UI_TEXT_LED2_ONOFF_FR          "LED  gauche:%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_FR    "Vitesse :%PS"
#define UI_TEXT_ROTATION_TABLE_FR     "Rotation +/-"
#define UI_TEXT_RELEASE_TABLE_FR      "Liberer table"
#define UI_TEXT_Z_MIN_CALCULATION_FR      "Calcul de Zmin"



// *************** Czech translation ****************
// ***************      By Majkl     ****************
// version: 2014/08/27


#define UI_TEXT_ON_CZ               "Zap"
#define UI_TEXT_OFF_CZ              "Vyp"
#define UI_TEXT_NA_CZ               "neni" // Output for not available
#define UI_TEXT_YES_CZ              "Ano"
#define UI_TEXT_NO_CZ               "Ne"
#define UI_TEXT_PRINT_POS_CZ        "Tisknu..."
#define UI_TEXT_PRINTING_CZ         "Tisknu"
#define UI_TEXT_IDLE_CZ             "V klidu"
#define UI_TEXT_NOSDCARD_CZ         "Neni SD karta"
#define UI_TEXT_ERROR_CZ            "**** CHYBA ****"
#define UI_TEXT_BACK_CZ             "Zpet \001"
#define UI_TEXT_QUICK_SETTINGS_CZ   "Zakladni nastaveni"
#define UI_TEXT_ERRORMSG_CZ         "%oe"
#define UI_TEXT_CONFIGURATION_CZ    "Nastaveni"
#define UI_TEXT_POSITION_CZ         "Pozice"
#define UI_TEXT_EXTRUDER_CZ         "Extruder"
#define UI_TEXT_SD_CARD_CZ          "SD karta"
#define UI_TEXT_DEBUGGING_CZ        "Debug"
#define UI_TEXT_HOME_DELTA_CZ       "Home delta"
#define UI_TEXT_HOME_ALL_CZ         "Home vsech"
#define UI_TEXT_HOME_X_CZ           "Home X"
#define UI_TEXT_HOME_Y_CZ           "Home Y"
#define UI_TEXT_HOME_Z_CZ           "Home Z"
#define UI_TEXT_PREHEAT_PLA_CZ      "Ohrat pro PLA %B1"
#define UI_TEXT_PREHEAT_ABS_CZ      "Ohrat pro ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_CZ     "Svetla:%lo"
#define UI_TEXT_COOLDOWN_CZ         "Zchladit"
#define UI_TEXT_SET_TO_ORIGIN_CZ    "Nastavit pocatek"
#define UI_TEXT_DISABLE_STEPPER_CZ  "Vypnout motory"
#define UI_TEXT_X_POSITION_CZ       "X pozice"
#define UI_TEXT_X_POS_FAST_CZ       "X rychle"
#define UI_TEXT_Y_POSITION_CZ       "Y pozice"
#define UI_TEXT_Y_POS_FAST_CZ       "Y rychle"
#define UI_TEXT_Z_POSITION_CZ       "Z pozice"
#define UI_TEXT_Z_POS_FAST_CZ       "Z rychle"
#define UI_TEXT_E_POSITION_CZ       "Pozice extruderu"
#define UI_TEXT_BED_TEMP_CZ         "Teplota desky: %eb/%Eb\002C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_CZ         "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_CZ         "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_CZ         "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_CZ         "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_CZ          "Extr. vyp.%B4"
#else
#define UI_TEXT_EXTR0_OFF_CZ          "Extruder 1 vyp.%B4"
#endif
#define UI_TEXT_EXTR1_OFF_CZ          "Extruder 2 vyp.%B5"
#define UI_TEXT_EXTR2_OFF_CZ          "Extruder 3 vyp.%B6"
#define UI_TEXT_EXTR0_SELECT_CZ     "%X0 Zvolit Extr. 1"
#define UI_TEXT_EXTR1_SELECT_CZ     "%X1 Zvolit Extr. 2"
#define UI_TEXT_EXTR2_SELECT_CZ     "%X1 Zvolit Extr. 3"
#define UI_TEXT_EXTR_ORIGIN_CZ      "Nastavit pocatek"
#define UI_TEXT_PRINT_X_CZ          "Tisk X:%ax"
#define UI_TEXT_PRINT_Y_CZ          "Tisk Y:%ay"
#define UI_TEXT_PRINT_Z_CZ          "Tisk Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_CZ    "Tisk:%az"
#define UI_TEXT_MOVE_X_CZ           "Posun X:%aX"
#define UI_TEXT_MOVE_Y_CZ           "Posun Y:%aY"
#define UI_TEXT_MOVE_Z_CZ           "Posun Z:%aZ"
#define UI_TEXT_MOVE_Z_DELTA_CZ     "Posun:%aZ"
#define UI_TEXT_JERK_CZ             "Jerk:%aj"
#define UI_TEXT_ZJERK_CZ            "Z-Jerk:%aJ"
#define UI_TEXT_ACCELERATION_CZ     "Akcelerace"
#define UI_TEXT_STORE_TO_EEPROM_CZ  "Ulozit do EEPROM"
#define UI_TEXT_LOAD_EEPROM_CZ      "Nahrat z EEPROM"
#define UI_TEXT_DBG_ECHO_CZ         "Echo   :%do"
#define UI_TEXT_DBG_INFO_CZ         "Info   :%di"
#define UI_TEXT_DBG_ERROR_CZ        "Chyby :%de"
#define UI_TEXT_DBG_DRYRUN_CZ       "Beh nanecisto:%dd"
#define UI_TEXT_DBG_ENDSTOP_CZ      "EndStop:%dp"
#define UI_TEXT_OPS_OFF_CZ          "%O0 OPS Vypnuto"
#define UI_TEXT_OPS_CLASSIC_CZ      "%O1 OPS Klasicke"
#define UI_TEXT_OPS_FAST_CZ         "%O2 OPS Rychle"
#define UI_TEXT_OPS_RETRACT_CZ      "Retrakce :%Or"
#define UI_TEXT_OPS_BACKSLASH_CZ    "Vule. :%Ob"
#define UI_TEXT_OPS_MINDIST_CZ      "Min.vzd,:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_CZ   "Posunuti po:%Oa"
#define UI_TEXT_ANTI_OOZE_CZ        "Proti kapani"
#define UI_TEXT_PRINT_FILE_CZ       "Tisknout soubor"
#define UI_TEXT_PAUSE_PRINT_CZ      "Pozastavit tisk"
#define UI_TEXT_CONTINUE_PRINT_CZ   "Pokracovani tisku"
#define UI_TEXT_UNMOUNT_CARD_CZ     "Odpojit kartu"
#define UI_TEXT_MOUNT_CARD_CZ       "Pripojit kartu"
#define UI_TEXT_DELETE_FILE_CZ      "Smazat soubor"
#define UI_TEXT_FEEDRATE_CZ         "Rychlost"
#define UI_TEXT_FEED_MAX_X_CZ       "Max X:%fx"
#define UI_TEXT_FEED_MAX_Y_CZ       "Max Y:%fy"
#define UI_TEXT_FEED_MAX_Z_CZ       "Max Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_CZ "Max:%fz"
#define UI_TEXT_FEED_HOME_X_CZ      "Home X:%fX"
#define UI_TEXT_FEED_HOME_Y_CZ      "Home Y:%fY"
#define UI_TEXT_FEED_HOME_Z_CZ      "Home Z:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_CZ "Home:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_CZ "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_CZ "Min koncak:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_CZ "Max koncak:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_CZ ""
#define UI_TEXT_ACTION_YPOSITION4A_CZ "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_CZ "Min koncak:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_CZ "Max koncak:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_CZ ""
#define UI_TEXT_ACTION_ZPOSITION4A_CZ "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_CZ "Min koncak:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_CZ "Max koncak:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_CZ ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_CZ "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_CZ "Min koncak:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_CZ "Max koncak:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_CZ ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_CZ "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_CZ "Min koncak:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_CZ "Max koncak:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_CZ ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_CZ "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_CZ "Min koncak:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_CZ "Max koncak:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_CZ ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_CZ "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_CZ "1 kliknuti = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_CZ "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_CZ "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_CZ "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_CZ "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_CZ "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_CZ "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_CZ "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_CZ "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_CZ "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_CZ "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_CZ "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_CZ "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_CZ          "Rychlost vetraku"
#define UI_TEXT_ACTION_FANSPEED_CZ   "Aktualni rychlost:%Fs%%%"
#define UI_TEXT_FAN_OFF_CZ           "Vypnout"
#define UI_TEXT_FAN_25_CZ            "Vetrak na 25%%%"
#define UI_TEXT_FAN_50_CZ            "Vetrak na 50%%%"
#define UI_TEXT_FAN_75_CZ            "Vetrak na 75%%%"
#define UI_TEXT_FAN_FULL_CZ          "Vetrak na plno"
#define UI_TEXT_STEPPER_INACTIVE_CZ  "Neaktivni motory"
#define UI_TEXT_STEPPER_INACTIVE2A_CZ "Vypnout po :%i m"
#define UI_TEXT_STEPPER_INACTIVE2B_CZ "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_CZ    "Max. neaktivni"
#define UI_TEXT_POWER_INACTIVE2A_CZ   "Vypnout po: %i m"
#define UI_TEXT_POWER_INACTIVE2B_CZ   "[min] 0=Off"
#define UI_TEXT_GENERAL_CZ           "Zakladni"
#define UI_TEXT_BAUDRATE_CZ          "Rychlost:%oc"
#define UI_TEXT_EXTR_STEPS_CZ        "kroku/MM:%Se"
#define UI_TEXT_EXTR_START_FEED_CZ   "Start FR:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_CZ     "Max FR:%XF"
#define UI_TEXT_EXTR_ACCEL_CZ        "Zrychl.:%XA"
#define UI_TEXT_EXTR_WATCH_CZ        "Stab.cas:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_CZ    "Rozsir. lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_CZ    "Rozsir. quad:%Xa"
#define UI_TEXT_EXTR_MANAGER_CZ      "Control:%Xh"
#define UI_TEXT_EXTR_PGAIN_CZ        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_CZ     "Mrtva doba:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_CZ      "Ovladani PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_CZ        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_CZ        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_CZ         "Drive Min:%Xm"
#define UI_TEXT_EXTR_DMAX_CZ         "Drive Max:%XM"
#define UI_TEXT_EXTR_PMAX_CZ         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_CZ "BangBang"
#define UI_TEXT_STRING_HM_PID_CZ     "PID"
#define UI_TEXT_STRING_ACTION_CZ     "Akce:%la"
#define UI_TEXT_HEATING_EXTRUDER_CZ  "Ohrivani extruderu%Ne"
#define UI_TEXT_HEATING_BED_CZ       "Ohrivani desky"
#define UI_TEXT_KILLED_CZ            "Zastaveno"
#define UI_TEXT_STEPPER_DISABLED_CZ  "Motor vypnut"
#define UI_TEXT_EEPROM_STOREDA_CZ     "Konfigurace"
#define UI_TEXT_EEPROM_STOREDB_CZ     "Ulozena v EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_CZ     "Konfigurace"
#define UI_TEXT_EEPROM_LOADEDB_CZ     "Nactena z EEPROM"
#define UI_TEXT_UPLOADING_CZ         "Nahravam..."
#define UI_TEXT_PAGE_BUFFER_CZ       "Buffer:%oB"
#define UI_TEXT_PAGE_EXTRUDER_CZ     " E:%ec/%Ec\002C\176%oC"
#define UI_TEXT_PAGE_EXTRUDER1_CZ     "E1:%e0/%E0\002C\176%o0"
#define UI_TEXT_PAGE_EXTRUDER2_CZ     "E2:%e1/%E1\002C\176%o1"
#define UI_TEXT_PAGE_EXTRUDER3_CZ     "E3:%e2/%E2\002C\176%o2"
#define UI_TEXT_PAGE_BED_CZ          " B:%eb/%Eb\002C\176%ob"
#define UI_TEXT_SPEED_MULTIPLY_CZ    "Rychlost:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_CZ     "Flow nasobit:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_CZ  "Ukazat merení"
#define UI_TEXT_RESET_MEASUREMENT_CZ "Obnovit mereni"
#define UI_TEXT_SET_MEASURED_ORIGIN_CZ "Set Z=0"
#define UI_TEXT_ZCALIB_CZ             "Z kalib."
#define UI_TEXT_SET_P1_CZ            "Set P1"
#define UI_TEXT_SET_P2_CZ            "Set P2"
#define UI_TEXT_SET_P3_CZ            "Set P3"
#define UI_TEXT_CALCULATE_LEVELING_CZ "Vypocitat leveling"
#define UI_TEXT_LEVEL_CZ             "Level delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_CZ  "Tepl. cekani%XT\002C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_CZ "Jedn. cekani:%XUmm"
#define UI_TEXT_SD_REMOVED_CZ       "SD karta vyjmuta."
#define UI_TEXT_SD_INSERTED_CZ      "Vlozena SD karta"
#define UI_TEXT_PRINTER_READY_CZ    "Tiskarna OK"
#define UI_TEXT_PRINTTIME_DAYS_CZ   " dnu "
#define UI_TEXT_PRINTTIME_HOURS_CZ  ":"
#define UI_TEXT_PRINTTIME_MINUTES_CZ ""
#define UI_TEXT_PRINT_TIME_CZ     "Cas tisku"
#define UI_TEXT_PRINT_FILAMENT_CZ "Filament tisteny"
#define UI_TEXT_PRINTED_CZ           "Protisknuto"
#define UI_TEXT_POWER_CZ            "Zapnout ATX zdroj"
#define UI_TEXT_STRING_HM_DEADTIME_CZ     "Dead Time"
#define UI_TEXT_STRING_HM_SLOWBANG_CZ     "SlowBang"
#define UI_TEXT_STOP_PRINT_CZ "Zastavit tisk"
#define UI_TEXT_Z_BABYSTEPPING_CZ "Z babystep.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_CZ "Zmena filament"
#define UI_TEXT_WIZ_CH_FILAMENT1_CZ "Zmena filament"
#define UI_TEXT_WIZ_CH_FILAMENT2_CZ "Otocit se pohybovat"
#define UI_TEXT_WIZ_CH_FILAMENT3_CZ "vlaken nahoru/dolu"
#define UI_TEXT_CLICK_DONE_CZ "Pokr. s cvaknutim"
#define UI_TEXT_AUTOLEVEL_ONOFF_CZ  "Autolevel: %ll"
#define UI_TEXT_SERVOPOS_CZ "Servo pozice: %oS"
#define UI_TEXT_IGNORE_M106_CZ      "Ignorovat M106 %Fi"
#define UI_TEXT_WIZ_REHEAT1_CZ "Klikněte ohrat"
#define UI_TEXT_WIZ_REHEAT2_CZ "extruders."
#define UI_TEXT_WIZ_WAITTEMP1_CZ "Pockejte na cilove"
#define UI_TEXT_WIZ_WAITTEMP2_CZ "teploty ..."
#define UI_TEXT_EXTRUDER_JAM_CZ "Extruder jam"
#define UI_TEXT_STANDBY_CZ "Standby"
#define UI_TEXT_BED_COATING_CZ "Postel nater"
#define UI_TEXT_BED_COATING_SET1_CZ "Postel nater:"
#define UI_TEXT_BED_COATING_SET2_CZ ""
#define UI_TEXT_NOCOATING_CZ "No nater"
#define UI_TEXT_BUILDTAK_CZ "BuildTak"
#define UI_TEXT_KAPTON_CZ "Kapton"
#define UI_TEXT_BLUETAPE_CZ "Modra mask. paska"
#define UI_TEXT_PETTAPE_CZ "Zelena PET paska"
#define UI_TEXT_GLUESTICK_CZ "Lepici tycinka"
#define UI_TEXT_CUSTOM_CZ "Vlastni"
#define UI_TEXT_COATING_CUSTOM_CZ "Vlastni:%BCmm"
#define UI_TEXT_LANGUAGE_CZ "Jazyk"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_CZ "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_CZ "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_CZ "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_CZ "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_CZ "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_CZ "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_CZ "Kopii: %ed     Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_CZ "Flow:\xfd %of%%%   Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_CZ "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_CZ "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_CZ "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_CZ cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_CZ "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_CZ "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_CZ "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_CZ "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_CZ "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_CZ "%Uf m"
#define UI_TEXT_METER_PRINTED_CZ "%Uf m " UI_TEXT_PRINTED_CZ
#define UI_TEXT_STATUS_CZ "%os"
#define UI_TEXT_EMPTY_CZ ""
#define UI_TEXT_TEMP_SET_CZ cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_CZ cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_CZ " %BCmm"
#define UI_TEXT_EXTR3_TEMP_CZ "Teplota 4:%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_CZ "Teplota 5:%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_CZ "Teplota 6:%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_CZ "Extruder 4 vyp."
#define UI_TEXT_EXTR4_OFF_CZ "Extruder 5 vyp."
#define UI_TEXT_EXTR5_OFF_CZ "Extruder 6 vyp."
#define UI_TEXT_EXTR3_SELECT_CZ "%X3 Zvolit Extr. 4"
#define UI_TEXT_EXTR4_SELECT_CZ "%X4 Zvolit Extr. 5"
#define UI_TEXT_EXTR5_SELECT_CZ "%X5 Zvolit Extr. 6"
#define UI_TEXT_DITTO_0_CZ "%D0 Zadne Kopie"
#define UI_TEXT_DITTO_1_CZ "%D1 1 Kopie"
#define UI_TEXT_DITTO_2_CZ "%D2 2 Kopii"
#define UI_TEXT_DITTO_3_CZ "%D3 3 Kopii"
#define UI_TEXT_ZPROBE_HEIGHT_CZ "Vyska z-test:%zh"
#define UI_TEXT_OFFSETS_CZ "Set print offsets"
#define UI_TEXT_X_OFFSET_CZ "Set X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_CZ "Set Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_CZ "Set Z offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_CZ   "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_CZ   "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_CZ   "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_CZ   "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_CZ   "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_CZ   "Dripbox"
#define UI_TEXT_EXTR_XOFF_CZ   "Xoffset:%Xx"
#define UI_TEXT_EXTR_YOFF_CZ   "Yoffset:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_CZ   "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_CZ   "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_CZ   "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_CZ   "Dripboxes"
#define UI_TEXT_EXTR_XOFF_CZ   "Xoffst:%Xx"
#define UI_TEXT_EXTR_YOFF_CZ   "Yoffst:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_CZ   "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_CZ   "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_CZ   "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_CZ   "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_CZ   "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_CZ   "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_CZ   "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_CZ   "Bed Down"
#define UI_TEXT_WIFI_ONOFF_CZ   "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_CZ   "Do it again ?"
#define UI_TEXT_X_LENGTH_CZ   "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_CZ   "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_CZ   "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_CZ   "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_CZ   "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_CZ   "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_CZ   "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_CZ   "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_CZ   "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_CZ   "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_CZ   "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_CZ   "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_CZ   "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_CZ   "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_CZ   "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_CZ   "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_CZ   "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_CZ   "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_CZ   "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_CZ   "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_CZ   "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_CZ   "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_CZ   "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_CZ   "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_CZ   "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_CZ   "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_CZ   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_CZ   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_CZ   "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_CZ   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_CZ   "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_CZ   "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_CZ   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_CZ   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_CZ   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_CZ   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_CZ   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_CZ   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_CZ   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_CZ   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_CZ   "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_CZ   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_CZ   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_CZ   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_CZ   "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_CZ   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_CZ   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_CZ   "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_CZ   "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_CZ   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_CZ   "and plate."
#define UI_TEXT_X_MIN_SETTING_CZ   "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_CZ   "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_CZ   "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_CZ   "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_CZ   "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_CZ   "System Version"
#define UI_TEXT_COOLDOWN_MENU_CZ   "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_CZ   "Ok to continue"
#define UI_TEXT_FAILED_CZ   "Failed"
#define UI_TEXT_PLEASE_WAIT_CZ   "Please wait"
#define UI_TEXT_MANUAL_LEVEL_CZ   "Manual leveling"
#define UI_TEXT_WARNING_CZ   "Warning!"
#define UI_TEXT_EXTRUDER_COLD_CZ   "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_CZ   "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_CZ   "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_CZ   "Powersave"
#define UI_TEXT_MAINTENANCE_CZ   "Maintenance"
#define UI_TEXT_SETTINGS_CZ   "Settings"
#define UI_TEXT_LOAD_UNLOAD_CZ   "Load/Unload"
#define UI_TEXT_BED_CZ   "Bed"
#define UI_TEXT_BED_OFF_CZ   "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_CZ   "Light AutoOff "
#define UI_TEXT_LIGHT_OFF_AFTER2A_CZ   "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_CZ   "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_CZ   "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_CZ   "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_CZ   "Real Z unknown !"
#define UI_TEXT_MOVING_CZ   "Moving..."
#define UI_TEXT_PREPARING_CZ   "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_CZ   "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_CZ   "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_CZ   "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_CZ   "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_CZ   "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_CZ   "Z-probe failed"
#define UI_TEXT_SAVE_CZ   "Save to EEPROM ?"
#define UI_TEXT_ZMIN_CZ   "zMin : %zm"
#define UI_TEXT_ZPROBING_CZ   "Probing..."
#define UI_TEXT_ZPOSITION_CZ   "Go position..."
#define UI_TEXT_DO_YOU_CZ   "Do you want to:"
#define UI_TEXT_CLEAN1_CZ   "Clean nozzle and"
#define UI_TEXT_CLEAN2_CZ   "plates first ?"
#define UI_TEXT_DISPLAY_MODE_CZ   "Mode:%Md"
#define UI_TEXT_EASY_MODE_CZ   "Easy"
#define UI_TEXT_ADVANCED_MODE_CZ   "Advanced"
#define UI_TEXT_POWER_SAVE_CZ   "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_CZ   "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_CZ   "Sound:  %so"
#define UI_TEXT_YES_SELECTED_CZ   "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_CZ   "   Yes    [No]"
#define UI_TEXT_PLEASE_CONFIRM_CZ   "Please confirm"
#define UI_TEXT_CANCEL_ACTION_CZ   "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_CZ   "Continue"
#define UI_TEXT_CANCELED_CZ   "Action canceled!"
#define UI_TEXT_CONFIRMED_CZ   "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_CZ   "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_CZ   "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_CZ   "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_CZ   "And Clean"
#define UI_TEXT_PUSH_FILAMENT_CZ   "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_CZ   "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_CZ   "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_CZ   "Load Fail-Safe"
#define UI_TEXT_HEATING_CZ   "Heating..."
#define UI_TEXT_WAIT_FOR_OK_CZ   "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_CZ    "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_CZ    "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_CZ   "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_CZ   "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_CZ   "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_CZ "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_CZ "Z:%x2"
#define UI_TEXT_P1_L3_CZ "Mul:%om Buf:%oB"

#define UI_TEXT_P1_L1_EASY_CZ "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_CZ "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_CZ "Z:%x2"

#else
#define UI_TEXT_P1_L1_CZ "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_CZ "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_CZ "Z:%x2"

#define UI_TEXT_P1_L1_EASY_CZ "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_CZ "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_CZ "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_CZ "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_CZ "Z:%x2"
#define UI_TEXT_P1_L3_CZ "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_CZ "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_CZ "Z:%x2"
#define UI_TEXT_P1_L3_EASY_CZ "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_CZ "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_CZ "Z:%x2"
#define UI_TEXT_P1_L3_CZ "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_CZ "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_CZ "Z:%x2"
#define UI_TEXT_P1_L3_EASY_CZ "Buffer:%oB"
#endif
#endif
#define UI_TEXT_MENU_SCANNER_CZ    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_CZ    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_CZ        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_CZ       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_CZ           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_CZ          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_CZ    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_CZ     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_CZ      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_CZ      "Zmin evaluation"



// *************** Polish translation ****************
// ***************       By MIS       ****************
// version: 2015/01/18

#define UI_TEXT_ON_PL               "Zal"
#define UI_TEXT_OFF_PL              "Wyl"
#define UI_TEXT_NA_PL               "Brak" // Output for not available
#define UI_TEXT_YES_PL              "Tak"
#define UI_TEXT_NO_PL               "Nie"
#define UI_TEXT_PRINT_POS_PL        "Drukowanie..."
#define UI_TEXT_PRINTING_PL         "Drukowanie"
#define UI_TEXT_IDLE_PL             "Wolna"
#define UI_TEXT_NOSDCARD_PL         "Brak karty SD"
#define UI_TEXT_ERROR_PL            "**** BLAD ****"
#define UI_TEXT_BACK_PL             "Powrot " cUP
#define UI_TEXT_QUICK_SETTINGS_PL   "Szybkie ustawienia"
#define UI_TEXT_ERRORMSG_PL         "%oe"
#define UI_TEXT_CONFIGURATION_PL    "Konfiguracja"
#define UI_TEXT_POSITION_PL         "Pozycja"
#define UI_TEXT_EXTRUDER_PL         "Ekstruder"
#define UI_TEXT_SD_CARD_PL          "Karta SD"
#define UI_TEXT_DEBUGGING_PL        "Testowanie"
#define UI_TEXT_HOME_DELTA_PL       "Home delta"
#define UI_TEXT_HOME_ALL_PL         "Zeruj wszystkie osie"
#define UI_TEXT_HOME_X_PL           "Zeruj X"
#define UI_TEXT_HOME_Y_PL           "Zeruj Y"
#define UI_TEXT_HOME_Z_PL           "Zeruj Z"
#define UI_TEXT_PREHEAT_PLA_PL      "Rozgrzej PLA %B1"
#define UI_TEXT_PREHEAT_ABS_PL      "Rozgrzej ABS %B2"
#define UI_TEXT_LIGHTS_ONOFF_PL     "Oswietlenie :%lo"
#define UI_TEXT_COOLDOWN_PL         "Chlodzenie"
#define UI_TEXT_SET_TO_ORIGIN_PL    "Ustaw jako zero"
#define UI_TEXT_DISABLE_STEPPER_PL  "Wylacz silniki"
#define UI_TEXT_X_POSITION_PL       "Pozycja X"
#define UI_TEXT_X_POS_FAST_PL       "Pozycja X Szybko"
#define UI_TEXT_Y_POSITION_PL       "Pozycja Y"
#define UI_TEXT_Y_POS_FAST_PL       "Pozycja Y Szybko"
#define UI_TEXT_Z_POSITION_PL       "Pozycja Z"
#define UI_TEXT_Z_POS_FAST_PL       "Pozycja Z Szybko"
#define UI_TEXT_E_POSITION_PL       "Pozycja Extrudera"
#define UI_TEXT_BED_TEMP_PL         "Temp.Stolu:%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_PL         "Ext :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_PL         "Ext1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_PL         "Ext2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_PL         "Ext3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_PL          "Wyl. Extruder %B4"
#else
#define UI_TEXT_EXTR0_OFF_PL          "Wyl. Extruder 1%B4"
#endif
#define UI_TEXT_EXTR1_OFF_PL          "Wyl. Extruder 2%B5"
#define UI_TEXT_EXTR2_OFF_PL          "Wyl. Extruder 3%B6"
#define UI_TEXT_EXTR0_SELECT_PL     "%X0 Wybierz Extr. 1"
#define UI_TEXT_EXTR1_SELECT_PL     "%X1 Wybierz Extr. 2"
#define UI_TEXT_EXTR2_SELECT_PL     "%X1 Wybierz Extr. 3"
#define UI_TEXT_EXTR_ORIGIN_PL      "Ustaw jako zero"
#define UI_TEXT_PRINT_X_PL          "Drukow. X   :%ax"
#define UI_TEXT_PRINT_Y_PL          "Drukow. Y   :%ay"
#define UI_TEXT_PRINT_Z_PL          "Drukow. Z   :%az"
#define UI_TEXT_PRINT_Z_DELTA_PL    "Drukowania  :%az"
#define UI_TEXT_MOVE_X_PL           "Przesuw. X  :%aX"
#define UI_TEXT_MOVE_Y_PL           "Przesuw. Y  :%aY"
#define UI_TEXT_MOVE_Z_PL           "Przesuw. Z  :%aZ"
#define UI_TEXT_MOVE_Z_DELTA_PL     "Przesuwania :%aZ"
#define UI_TEXT_JERK_PL             "Jerk:%aj"
#define UI_TEXT_ZJERK_PL            "Z-Jerk:%aJ"
#define UI_TEXT_ACCELERATION_PL     "Przyspieszenie"
#define UI_TEXT_STORE_TO_EEPROM_PL  "Zapisz do EEPROM"
#define UI_TEXT_LOAD_EEPROM_PL      "Odczyt z EEPROM"
#define UI_TEXT_DBG_ECHO_PL         "Echo        : %do"
#define UI_TEXT_DBG_INFO_PL         "Informacje  : %di"
#define UI_TEXT_DBG_ERROR_PL        "Bledy       : %de"
#define UI_TEXT_DBG_DRYRUN_PL       "Bez wydruku : %dd"
#define UI_TEXT_DBG_ENDSTOP_PL      "EndStop     : %dp"
#define UI_TEXT_OPS_OFF_PL          "%O0 OPS Off"
#define UI_TEXT_OPS_CLASSIC_PL      "%O1 OPS Classic"
#define UI_TEXT_OPS_FAST_PL         "%O2 OPS Fast"
#define UI_TEXT_OPS_RETRACT_PL      "Retract :%Or"
#define UI_TEXT_OPS_BACKSLASH_PL    "Backsl. :%Ob"
#define UI_TEXT_OPS_MINDIST_PL      "Min.dist:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_PL   "Przesun po:%Oa"
#define UI_TEXT_ANTI_OOZE_PL        "Anti Ooze"
#define UI_TEXT_PRINT_FILE_PL       "Drukuj z pliku"
#define UI_TEXT_PAUSE_PRINT_PL      "Wstrzymaj druk"
#define UI_TEXT_CONTINUE_PRINT_PL   "Wznow drukow."
#define UI_TEXT_UNMOUNT_CARD_PL     "Odmontuj karte"
#define UI_TEXT_MOUNT_CARD_PL       "Zamontuj karte"
#define UI_TEXT_DELETE_FILE_PL      "Usun plik"
#define UI_TEXT_FEEDRATE_PL         "Szybkosc"
#define UI_TEXT_FEED_MAX_X_PL       "Max X:%fx"
#define UI_TEXT_FEED_MAX_Y_PL       "Max Y:%fy"
#define UI_TEXT_FEED_MAX_Z_PL       "Max Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_PL "Max:%fz"
#define UI_TEXT_FEED_HOME_X_PL      "Powrot X:%fX"
#define UI_TEXT_FEED_HOME_Y_PL      "Powrot Y:%fY"
#define UI_TEXT_FEED_HOME_Z_PL      "Powrot Z:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_PL "Powrot:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_PL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION4B_PL "Krancowka MIN: %sx"
#define UI_TEXT_ACTION_XPOSITION4C_PL "Krancowka MAX: %sX"
#define UI_TEXT_ACTION_XPOSITION4D_PL ""
#define UI_TEXT_ACTION_YPOSITION4A_PL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION4B_PL "Krancowka MIN: %sy"
#define UI_TEXT_ACTION_YPOSITION4C_PL "Krancowka Max: %sY"
#define UI_TEXT_ACTION_YPOSITION4D_PL ""
#define UI_TEXT_ACTION_ZPOSITION4A_PL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION4B_PL "Krancowka MIN: %sz"
#define UI_TEXT_ACTION_ZPOSITION4C_PL "Krancowka Max: %sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_PL ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_PL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_PL "Krancowka MIN: %sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_PL "Krancowka MAX: %sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_PL ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_PL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_PL "Krancowka MIN: %sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_PL "Krancowka MAX: %sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_PL ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_PL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_PL "Krancowka MIN: %sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_PL "Krancowka MAX: %sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_PL ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_PL "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_PL "Jednostka 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_PL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_PL "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_PL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_PL "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_PL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_PL "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_PL "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_PL "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_PL "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_PL "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_PL "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_PL "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_PL          "Obroty wiatraka"
#define UI_TEXT_ACTION_FANSPEED_PL   "Obroty wiatraka:%Fs%%%"
#define UI_TEXT_FAN_OFF_PL           "Wylacz wiatrak"
#define UI_TEXT_FAN_25_PL            "Ustaw na 25%%%"
#define UI_TEXT_FAN_50_PL            "Ustaw na 50%%%"
#define UI_TEXT_FAN_75_PL            "Ustaw na 75%%%"
#define UI_TEXT_FAN_FULL_PL          "Ustaw na 100%%%"
#define UI_TEXT_STEPPER_INACTIVE_PL  "Wylacz silniki po"
#define UI_TEXT_STEPPER_INACTIVE2A_PL "Wylacz po: %is min."
#define UI_TEXT_STEPPER_INACTIVE2B_PL "0=Nie wylaczaj"
#define UI_TEXT_POWER_INACTIVE_PL    "Wylacz zasil. po"
#define UI_TEXT_POWER_INACTIVE2A_PL   "Wylacz po: %ip min."
#define UI_TEXT_POWER_INACTIVE2B_PL   "0=Nie wylaczaj"
#define UI_TEXT_GENERAL_PL           "Ustawienia glowne"
#define UI_TEXT_BAUDRATE_PL          "Szybkosc USB:%oc"
#define UI_TEXT_EXTR_STEPS_PL        "Kroki/mm : %Se"
#define UI_TEXT_EXTR_START_FEED_PL   "Poczatkowa SZ:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_PL     "Max Szybkosc :%XF"
#define UI_TEXT_EXTR_ACCEL_PL        "Przyspiesz.  :%XA"
#define UI_TEXT_EXTR_WATCH_PL        "Czas stabil. :%Xws"
#define UI_TEXT_EXTR_ADVANCE_L_PL    "Advance lin  :%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_PL    "Advance quad :%Xa"
#define UI_TEXT_EXTR_MANAGER_PL      "Regulator:%Xh"
#define UI_TEXT_EXTR_DEADTIME_PL     "Deadtime :%Xp"
#define UI_TEXT_EXTR_DMAX_DT_PL      "Max. PWM :%XD"
#define UI_TEXT_EXTR_PGAIN_PL        "PID P    :%Xp"
#define UI_TEXT_EXTR_IGAIN_PL        "PID I    :%Xi"
#define UI_TEXT_EXTR_DGAIN_PL        "PID D    :%Xd"
#define UI_TEXT_EXTR_DMIN_PL         "Min I war: %Xm"
#define UI_TEXT_EXTR_DMAX_PL         "Max I war: %XM"
#define UI_TEXT_EXTR_PMAX_PL         "Max ster.: %XD"
#define UI_TEXT_STRING_HM_BANGBANG_PL "Dwustanowy"
#define UI_TEXT_STRING_HM_PID_PL     "PID"
#define UI_TEXT_STRING_ACTION_PL     "Akcja:%la"
#define UI_TEXT_HEATING_EXTRUDER_PL  "Grzanie glowicy%Ne"
#define UI_TEXT_HEATING_BED_PL       "Grzanie stolu"
#define UI_TEXT_KILLED_PL            "Zatrzymany"
#define UI_TEXT_STEPPER_DISABLED_PL  "Silniki wylaczone"
#define UI_TEXT_EEPROM_STOREDA_PL     "Konfiguracja"
#define UI_TEXT_EEPROM_STOREDB_PL     "zapisana do EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_PL     "Konfiguracja"
#define UI_TEXT_EEPROM_LOADEDB_PL     "odczytana z EEPROM"
#define UI_TEXT_UPLOADING_PL         "Ladowanie..."
#define UI_TEXT_PAGE_BUFFER_PL       "Bufor:%oB"
#define UI_TEXT_PAGE_EXTRUDER_PL     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_PL     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_PL     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_PL     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_PL          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_PL    "Szybkosc druku :%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_PL     "Przeplyw filam.:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_PL  "Pokaz pomiar"
#define UI_TEXT_RESET_MEASUREMENT_PL "Zeruj pomiar"
#define UI_TEXT_SET_MEASURED_ORIGIN_PL "Ustaw Z=0"
#define UI_TEXT_ZCALIB_PL             "Z kalib."
#define UI_TEXT_SET_P1_PL            "Ustaw P1"
#define UI_TEXT_SET_P2_PL            "Ustaw P2"
#define UI_TEXT_SET_P3_PL            "Ustaw P3"
#define UI_TEXT_CALCULATE_LEVELING_PL "Oblicz poziomowania"
#define UI_TEXT_LEVEL_PL             "Poziom delty"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_PL  "MinTemp wysuwu:%XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_PL "Wysuw rozgrzew:%XUmm"
#define UI_TEXT_SD_REMOVED_PL       "Karta SD wyjeta"
#define UI_TEXT_SD_INSERTED_PL      "Karta SD wlozona"
#define UI_TEXT_PRINTER_READY_PL    "Drukarka gotowa"
#define UI_TEXT_PRINTTIME_DAYS_PL   " dni, "
#define UI_TEXT_PRINTTIME_HOURS_PL  ":"
#define UI_TEXT_PRINTTIME_MINUTES_PL " h."
#define UI_TEXT_PRINT_TIME_PL     "Czas pracy"
#define UI_TEXT_PRINT_FILAMENT_PL "Zuzyto filamentu"
#define UI_TEXT_PRINTED_PL           "drukowane"
#define UI_TEXT_POWER_PL            "Wl/Wyl zasilanie"
#define UI_TEXT_STRING_HM_DEADTIME_PL     "Dead Time"
#define UI_TEXT_STRING_HM_SLOWBANG_PL     "SlowBang"
#define UI_TEXT_STOP_PRINT_PL       "Przerwij wydruk"
#define UI_TEXT_Z_BABYSTEPPING_PL   "Doreguluj Z:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_PL  "Zmiana filamentu"
#define UI_TEXT_WIZ_CH_FILAMENT1_PL "Zmien filament"
#define UI_TEXT_WIZ_CH_FILAMENT2_PL "Krec aby przesuwac"
#define UI_TEXT_WIZ_CH_FILAMENT3_PL "filament w gore/dol"
#define UI_TEXT_CLICK_DONE_PL       "Kliknij jak gotowe"
#define UI_TEXT_AUTOLEVEL_ONOFF_PL  "Autopoziomow.: %ll"
#define UI_TEXT_SERVOPOS_PL         "Pozycja Serwa: %oS"
#define UI_TEXT_IGNORE_M106_PL      "Ignoruj kom. M106 %Fi"
#define UI_TEXT_WIZ_REHEAT1_PL "Kliknij aby znów"
#define UI_TEXT_WIZ_REHEAT2_PL "rozgrzac ekstrudery"
#define UI_TEXT_WIZ_WAITTEMP1_PL "Poczekaj na zadane"
#define UI_TEXT_WIZ_WAITTEMP2_PL "temperatury ..."
#define UI_TEXT_EXTRUDER_JAM_PL "Ekstruder zablokowany"
#define UI_TEXT_STANDBY_PL "Standby"
#define UI_TEXT_BED_COATING_PL "Pokrycie stolu"
#define UI_TEXT_BED_COATING_SET1_PL "Wybierz material"
#define UI_TEXT_BED_COATING_SET2_PL "stolu:"
#define UI_TEXT_NOCOATING_PL "Czysty stol"
#define UI_TEXT_BUILDTAK_PL "BuildTak"
#define UI_TEXT_KAPTON_PL "Kapton"
#define UI_TEXT_BLUETAPE_PL "Nieb. tasma mask."
#define UI_TEXT_PETTAPE_PL "Zielona tasma PET"
#define UI_TEXT_GLUESTICK_PL "Klej w sztyfcie"
#define UI_TEXT_CUSTOM_PL "Inna"
#define UI_TEXT_COATING_CUSTOM_PL "Inna grubosc:%BCmm"
#define UI_TEXT_LANGUAGE_PL "Jezyk"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_PL "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_PL "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_PL "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_PL "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_PL "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_PL "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_PL "Kopie: %ed     Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_PL "Przep:\xfd %of%%%  Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_PL "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_PL "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_PL "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_PL cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_PL "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_PL "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_PL "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_PL "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_PL "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_PL "%Uf m"
#define UI_TEXT_METER_PRINTED_PL "%Uf m " UI_TEXT_PRINTED_PL
#define UI_TEXT_STATUS_PL "%os"
#define UI_TEXT_EMPTY_PL ""
#define UI_TEXT_TEMP_SET_PL cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_PL cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_PL " %BCmm"
#define UI_TEXT_EXTR3_TEMP_PL "Temp.Ex4:%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_PL "Temp.Ex5:%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_PL "Temp.Ex6:%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_PL "Wyl. Extruder 4"
#define UI_TEXT_EXTR4_OFF_PL "Wyl. Extruder 5"
#define UI_TEXT_EXTR5_OFF_PL "Wyl. Extruder 6"
#define UI_TEXT_EXTR3_SELECT_PL "%X3 Wybierz Extr. 4"
#define UI_TEXT_EXTR4_SELECT_PL "%X4 Wybierz Extr. 5"
#define UI_TEXT_EXTR5_SELECT_PL "%X5 Wybierz Extr. 6"
#define UI_TEXT_DITTO_0_PL "%D0 Nie kopiuj"
#define UI_TEXT_DITTO_1_PL "%D1 1 Kopia"
#define UI_TEXT_DITTO_2_PL "%D2 2 Kopie"
#define UI_TEXT_DITTO_3_PL "%D3 3 Kopie"
#define UI_TEXT_ZPROBE_HEIGHT_PL "Wys. Sondy Z:%zh"
#define UI_TEXT_OFFSETS_PL  "Polozenie wydruku"
#define UI_TEXT_X_OFFSET_PL "Przesun w X : %T0mm"
#define UI_TEXT_Y_OFFSET_PL "Przesun w Y : %T1mm"
#define UI_TEXT_Z_OFFSET_PL "Przesun w Z :%T2mm"
#define UI_TEXT_LOAD_FILAMENT_PL   "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_PL   "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_PL   "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_PL   "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_PL   "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_PL   "Dripbox"
#define UI_TEXT_EXTR_XOFF_PL   "Xoffset:%Xx"
#define UI_TEXT_EXTR_YOFF_PL   "Yoffset:%Xy"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_PL   "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_PL   "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_PL   "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_PL   "Dripboxes"
#define UI_TEXT_EXTR_XOFF_PL   "Xoffst:%Xx"
#define UI_TEXT_EXTR_YOFF_PL   "Yoffst:%Xy"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_PL   "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_PL   "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_PL   "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_PL   "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_PL   "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_PL   "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_PL   "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_PL   "Bed Down"
#define UI_TEXT_WIFI_ONOFF_PL   "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_PL   "Do it again ?"
#define UI_TEXT_X_LENGTH_PL   "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_PL   "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_PL   "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_PL   "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_PL   "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_PL   "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_PL   "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_PL   "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_PL   "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_PL   "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_PL   "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_PL   "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_PL   "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_PL   "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_PL   "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_PL   "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_PL   "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_PL   "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_PL   "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_PL   "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_PL   "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_PL   "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_PL   "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_PL   "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_PL   "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_PL   "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_PL   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_PL   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_PL   "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_PL   "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_PL   "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_PL   "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_PL   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_PL   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_PL   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_PL   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_PL   "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_PL   "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_PL   "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_PL   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_PL   "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_PL   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_PL   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_PL   "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_PL   "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_PL   "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_PL   "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_PL   "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_PL   "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_PL   "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_PL   "and plate."
#define UI_TEXT_X_MIN_SETTING_PL   "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_PL   "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_PL   "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_PL   "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_PL   "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_PL   "System Version"
#define UI_TEXT_COOLDOWN_MENU_PL   "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_PL   "Ok to continue"
#define UI_TEXT_FAILED_PL   "Failed"
#define UI_TEXT_PLEASE_WAIT_PL   "Please wait"
#define UI_TEXT_MANUAL_LEVEL_PL   "Manual leveling"
#define UI_TEXT_WARNING_PL   "Warning!"
#define UI_TEXT_EXTRUDER_COLD_PL   "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_PL   "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_PL   "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_PL   "Powersave"
#define UI_TEXT_MAINTENANCE_PL   "Maintenance"
#define UI_TEXT_SETTINGS_PL   "Settings"
#define UI_TEXT_LOAD_UNLOAD_PL   "Load/Unload"
#define UI_TEXT_BED_PL   "Bed"
#define UI_TEXT_BED_OFF_PL   "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_PL   "Light AutoOff "
#define UI_TEXT_LIGHT_OFF_AFTER2A_PL   "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_PL   "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_PL   "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_PL   "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_PL   "Real Z unknown !"
#define UI_TEXT_MOVING_PL   "Moving..."
#define UI_TEXT_PREPARING_PL   "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_PL   "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_PL   "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_PL   "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_PL   "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_PL   "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_PL   "Z-probe failed"
#define UI_TEXT_SAVE_PL   "Save to EEPROM ?"
#define UI_TEXT_ZMIN_PL   "zMin : %zm"
#define UI_TEXT_ZPROBING_PL   "Probing..."
#define UI_TEXT_ZPOSITION_PL   "Go position..."
#define UI_TEXT_DO_YOU_PL   "Do you want to:"
#define UI_TEXT_CLEAN1_PL   "Clean nozzle and"
#define UI_TEXT_CLEAN2_PL   "plates first ?"
#define UI_TEXT_DISPLAY_MODE_PL   "Mode:%Md"
#define UI_TEXT_EASY_MODE_PL   "Easy"
#define UI_TEXT_ADVANCED_MODE_PL   "Advanced"
#define UI_TEXT_POWER_SAVE_PL   "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_PL   "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_PL   "Sound:  %so"
#define UI_TEXT_YES_SELECTED_PL   "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_PL   "   Yes    [No]"
#define UI_TEXT_PLEASE_CONFIRM_PL   "Please confirm"
#define UI_TEXT_CANCEL_ACTION_PL   "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_PL   "Continue"
#define UI_TEXT_CANCELED_PL   "Action canceled!"
#define UI_TEXT_CONFIRMED_PL   "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_PL   "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_PL   "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_PL   "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_PL   "And Clean"
#define UI_TEXT_PUSH_FILAMENT_PL   "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_PL   "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_PL   "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_PL   "Load Fail-Safe"
#define UI_TEXT_HEATING_PL   "Heating..."
#define UI_TEXT_WAIT_FOR_OK_PL   "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_PL    "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_PL    "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_PL   "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_PL   "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_PL   "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_PL "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_PL "Z:%x2"
#define UI_TEXT_P1_L3_PL "Mul:%om Buf:%oB"

#define UI_TEXT_P1_L1_EASY_PL "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_PL "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_PL "Z:%x2"

#else
#define UI_TEXT_P1_L1_PL "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_PL "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_PL "Z:%x2"

#define UI_TEXT_P1_L1_EASY_PL "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_PL "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_PL "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_PL "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_PL "Z:%x2"
#define UI_TEXT_P1_L3_PL "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_PL "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_PL "Z:%x2"
#define UI_TEXT_P1_L3_EASY_PL "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_PL "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_PL "Z:%x2"
#define UI_TEXT_P1_L3_PL "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_PL "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_PL "Z:%x2"
#define UI_TEXT_P1_L3_EASY_PL "Buffer:%oB"
#endif
#endif
#define UI_TEXT_MENU_SCANNER_PL    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_PL    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_PL        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_PL       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_PL           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_PL          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_PL    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_PL     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_PL      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_PL      "Zmin evaluation"

// Türk

#define UI_TEXT_ON_TR               "Acik"
#define UI_TEXT_OFF_TR              "Kapali"
#define UI_TEXT_NA_TR               "N/A" // Output for not available
#define UI_TEXT_YES_TR              "Evet"
#define UI_TEXT_NO_TR               "Hayir"
#define UI_TEXT_PRINT_POS_TR        "Yaziyor..."
#define UI_TEXT_PRINTING_TR         "Yaziyor"
#define UI_TEXT_IDLE_TR             "Bosta"
#define UI_TEXT_NOSDCARD_TR         "SD Kart Yok"
#define UI_TEXT_ERROR_TR            "**** HATA ****"
#define UI_TEXT_BACK_TR             "Geri " cUP
#define UI_TEXT_QUICK_SETTINGS_TR   "Hizli Ayar"
#define UI_TEXT_ERRORMSG_TR         "%oe"
#define UI_TEXT_CONFIGURATION_TR    "Yapilandirma"
#define UI_TEXT_POSITION_TR         "Konum-Hareket"
#define UI_TEXT_EXTRUDER_TR         "Extruder"
#define UI_TEXT_SD_CARD_TR          "SD Kart"
#define UI_TEXT_DEBUGGING_TR        "Debugging"
#define UI_TEXT_HOME_DELTA_TR       "Deltayi Sifr. Gond."
#define UI_TEXT_HOME_ALL_TR         "Hepsini S.Gonder"
#define UI_TEXT_HOME_X_TR           "X S.Gonder"
#define UI_TEXT_HOME_Y_TR           "Y S.Gonder"
#define UI_TEXT_HOME_Z_TR           "Z S.Gonder"
#define UI_TEXT_PREHEAT_PLA_TR      "PLA On Isitma"
#define UI_TEXT_PREHEAT_ABS_TR      "ABS On Isitma"
#define UI_TEXT_LIGHTS_ONOFF_TR     "Isiklar :%lo"
#define UI_TEXT_COOLDOWN_TR         "Soguma"
#define UI_TEXT_SET_TO_ORIGIN_TR    "Orijine ayarla"
#define UI_TEXT_DISABLE_STEPPER_TR  "Motoru Kapat"
#define UI_TEXT_X_POSITION_TR       "X Hareketi"
#define UI_TEXT_X_POS_FAST_TR       "X Hizli Hareketi"
#define UI_TEXT_Y_POSITION_TR       "Y Hareketi"
#define UI_TEXT_Y_POS_FAST_TR       "Y Hizli Hareketi"
#define UI_TEXT_Z_POSITION_TR       "Z Hareketi"
#define UI_TEXT_Z_POS_FAST_TR       "Z Hizli Hareketi"
#define UI_TEXT_E_POSITION_TR       "Hareketi"
#define UI_TEXT_BED_TEMP_TR         "Tabla Sic: %Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_TR       "Sic :%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_TR       "Sic1 :%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_TR       "Sic2 :%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_TR       "Sic3 :%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_TR        "Extr. Kapa%B4"
#else
#define UI_TEXT_EXTR0_OFF_TR       "Extr. 1'i Kapa%B4"
#endif
#define UI_TEXT_EXTR1_OFF_TR        "Extr. 2'i Kapa%B5"
#define UI_TEXT_EXTR2_OFF_TR        "Extr. 3'i Kapa%B6"
#define UI_TEXT_EXTR0_SELECT_TR     "%X0 Extr. 1'i Sec"
#define UI_TEXT_EXTR1_SELECT_TR     "%X1 Extr. 2'yi Sec"
#define UI_TEXT_EXTR2_SELECT_TR     "%X2 Extr. 3'u Sec"
#define UI_TEXT_EXTR_ORIGIN_TR      "Origin i Ayarla"
#define UI_TEXT_PRINT_X_TR          "Yazma H. X:%ax"
#define UI_TEXT_PRINT_Y_TR          "Yazma H. Y:%ay"
#define UI_TEXT_PRINT_Z_TR          "Yazma H. Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_TR    "Yazma H.:%az"
#define UI_TEXT_MOVE_X_TR           "Hareket X :%aX"
#define UI_TEXT_MOVE_Y_TR           "Hareket Y :%aY"
#define UI_TEXT_MOVE_Z_TR           "Hareket Z :%aZ"
#define UI_TEXT_MOVE_Z_DELTA_TR     "Hareket :%aZ"
#define UI_TEXT_JERK_TR             "X-Y Jerk :%aj"
#define UI_TEXT_ZJERK_TR            "Z-Jerk :%aJ"
#define UI_TEXT_ACCELERATION_TR     "Ivme"
#define UI_TEXT_STORE_TO_EEPROM_TR  "EEPROM'a kaydet"
#define UI_TEXT_LOAD_EEPROM_TR      "EEPROM'dan cagir"
#define UI_TEXT_DBG_ECHO_TR         "Echo    :%do"
#define UI_TEXT_DBG_INFO_TR         "Bilgi   :%di"
#define UI_TEXT_DBG_ERROR_TR        "Hatalar :%de"
#define UI_TEXT_DBG_DRYRUN_TR       "Bosta Calis:%dd"
#define UI_TEXT_DBG_ENDSTOP_TR      "EndStop :%dp"
#define UI_TEXT_OPS_OFF_TR          "%O0 OPS Kapali"
#define UI_TEXT_OPS_CLASSIC_TR      "%O1 OPS Klasik"
#define UI_TEXT_OPS_FAST_TR         "%O2 OPS Hizli"
#define UI_TEXT_OPS_RETRACT_TR      "Geri Cekl.:%Or"
#define UI_TEXT_OPS_BACKSLASH_TR    "Eksen Bosl.:%Ob"
#define UI_TEXT_OPS_MINDIST_TR      "Min. Mesafe:%Od"
#define UI_TEXT_OPS_MOVE_AFTER_TR   "Move after:%Oa"
#define UI_TEXT_ANTI_OOZE_TR        "Anti Ooze"
#define UI_TEXT_PRINT_FILE_TR       "Dosya Yazdir"
#define UI_TEXT_PAUSE_PRINT_TR      "Baskiyi Duraklat"
#define UI_TEXT_CONTINUE_PRINT_TR   "Baskiya Devam"
#define UI_TEXT_UNMOUNT_CARD_TR     "Karti Cikar"
#define UI_TEXT_MOUNT_CARD_TR       "Karti Bagla"
#define UI_TEXT_DELETE_FILE_TR      "Dosyayi Sil"
#define UI_TEXT_FEEDRATE_TR         "Ilerleme"
#define UI_TEXT_FEED_MAX_X_TR       "Max X:%fx"
#define UI_TEXT_FEED_MAX_Y_TR       "Max Y:%fy"
#define UI_TEXT_FEED_MAX_Z_TR       "Max Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_TR "Max:%fz"
#define UI_TEXT_FEED_HOME_X_TR      "X'i Sif.Gon.Hiz.:%fX"
#define UI_TEXT_FEED_HOME_Y_TR      "Y'yi Sif.Gon.Hiz.:%fY"
#define UI_TEXT_FEED_HOME_Z_TR      "Z'i Sif.Gon.Hiz:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_TR "Delta Sif.Gon.Hiz:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_TR "X:%x0 mm %dx%dX"
#define UI_TEXT_ACTION_XPOSITION4B_TR "Min endstop:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_TR "Max endstop:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_TR ""
#define UI_TEXT_ACTION_YPOSITION4A_TR "Y:%x1 mm %dy%dY"
#define UI_TEXT_ACTION_YPOSITION4B_TR "Min endstop:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_TR "Max endstop:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_TR ""
#define UI_TEXT_ACTION_ZPOSITION4A_TR "Z:%x2 mm %dz%dZ"
#define UI_TEXT_ACTION_ZPOSITION4B_TR "Min endstop:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_TR "Max endstop:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_TR ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_TR "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_TR "Min endstop:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_TR "Max endstop:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_TR ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_TR "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_TR "Min endstop:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_TR "Max endstop:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_TR ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_TR "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_TR "Min endstop:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_TR "Max endstop:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_TR ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_TR "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_TR "1 tik = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_TR "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_TR "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_TR "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_TR "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_TR "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_TR "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_TR "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_TR "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_TR "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_TR "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_TR "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_TR "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_TR          "Fan Hizi"
#define UI_TEXT_ACTION_FANSPEED_TR   "Fan Hizi:%Fs%%%"
#define UI_TEXT_FAN_OFF_TR           "Fan'i Kapat"
#define UI_TEXT_FAN_25_TR            "Fan'i Ayarla 25%%%"
#define UI_TEXT_FAN_50_TR            "Fan'i Ayarla 50%%%"
#define UI_TEXT_FAN_75_TR            "Fan'i Ayarla 75%%%"
#define UI_TEXT_FAN_FULL_TR          "Fan Tam Acik"
#define UI_TEXT_STEPPER_INACTIVE_TR  "Motorlar Bosta"
#define UI_TEXT_STEPPER_INACTIVE2A_TR "Motor Kapa: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_TR "[min] 0=Off"
#define UI_TEXT_POWER_INACTIVE_TR    "Max. Bosta"
#define UI_TEXT_POWER_INACTIVE2A_TR   "Kapama sur.: %ip"
#define UI_TEXT_POWER_INACTIVE2B_TR    "[min] 0=Kapali"
#define UI_TEXT_GENERAL_TR           "Genel"
#define UI_TEXT_BAUDRATE_TR          "Baudrate:%oc"
#define UI_TEXT_EXTR_STEPS_TR        "Adim/MM:%Se"
#define UI_TEXT_EXTR_START_FEED_TR   "Baslangic FR:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_TR     "Max FR:%XF"
#define UI_TEXT_EXTR_ACCEL_TR        "Ivme:%XA"
#define UI_TEXT_EXTR_WATCH_TR        "Stab.Time:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_TR    "Gelismis lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_TR    "Gelismis quad:%Xa"
#define UI_TEXT_EXTR_MANAGER_TR      "Kontrol:%Xh"
#define UI_TEXT_EXTR_PGAIN_TR        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_TR     "Bos Zamn:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_TR      "Kontrol PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_TR        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_TR        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_TR         "Surucu Min:%Xm"
#define UI_TEXT_EXTR_DMAX_TR         "Surucu Max:%XM"
#define UI_TEXT_EXTR_PMAX_TR         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_TR "BangBang"
#define UI_TEXT_STRING_HM_PID_TR     "PID"
#define UI_TEXT_STRING_ACTION_TR     "Eylem:%la"
#define UI_TEXT_HEATING_EXTRUDER_TR  "Extruder Isiniyor"
#define UI_TEXT_HEATING_BED_TR       "Bed Isiniyor"
#define UI_TEXT_KILLED_TR            "DURDU"
#define UI_TEXT_STEPPER_DISABLED_TR  "Motorlar Kapali"
#define UI_TEXT_EEPROM_STOREDA_TR     "Konfigurasyon"
#define UI_TEXT_EEPROM_STOREDB_TR     "EEPROM'a kaydedildi"
#define UI_TEXT_EEPROM_LOADEDA_TR     "Konfigurasyon"
#define UI_TEXT_EEPROM_LOADEDB_TR     "EEPROM'dan cagrildi"
#define UI_TEXT_UPLOADING_TR         "Yukluyor..."
#define UI_TEXT_PAGE_BUFFER_TR       "Onbellek:%oB"
#define UI_TEXT_PAGE_EXTRUDER_TR     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_TR     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_TR     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_TR     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_TR          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_TR    "Hiz Carp.:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_TR     "Akis Carp. :%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_TR  "Olcumu Goster"
#define UI_TEXT_RESET_MEASUREMENT_TR "Olcumu Sifirla"
#define UI_TEXT_SET_MEASURED_ORIGIN_TR "Z=0'a ayarla"
#define UI_TEXT_ZCALIB_TR             "Z Kalib."
#define UI_TEXT_SET_P1_TR            "Set P1"
#define UI_TEXT_SET_P2_TR            "Set P2"
#define UI_TEXT_SET_P3_TR            "Set P3"
#define UI_TEXT_CALCULATE_LEVELING_TR "Sevieyi Hesapla"
#define UI_TEXT_LEVEL_TR             "Deltayi seviyele"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_TR "Bekle Sic. %XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_TR "Bekle: %XU mm"
#define UI_TEXT_SD_REMOVED_TR       "SD Card Cikarildi"
#define UI_TEXT_SD_INSERTED_TR      "SD Card Takildi"
#define UI_TEXT_PRINTER_READY_TR    "Yazici Hazir"
#define UI_TEXT_PRINTTIME_DAYS_TR   " gun "
#define UI_TEXT_PRINTTIME_HOURS_TR  ":"
#define UI_TEXT_PRINTTIME_MINUTES_TR ""
#define UI_TEXT_PRINT_TIME_TR     "Yazdirma Zamani"
#define UI_TEXT_PRINT_FILAMENT_TR "Filament harcandi"
#define UI_TEXT_PRINTED_TR           "yazildi"
#define UI_TEXT_POWER_TR            "ATX gucu on/off"
#define UI_TEXT_STRING_HM_DEADTIME_TR     "Bosta Zaman"
#define UI_TEXT_STRING_HM_SLOWBANG_TR     "SlowBang"
#define UI_TEXT_STOP_PRINT_TR "Yazdirmayi durdur"
#define UI_TEXT_Z_BABYSTEPPING_TR "Z Babystep.:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_TR "Filament degistir"
#define UI_TEXT_WIZ_CH_FILAMENT1_TR "Filament degistir"
#define UI_TEXT_WIZ_CH_FILAMENT2_TR "Filameti yukari asagi"
#define UI_TEXT_WIZ_CH_FILAMENT3_TR "almak icin cevirin"
#define UI_TEXT_CLICK_DONE_TR "Tamamlandiginda butona basin"
#define UI_TEXT_AUTOLEVEL_ONOFF_TR  "Oto seviye: %ll"
#define UI_TEXT_SERVOPOS_TR "Servo pos.: %oS"
#define UI_TEXT_IGNORE_M106_TR    "M106Yi atla %Fi"
#define UI_TEXT_WIZ_REHEAT1_TR "Extruderlari tekrar"
#define UI_TEXT_WIZ_REHEAT2_TR "isitmak icin tiklayin"
#define UI_TEXT_WIZ_WAITTEMP1_TR "Istnilen sicakliklar"
#define UI_TEXT_WIZ_WAITTEMP2_TR "icin bekleyin"
#define UI_TEXT_EXTRUDER_JAM_TR "Extruder takildi"
#define UI_TEXT_STANDBY_TR "Bekleme Modu"
#define UI_TEXT_BED_COATING_TR "Tabla ustu kaplama"
#define UI_TEXT_BED_COATING_SET1_TR "Kaplama Kalinligi"
#define UI_TEXT_BED_COATING_SET2_TR ""
#define UI_TEXT_NOCOATING_TR "Kaplama Yok"
#define UI_TEXT_BUILDTAK_TR "BuildTak"
#define UI_TEXT_KAPTON_TR "Kapton"
#define UI_TEXT_BLUETAPE_TR "Mavi Boya Kagidi"
#define UI_TEXT_PETTAPE_TR "Yesil PET Tape"
#define UI_TEXT_GLUESTICK_TR "Yapiskan"
#define UI_TEXT_CUSTOM_TR "Diger"
#define UI_TEXT_COATING_CUSTOM_TR "Diger : %oCmm"
#define UI_TEXT_LANGUAGE_TR "Dil"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_TR "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_TR "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_TR "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_TR "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_TR "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_TR "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_TR "Kopie: %ed     Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_TR "Przep:\xfd %of%%%  Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_TR "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_TR "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_TR "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_TR cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_TR "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_TR "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_TR "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_TR "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_TR "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_TR "%Uf m"
#define UI_TEXT_METER_PRINTED_TR "%Uf m " UI_TEXT_PRINTED_TR
#define UI_TEXT_STATUS_TR "%os"
#define UI_TEXT_EMPTY_TR ""
#define UI_TEXT_TEMP_SET_TR cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_TR cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_TR " %oCmm"
#define UI_TEXT_EXTR3_TEMP_TR       "Sic. 4 : %E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_TR       "Sic. 5 : %E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_TR       "Sic. 6 : %E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_TR "Ext. 4’u kapat"
#define UI_TEXT_EXTR4_OFF_TR " Ext. 5’i kapat "
#define UI_TEXT_EXTR5_OFF_TR " Ext. 6’yi kapat "
#define UI_TEXT_EXTR3_SELECT_TR "%X3 Ext. 4’u sec "
#define UI_TEXT_EXTR4_SELECT_TR "%X4 Ext. 5’i sec "
#define UI_TEXT_EXTR5_SELECT_TR "%X5 Ext. 6’yi sec "
#define UI_TEXT_DITTO_0_TR "%D0 Kopyalama"
#define UI_TEXT_DITTO_1_TR "%D1 1 kopya"
#define UI_TEXT_DITTO_2_TR "%D2 2 kopya"
#define UI_TEXT_DITTO_3_TR "%D3 3 kopya"
#define UI_TEXT_ZPROBE_HEIGHT_TR "Z-prob yuksekligi:%zh"
#define UI_TEXT_OFFSETS_TR "Set print offsets"
#define UI_TEXT_X_OFFSET_TR "Set X offset:%T0mm"
#define UI_TEXT_Y_OFFSET_TR "Set Y offset:%T1mm"
#define UI_TEXT_Z_OFFSET_TR "Set Z offset:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_TR "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_TR "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_TR "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_TR "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_TR "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_TR "Dripbox"
#define UI_TEXT_EXTR_XOFF_TR "Xoffst:%Xxmm"
#define UI_TEXT_EXTR_YOFF_TR "Yoffst:%Xymm"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_TR "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_TR "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_TR "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_TR "Dripboxes"
#define UI_TEXT_EXTR_XOFF_TR "Xoffst:%Xxmm"
#define UI_TEXT_EXTR_YOFF_TR "Yoffst:%Xymm"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_TR "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_TR "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_TR "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_TR "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_TR "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_TR "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_TR "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_TR "Bed Down"
#define UI_TEXT_WIFI_ONOFF_TR "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_TR "Do it again ?"
#define UI_TEXT_X_LENGTH_TR "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_TR "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_TR "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_TR "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_TR "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_TR "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_TR "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_TR "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_TR "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_TR "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_TR "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_TR "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_TR "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_TR "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_TR "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_TR "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_TR "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_TR "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_TR "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_TR "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_TR "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_TR "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_TR "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_TR "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_TR "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_TR "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_TR "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_TR "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_TR "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_TR "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_TR "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_TR "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_TR "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_TR "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_TR "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_TR "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_TR "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_TR "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_TR "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_TR "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_TR "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_TR "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_TR "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_TR "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_TR "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_TR "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_TR "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_TR "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_TR "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_TR "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_TR "and plate."
#define UI_TEXT_X_MIN_SETTING_TR "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_TR "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_TR "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_TR "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_TR "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_TR "System Version"
#define UI_TEXT_COOLDOWN_MENU_TR "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_TR "Ok to continue"
#define UI_TEXT_FAILED_TR "Failed"
#define UI_TEXT_PLEASE_WAIT_TR "Please wait"
#define UI_TEXT_MANUAL_LEVEL_TR "Manual leveling"
#define UI_TEXT_WARNING_TR "Warning!"
#define UI_TEXT_EXTRUDER_COLD_TR "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_TR "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_TR "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_TR "Powersave"
#define UI_TEXT_MAINTENANCE_TR "Maintenance"
#define UI_TEXT_SETTINGS_TR "Settings"
#define UI_TEXT_LOAD_UNLOAD_TR "Load/Unload"
#define UI_TEXT_BED_TR "Bed"
#define UI_TEXT_BED_OFF_TR "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_TR "Light AutoOff"
#define UI_TEXT_LIGHT_OFF_AFTER2A_TR "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_TR "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_TR "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_TR "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_TR "Real Z unknown !"
#define UI_TEXT_MOVING_TR "Moving..."
#define UI_TEXT_PREPARING_TR "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_TR "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_TR "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_TR "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_TR "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_TR "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_TR "Z-probe failed"
#define UI_TEXT_SAVE_TR "Save to EEPROM ?"
#define UI_TEXT_ZMIN_TR "zMin : %zm"
#define UI_TEXT_ZPROBING_TR "Probing..."
#define UI_TEXT_ZPOSITION_TR "Go position..."
#define UI_TEXT_DO_YOU_TR "Do you want to:"
#define UI_TEXT_CLEAN1_TR "Clean nozzle and"
#define UI_TEXT_CLEAN2_TR "plates first ?"
#define UI_TEXT_DISPLAY_MODE_TR "Mode:%Md"
#define UI_TEXT_EASY_MODE_TR "      Easy"
#define UI_TEXT_ADVANCED_MODE_TR "  Advanced"
#define UI_TEXT_POWER_SAVE_TR "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_TR "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_TR "Sound:      %so"
#define UI_TEXT_YES_SELECTED_TR "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_TR "   Yes    [No] "
#define UI_TEXT_PLEASE_CONFIRM_TR "Please confirm"
#define UI_TEXT_CANCEL_ACTION_TR "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_TR "Continue"
#define UI_TEXT_CANCELED_TR "Action canceled!"
#define UI_TEXT_CONFIRMED_TR "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_TR "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_TR "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_TR "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_TR "And Clean"
#define UI_TEXT_PUSH_FILAMENT_TR "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_TR "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_TR "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_TR "Load Fail-Safe"
#define UI_TEXT_HEATING_TR "Heating..."
#define UI_TEXT_WAIT_FOR_OK_TR "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_TR  "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_TR  "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_TR "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_TR "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_TR "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_TR "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_TR "Z:%x2"
#define UI_TEXT_P1_L3_TR "Mul:%om%%%Buf:%oB"

#define UI_TEXT_P1_L1_EASY_TR "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_TR "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_TR "Z:%x2"

#else
#define UI_TEXT_P1_L1_TR "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_TR "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_TR "Z:%x2"

#define UI_TEXT_P1_L1_EASY_TR "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_TR "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_TR "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_TR "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_TR "Z:%x2"
#define UI_TEXT_P1_L3_TR "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_TR "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_TR "Z:%x2"
#define UI_TEXT_P1_L3_EASY_TR "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_TR "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_TR "Z:%x2"
#define UI_TEXT_P1_L3_TR "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_TR "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_TR "Z:%x2"
#define UI_TEXT_P1_L3_EASY_TR "Buffer:%oB"
#endif
#endif
#define UI_TEXT_MENU_SCANNER_TR    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_TR    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_TR        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_TR       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_TR           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_TR          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_TR    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_TR     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_TR      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_TR      "Zmin evaluation"

// *************** Finnish translation ****************
// ********** By Matti Granstedt 28.01.2016 ***********

#define UI_TEXT_ON_FI               "P" STR_auml STR_auml "lle"
#define UI_TEXT_OFF_FI              "Sammuta"
#define UI_TEXT_NA_FI               "N/A" // Output for not available
#define UI_TEXT_YES_FI              "Kyll" STR_auml ""
#define UI_TEXT_NO_FI               "Ei"
#define UI_TEXT_PRINT_POS_FI        "Tulostaa..."
#define UI_TEXT_PRINTING_FI         "Tulosta"
#define UI_TEXT_IDLE_FI             "Valmis"
#define UI_TEXT_NOSDCARD_FI         "Ei SD korttia"
#define UI_TEXT_ERROR_FI            "**** VIRHE ****"
#define UI_TEXT_BACK_FI             "Takaisin" cUP
#define UI_TEXT_QUICK_SETTINGS_FI   "Pika-asetukset"
#define UI_TEXT_ERRORMSG_FI         "%oe"
#define UI_TEXT_CONFIGURATION_FI    "Kokoonpano"
#define UI_TEXT_POSITION_FI         "Paikka"
#define UI_TEXT_EXTRUDER_FI         "Pursotin"
#define UI_TEXT_SD_CARD_FI          "SD kortti"
#define UI_TEXT_DEBUGGING_FI        "Virheenetsint" STR_auml ""
#define UI_TEXT_HOME_DELTA_FI       "Koti delta"
#define UI_TEXT_HOME_ALL_FI         "Koti kaikki"
#define UI_TEXT_HOME_X_FI           "Koti X"
#define UI_TEXT_HOME_Y_FI           "Koti Y"
#define UI_TEXT_HOME_Z_FI           "Koti Z"
#define UI_TEXT_PREHEAT_PLA_FI      "Esilämmitys PLA"
#define UI_TEXT_PREHEAT_ABS_FI      "Esilämmitys ABS"
#define UI_TEXT_LIGHTS_ONOFF_FI     "Valot:%lo"
#define UI_TEXT_COOLDOWN_FI         "J" STR_auml STR_auml "hdytys"
#define UI_TEXT_SET_TO_ORIGIN_FI    "Aseta l" STR_auml "ht" STR_ouml "piste"
#define UI_TEXT_DISABLE_STEPPER_FI  "Sammuta moottori"
#define UI_TEXT_X_POSITION_FI       "X Paikka"
#define UI_TEXT_X_POS_FAST_FI       "X Paikka nopea"
#define UI_TEXT_Y_POSITION_FI       "Y Paikka"
#define UI_TEXT_Y_POS_FAST_FI       "Y Paikka nopea"
#define UI_TEXT_Z_POSITION_FI       "Z Paikka"
#define UI_TEXT_Z_POS_FAST_FI       "Z Paikka nopea"
#define UI_TEXT_E_POSITION_FI       "Pursot. paikka"
#define UI_TEXT_BED_TEMP_FI         "Alustan :%eb/%Eb" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_TEMP_FI       "Purso:%e0/%E0" cDEG "C"
#else
#define UI_TEXT_EXTR0_TEMP_FI       "Purso:%e0/%E0" cDEG "C"
#endif
#define UI_TEXT_EXTR1_TEMP_FI       "Purso:%e1/%E1" cDEG "C"
#define UI_TEXT_EXTR2_TEMP_FI       "Purso:%e2/%E2" cDEG "C"
#if NUM_EXTRUDER == 1
#define UI_TEXT_EXTR0_OFF_FI        "Pursotin Ei%B4"
#else
#define UI_TEXT_EXTR0_OFF_FI        "Pursotin 1 Ei%B4"
#endif
#define UI_TEXT_EXTR1_OFF_FI        "Pursotin 2 Ei%B5"
#define UI_TEXT_EXTR2_OFF_FI        "Pursotin 3 Ei%B6"
#define UI_TEXT_EXTR0_SELECT_FI     "%X0 Valitse purs.1"
#define UI_TEXT_EXTR1_SELECT_FI     "%X1 Valitse purs.2"
#define UI_TEXT_EXTR2_SELECT_FI     "%X2 Valitse purs.3"
#define UI_TEXT_EXTR_ORIGIN_FI      "Aseta l" STR_auml "ht" STR_ouml "piste"
#define UI_TEXT_PRINT_X_FI          "Tulosta X:%ax"
#define UI_TEXT_PRINT_Y_FI          "Tulosta Y:%ay"
#define UI_TEXT_PRINT_Z_FI          "Tulosta Z:%az"
#define UI_TEXT_PRINT_Z_DELTA_FI    "Tulosta:%az"
#define UI_TEXT_MOVE_X_FI           "Siirr" STR_auml "X :%aX"
#define UI_TEXT_MOVE_Y_FI           "Siirr" STR_auml "Y :%aY"
#define UI_TEXT_MOVE_Z_FI           "Siirr" STR_auml "Z :%aZ"
#define UI_TEXT_MOVE_Z_DELTA_FI     "Siirr" STR_auml ":%aZ"
#define UI_TEXT_JERK_FI             "Askel   :%aj"
#define UI_TEXT_ZJERK_FI            "Z-Askel :%aJ"
#define UI_TEXT_ACCELERATION_FI     "Kiihtyvyys"
#define UI_TEXT_STORE_TO_EEPROM_FI  "Tallenna EEPROM"
#define UI_TEXT_LOAD_EEPROM_FI      "Lataa    EEPROM"
#define UI_TEXT_DBG_ECHO_FI         "Toista   :%do"
#define UI_TEXT_DBG_INFO_FI         "Tiedot   :%di"
#define UI_TEXT_DBG_ERROR_FI        "Virheet  :%de"
#define UI_TEXT_DBG_DRYRUN_FI       "Kuiva-ajo:%dd"
#define UI_TEXT_DBG_ENDSTOP_FI      "EndStop  :%dp"
#define UI_TEXT_OPS_OFF_FI          "%O0 OPS ei"
#define UI_TEXT_OPS_CLASSIC_FI      "%O1 OPS normaali"
#define UI_TEXT_OPS_FAST_FI         "%O2 OPS nopea"
#define UI_TEXT_OPS_RETRACT_FI      "Peruuta   :%Or"
#define UI_TEXT_OPS_BACKSLASH_FI    "Backsl.   :%Ob"
#define UI_TEXT_OPS_MINDIST_FI      "Min.dist  :%Od"
#define UI_TEXT_OPS_MOVE_AFTER_FI   "Siirr" STR_auml "j" STR_auml "lkeen:%Oa"
#define UI_TEXT_ANTI_OOZE_FI        "Vuodon poisto"
#define UI_TEXT_PRINT_FILE_FI       "Tulosta Tiedosto"
#define UI_TEXT_PAUSE_PRINT_FI      "Tulostustauko"
#define UI_TEXT_CONTINUE_PRINT_FI   "Jatka tulostusta"
#define UI_TEXT_UNMOUNT_CARD_FI     "Kortti poistettu"
#define UI_TEXT_MOUNT_CARD_FI       "Kortti asetettu"
#define UI_TEXT_DELETE_FILE_FI      "Poista tiedosto"
#define UI_TEXT_FEEDRATE_FI         "Sy" STR_ouml "tt" STR_ouml "nopeus"
#define UI_TEXT_FEED_MAX_X_FI       "Maks. X:%fx"
#define UI_TEXT_FEED_MAX_Y_FI       "Maks. Y:%fy"
#define UI_TEXT_FEED_MAX_Z_FI       "Maks. Z:%fz"
#define UI_TEXT_FEED_MAX_Z_DELTA_FI "Maks.:%fz"
#define UI_TEXT_FEED_HOME_X_FI      "Koti X:%fX"
#define UI_TEXT_FEED_HOME_Y_FI      "Koti Y:%fY"
#define UI_TEXT_FEED_HOME_Z_FI      "Koti Z:%fZ"
#define UI_TEXT_FEED_HOME_Z_DELTA_FI "Koti:%fZ"
#define UI_TEXT_ACTION_XPOSITION4A_FI "X:%x0 mm %dx%dX"
#define UI_TEXT_ACTION_XPOSITION4B_FI "Min. rajoittimesta:%sx"
#define UI_TEXT_ACTION_XPOSITION4C_FI "Maks rajoittimesta:%sX"
#define UI_TEXT_ACTION_XPOSITION4D_FI ""
#define UI_TEXT_ACTION_YPOSITION4A_FI "Y:%x1 mm %dy%dY"
#define UI_TEXT_ACTION_YPOSITION4B_FI "Min. rajoittimesta:%sy"
#define UI_TEXT_ACTION_YPOSITION4C_FI "Maks rajoittimesta:%sY"
#define UI_TEXT_ACTION_YPOSITION4D_FI ""
#define UI_TEXT_ACTION_ZPOSITION4A_FI "Z:%x2 mm %dz%dZ"
#define UI_TEXT_ACTION_ZPOSITION4B_FI "Min. rajoittimesta:%sz"
#define UI_TEXT_ACTION_ZPOSITION4C_FI "Maks rajoittimesta:%sZ"
#define UI_TEXT_ACTION_ZPOSITION4D_FI ""
#define UI_TEXT_ACTION_XPOSITION_FAST4A_FI "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST4B_FI "Min. rajoittimesta:%sx"
#define UI_TEXT_ACTION_XPOSITION_FAST4C_FI "Maks rajoittimesta:%sX"
#define UI_TEXT_ACTION_XPOSITION_FAST4D_FI ""
#define UI_TEXT_ACTION_YPOSITION_FAST4A_FI "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST4B_FI "Min. rajoittimesta:%sy"
#define UI_TEXT_ACTION_YPOSITION_FAST4C_FI "Maks rajoittimesta:%sY"
#define UI_TEXT_ACTION_YPOSITION_FAST4D_FI ""
#define UI_TEXT_ACTION_ZPOSITION_FAST4A_FI "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST4B_FI "Min. rajoittimesta:%sz"
#define UI_TEXT_ACTION_ZPOSITION_FAST4C_FI "Maks rajoittimesta:%sZ"
#define UI_TEXT_ACTION_ZPOSITION_FAST4D_FI ""
#define UI_TEXT_ACTION_EPOSITION_FAST2A_FI "E:%x3 mm"
#define UI_TEXT_ACTION_EPOSITION_FAST2B_FI "1 klikkaus = 1 mm"
#define UI_TEXT_ACTION_XPOSITION2A_FI "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION2B_FI "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION2A_FI "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION2B_FI "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION2A_FI "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION2B_FI "Min:%sz Max:%sZ"
#define UI_TEXT_ACTION_XPOSITION_FAST2A_FI "X:%x0 mm"
#define UI_TEXT_ACTION_XPOSITION_FAST2B_FI "Min:%sx Max:%sX"
#define UI_TEXT_ACTION_YPOSITION_FAST2A_FI "Y:%x1 mm"
#define UI_TEXT_ACTION_YPOSITION_FAST2B_FI "Min:%sy Max:%sY"
#define UI_TEXT_ACTION_ZPOSITION_FAST2A_FI "Z:%x2 mm"
#define UI_TEXT_ACTION_ZPOSITION_FAST2B_FI "Min:%sz Max:%sZ"
#define UI_TEXT_FANSPEED_FI          "Tuulettimen nopeus"
#define UI_TEXT_ACTION_FANSPEED_FI   "Tuuletin nopeus:%Fs%%%"
#define UI_TEXT_FAN_OFF_FI           "Sammuta tuuletin"
#define UI_TEXT_FAN_25_FI            "Aseta nop. 25%%%"
#define UI_TEXT_FAN_50_FI            "Aseta nop. 50%%%"
#define UI_TEXT_FAN_75_FI            "Aseta nop. 75%%%"
#define UI_TEXT_FAN_FULL_FI          "Aseta nop. maks."
#define UI_TEXT_STEPPER_INACTIVE_FI  "Moottori toimeton"
#define UI_TEXT_STEPPER_INACTIVE2A_FI "Toimeton j" STR_auml "lkeen: %is"
#define UI_TEXT_STEPPER_INACTIVE2B_FI "[min] 0=sammuta"
#define UI_TEXT_POWER_INACTIVE_FI    "Maks. toimeton"
#define UI_TEXT_POWER_INACTIVE2A_FI   "Toimeton j" STR_auml "lkeen: %ip"
#define UI_TEXT_POWER_INACTIVE2B_FI   "[min] 0=sammuta"
#define UI_TEXT_GENERAL_FI           "Yleiset"
#define UI_TEXT_BAUDRATE_FI          "Siirtonopeus:%oc"
#define UI_TEXT_EXTR_STEPS_FI        "Askelta/MM:%Se"
#define UI_TEXT_EXTR_START_FEED_FI   "K" STR_auml "ynnist" STR_auml "FR:%Xf"
#define UI_TEXT_EXTR_MAX_FEED_FI     "Maks. FR:%XF"
#define UI_TEXT_EXTR_ACCEL_FI        "Kiihdytys:%XA"
#define UI_TEXT_EXTR_WATCH_FI        "Stab.Aika:%Xw"
#define UI_TEXT_EXTR_ADVANCE_L_FI    "Eteneminen lin:%Xl"
#define UI_TEXT_EXTR_ADVANCE_K_FI    "Eteneminen quad:%Xa"
#define UI_TEXT_EXTR_MANAGER_FI      "Hallinta:%Xh"
#define UI_TEXT_EXTR_PGAIN_FI        "PID P:%Xp"
#define UI_TEXT_EXTR_DEADTIME_FI     "Kuollut aika:%Xp"
#define UI_TEXT_EXTR_DMAX_DT_FI      "Valvonta PWM:%XM"
#define UI_TEXT_EXTR_IGAIN_FI        "PID I:%Xi"
#define UI_TEXT_EXTR_DGAIN_FI        "PID D:%Xd"
#define UI_TEXT_EXTR_DMIN_FI         "Ajo Min:%Xm"
#define UI_TEXT_EXTR_DMAX_FI         "Ajo Max:%XM"
#define UI_TEXT_EXTR_PMAX_FI         "PID Max:%XD"
#define UI_TEXT_STRING_HM_BANGBANG_FI "BangBang"
#define UI_TEXT_STRING_HM_PID_FI     "PID"
#define UI_TEXT_STRING_ACTION_FI     "Toiminta:%la"
#define UI_TEXT_HEATING_EXTRUDER_FI  "Kuumentaa Pursot."
#define UI_TEXT_HEATING_BED_FI       "Kuumentaa Alustaa"
#define UI_TEXT_KILLED_FI            "Poistettu"
#define UI_TEXT_STEPPER_DISABLED_FI  "Moottori pois k" STR_auml "yt."
#define UI_TEXT_EEPROM_STOREDA_FI     "Kokoonpano"
#define UI_TEXT_EEPROM_STOREDB_FI     "Tallennettu EEPROM"
#define UI_TEXT_EEPROM_LOADEDA_FI     "Kokoonpano"
#define UI_TEXT_EEPROM_LOADEDB_FI     "Ladattu EEPROM"
#define UI_TEXT_UPLOADING_FI         "Ladataan..."
#define UI_TEXT_PAGE_BUFFER_FI       "Puskuri:%oB"
#define UI_TEXT_PAGE_EXTRUDER_FI     " E:%ec/%Ec" cDEG "C" cARROW "%oC"
#define UI_TEXT_PAGE_EXTRUDER1_FI     "E1:%e0/%E0" cDEG "C" cARROW "%o0"
#define UI_TEXT_PAGE_EXTRUDER2_FI     "E2:%e1/%E1" cDEG "C" cARROW "%o1"
#define UI_TEXT_PAGE_EXTRUDER3_FI     "E3:%e2/%E2" cDEG "C" cARROW "%o2"
#define UI_TEXT_PAGE_BED_FI          " B:%eb/%Eb" cDEG "C" cARROW "%ob"
#define UI_TEXT_SPEED_MULTIPLY_FI    "Nopeuskerroin:%om%%%"
#define UI_TEXT_FLOW_MULTIPLY_FI     "Puhalluskerroin:%of%%%"
#define UI_TEXT_SHOW_MEASUREMENT_FI  "N" STR_auml "yt" STR_auml " mitat"
#define UI_TEXT_RESET_MEASUREMENT_FI "Poista mitat."
#define UI_TEXT_SET_MEASURED_ORIGIN_FI "Aseta Z=0"
#define UI_TEXT_ZCALIB_FI             "Z kalibrointi"
#define UI_TEXT_SET_P1_FI            "Aseta P1"
#define UI_TEXT_SET_P2_FI            "Aseta P2"
#define UI_TEXT_SET_P3_FI            "Aseta P3"
#define UI_TEXT_CALCULATE_LEVELING_FI "Laske tasot"
#define UI_TEXT_LEVEL_FI             "Taso delta"
#define UI_TEXT_EXTR_WAIT_RETRACT_TEMP_FI "Odottaa l" STR_auml "mm. %XT" cDEG "C"
#define UI_TEXT_EXTR_WAIT_RETRACT_UNITS_FI "Odottaa yks.:%XU mm"
#define UI_TEXT_SD_REMOVED_FI       "SD kortti pois"
#define UI_TEXT_SD_INSERTED_FI      "SD kortti on"
#define UI_TEXT_PRINTER_READY_FI    "Tulostin valmis."
#define UI_TEXT_PRINTTIME_DAYS_FI   " p" STR_auml "iv" STR_auml STR_auml " "
#define UI_TEXT_PRINTTIME_HOURS_FI  ":"
#define UI_TEXT_PRINTTIME_MINUTES_FI ""
#define UI_TEXT_PRINT_TIME_FI     "Tulostusaika"
#define UI_TEXT_PRINT_FILAMENT_FI "Lanka tulostettu"
#define UI_TEXT_PRINTED_FI           "Tulostettu"
#define UI_TEXT_POWER_FI            "ATX virtal. on/ei"
#define UI_TEXT_STRING_HM_DEADTIME_FI     "Joutoaika"
#define UI_TEXT_STRING_HM_SLOWBANG_FI     "SlowBang"
#define UI_TEXT_STOP_PRINT_FI "Lopeta tulostus"
#define UI_TEXT_Z_BABYSTEPPING_FI "Z mikroaskellus:%oYmm"
#define UI_TEXT_CHANGE_FILAMENT_FI "Vaihda tulostuslanka"
#define UI_TEXT_WIZ_CH_FILAMENT1_FI "Vaihda tulostuslanka"
#define UI_TEXT_WIZ_CH_FILAMENT2_FI "Py" STR_ouml "rit" STR_auml " siirr" STR_auml ""
#define UI_TEXT_WIZ_CH_FILAMENT3_FI "Tul.lanka yl" STR_ouml "s/alas"
#define UI_TEXT_CLICK_DONE_FI "Klik kun valmis"
#define UI_TEXT_AUTOLEVEL_ONOFF_FI  "Autom.taso: %ll"
#define UI_TEXT_SERVOPOS_FI "Servo paikka: %oS"
#define UI_TEXT_IGNORE_M106_FI      "Ohita M106 %Fi"
#define UI_TEXT_WIZ_REHEAT1_FI "Klik uusi kuumennus"
#define UI_TEXT_WIZ_REHEAT2_FI "pursottimet"
#define UI_TEXT_WIZ_WAITTEMP1_FI "Odottaa tavoitetta"
#define UI_TEXT_WIZ_WAITTEMP2_FI "l" STR_auml "mp" STR_ouml "tilat..."
#define UI_TEXT_EXTRUDER_JAM_FI "Pursottimen tukos"
#define UI_TEXT_STANDBY_FI "Valmiina"
#define UI_TEXT_BED_COATING_FI "Alustan pinnote"
#define UI_TEXT_BED_COATING_SET1_FI "Alustan pinta valittu"
#define UI_TEXT_BED_COATING_SET2_FI ""
#define UI_TEXT_NOCOATING_FI "Ei pinnotetta"
#define UI_TEXT_BUILDTAK_FI "BuildTak"
#define UI_TEXT_KAPTON_FI "Kapton"
#define UI_TEXT_BLUETAPE_FI "Sininen paperi teippi"
#define UI_TEXT_PETTAPE_FI "Vihreä PET teippi"
#define UI_TEXT_GLUESTICK_FI "Liimapuikko"
#define UI_TEXT_CUSTOM_FI "Mukautettu"
#define UI_TEXT_COATING_CUSTOM_FI "Mukautettu:%BCmm"
#define UI_TEXT_LANGUAGE_FI "Kieli"
#if NUM_EXTRUDER > 2 || MIXING_EXTRUDER != 0
#define UI_TEXT_MAINPAGE6_1_FI "\xa %ec/%Ec\xb0 X:%x0"
#else
#define UI_TEXT_MAINPAGE6_1_FI "\xa %e0/%E0\xb0 X:%x0"
#endif // NUM_EXTRUDER
#if NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_2_FI "\xa %e1/%E1\xb0 Y:%x1"
#elif HAVE_HEATED_BED
#define UI_TEXT_MAINPAGE6_2_FI "\xe %eb/%Eb\xb0 Y:%x1"
#else
#define UI_TEXT_MAINPAGE6_2_FI "             Y:%x1"
#endif
#if HAVE_HEATED_BED && NUM_EXTRUDER == 2 && MIXING_EXTRUDER == 0
#define UI_TEXT_MAINPAGE6_3_FI "\xe %eb/%Eb\xb0 Z:%x2"
#elif FEATURE_DITTO_PRINTING
#define UI_TEXT_MAINPAGE6_3_FI "Kopiot: %ed    Z:%x2"
#else
#define UI_TEXT_MAINPAGE6_3_FI "Virtaus:\xfd %of%%%   Z:%x2"
#endif
#define UI_TEXT_MAINPAGE6_4_FI "Mul: %om%%% \xfd   E: %x4m"
#define UI_TEXT_MAINPAGE6_5_FI "Buf: %oB"
#define UI_TEXT_MAINPAGE6_6_FI "%os"
#define UI_TEXT_MAINPAGE_TEMP_BED_FI cTEMP "%ec/%Ec" cDEG "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_BED_FI "B%eB/%Eb" cDEG
#define UI_TEXT_MAINPAGE_Z_BUF_FI "Z:%x2  Buf : %oB"
#define UI_TEXT_MAINPAGE_MUL_EUSAGE_FI "Mul: %om   E:%x4"
#define UI_TEXT_MAINPAGE_XY_FI "X:%x0 Y:%x1"
#define UI_TEXT_PRINT_TIME_VALUE_FI "%Ut"
#define UI_TEXT_PRINT_FILAMENT_VALUE_FI "%Uf m"
#define UI_TEXT_METER_PRINTED_FI "%Uf m " UI_TEXT_PRINTED_FI
#define UI_TEXT_STATUS_FI "%os"
#define UI_TEXT_EMPTY_FI ""
#define UI_TEXT_TEMP_SET_FI cTEMP "%ec/%Ec" cDEG
#define UI_TEXT_CURRENT_TEMP_FI cTEMP "%ec" cDEG
#define UI_TEXT_COATING_THICKNESS_FI " %BCmm"
#define UI_TEXT_EXTR3_TEMP_FI "L" STR_auml "mp" STR_ouml " 4 :%e3/%E3" cDEG "C"
#define UI_TEXT_EXTR4_TEMP_FI "L" STR_auml "mp" STR_ouml " 5 :%e4/%E4" cDEG "C"
#define UI_TEXT_EXTR5_TEMP_FI "L" STR_auml "mp" STR_ouml " 6 :%e5/%E5" cDEG "C"
#define UI_TEXT_EXTR3_OFF_FI "Pursotin 4 ei"
#define UI_TEXT_EXTR4_OFF_FI "Pursotin 5 ei"
#define UI_TEXT_EXTR5_OFF_FI "Pursotin 6 ei"
#define UI_TEXT_EXTR3_SELECT_FI "%X3 Valitse purs. 4"
#define UI_TEXT_EXTR4_SELECT_FI "%X4 Valitse purs. 5"
#define UI_TEXT_EXTR5_SELECT_FI "%X5 Valitse purs. 6"
#define UI_TEXT_DITTO_0_FI "%D0 Ei kopiota"
#define UI_TEXT_DITTO_1_FI "%D1 1 kopio"
#define UI_TEXT_DITTO_2_FI "%D2 2 kopiota"
#define UI_TEXT_DITTO_3_FI "%D3 3 kopiota"
#define UI_TEXT_ZPROBE_HEIGHT_FI "Z-anturin korkeus:%zh"
#define UI_TEXT_OFFSETS_FI "Aseta tulostimen poikkeamat"
#define UI_TEXT_X_OFFSET_FI "Aseta X poikkeama:%T0mm"
#define UI_TEXT_Y_OFFSET_FI "Aseta Y poikkeama:%T1mm"
#define UI_TEXT_Z_OFFSET_FI "Aseta Z poikkeama:%T2mm"
#define UI_TEXT_LOAD_FILAMENT_FI "Load Filament"
#define UI_TEXT_UNLOAD_FILAMENT_FI "Unload Filament"
#if NUM_EXTRUDER == 1
#define UI_TEXT_FIL_SENSOR_ONOFF_FI "Fil.Sensor: %sf"
#define UI_TEXT_CLEANING_DRIPBOX_FI "Cleaning Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_FI "Clean Dripbox"
#define UI_TEXT_CLEAN_DRIPBOX_3_FI "Dripbox"
#define UI_TEXT_EXTR_XOFF_FI "Xoffst:%Xxmm"
#define UI_TEXT_EXTR_YOFF_FI "Yoffst:%Xymm"
#else
#define UI_TEXT_FIL_SENSOR_ONOFF_FI "Fil.Sensors:%sf"
#define UI_TEXT_CLEAN_DRIPBOX_FI "Clean Dripboxes"
#define UI_TEXT_CLEANING_DRIPBOX_FI "Cleaning Dripboxes"
#define UI_TEXT_CLEAN_DRIPBOX_3_FI "Dripboxes"
#define UI_TEXT_EXTR_XOFF_FI "Xoffst:%Xxmm"
#define UI_TEXT_EXTR_YOFF_FI "Yoffst:%Xymm"
#endif
#define UI_TEXT_LOAD_RIGHT_FILAMENT_FI "Load Right \005"
#define UI_TEXT_UNLOAD_RIGHT_FILAMENT_FI "Unload Right \005"
#define UI_TEXT_LOAD_LEFT_FILAMENT_FI "Load Left \005"
#define UI_TEXT_UNLOAD_LEFT_FILAMENT_FI "Unload Left \005"
#define UI_TEXT_FEED_LOADING_FILAMENT_FI "Load Fil.:%fl"
#define UI_TEXT_FEED_UNLOADING_FILAMENT_FI "Unload F.:%fu"
#define UI_TEXT_LOADING_UNLOADING_DISTANCE_FI "Load dist.: %fd"
#define UI_TEXT_BED_DOWN_FI "Bed Down"
#define UI_TEXT_WIFI_ONOFF_FI "Wifi:       %wo"
#define UI_TEXT_REDO_ACTION_FI "Do it again ?"
#define UI_TEXT_X_LENGTH_FI "X Length:%Lxmm"
#define UI_TEXT_Y_LENGTH_FI "Y Length:%Lymm"
#define UI_TEXT_Z_LENGTH_FI "Z Length:%Lzmm"
#define UI_TEXT_EXT_TEMP_ABS_FI "T\002Ext.ABS:%t1\002C"
#define UI_TEXT_EXT_TEMP_PLA_FI "T\002Ext.PLA:%t2\002C"
#define UI_TEXT_BED_TEMP_ABS_FI "T\002Bed ABS:%t3\002C"
#define UI_TEXT_BED_TEMP_PLA_FI "T\002Bed PLA:%t4\002C"
#define UI_TEXT_ALERT_NO_FILAMENTA_FI "Extruder%Ne has no"
#define UI_TEXT_ALERT_NO_FILAMENTB_FI "filament !"
#define UI_TEXT_ALERT_NO_FILAMENTC_FI "Push Ok to go to"
#define UI_TEXT_ALERT_NO_FILAMENTD_FI "change filament"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1A_FI "Find the 3 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1B_FI "on the bottom of"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1C_FI "platform and"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_1D_FI "tighten 4 turns."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2A_FI "I am going to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2B_FI "move extruder to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2C_FI "some positions"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_2D_FI "for adjustment."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3A_FI "In each position"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3B_FI "we will need to"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3C_FI "adjust 2 knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_3D_FI "at same time."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4A_FI "Nozzle is on the"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4B_FI "right height if"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4C_FI "can just slide"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_4D_FI "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5A_FI "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5B_FI "and plate. Grab"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5C_FI "a sheet of paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_5D_FI "to proceed."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6A_FI "Adjust the front"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6B_FI "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6C_FI "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_6D_FI "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7A_FI "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7B_FI "knob(s) so paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7C_FI "just slides amid"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_7D_FI "nozzle and plate"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8A_FI "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8B_FI "and right knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8C_FI "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_8D_FI "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9A_FI "Adjust the back"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9B_FI "and left knobs"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9C_FI "until paper just"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_9D_FI "slides."
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10A_FI "Check that paper"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10B_FI "just slides"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10C_FI "between nozzle"
#define UI_TEXT_MANUAL_BED_LEVEL_PAGE_10D_FI "and plate."
#define UI_TEXT_X_MIN_SETTING_FI "Xmin: %Hxmm"
#define UI_TEXT_Y_MIN_SETTING_FI "Ymin: %Hymm"
#define UI_TEXT_Z_MIN_SETTING_FI "Zmin: %Hzmm"
#define UI_TEXT_TOP_COVER_OPEN_FI "Top cover open!"
#define UI_TEXT_TOP_SENSOR_ONOFF_FI "Top Sensor: %st"
#define UI_TEXT_VERSION_MENU_FI "System Version"
#define UI_TEXT_COOLDOWN_MENU_FI "Cooldown      %B3"
#define UI_TEXT_WAIT_OK_FI "Ok to continue"
#define UI_TEXT_FAILED_FI "Failed"
#define UI_TEXT_PLEASE_WAIT_FI "Please wait"
#define UI_TEXT_MANUAL_LEVEL_FI "Manual leveling"
#define UI_TEXT_WARNING_FI "Warning!"
#define UI_TEXT_EXTRUDER_COLD_FI "E%Ne is too cold!"
#define UI_TEXT_HEAT_EXTRUDER_FI "Heat extruder ?"
#define UI_TEXT_WRITE_DEBUG_FI "Write Debug"
#define UI_TEXT_POWER_SAVE_MENU_FI "Powersave"
#define UI_TEXT_MAINTENANCE_FI "Maintenance"
#define UI_TEXT_SETTINGS_FI "Settings"
#define UI_TEXT_LOAD_UNLOAD_FI "Load/Unload"
#define UI_TEXT_BED_FI "Bed"
#define UI_TEXT_BED_OFF_FI "SwitchOff Bed %B7"
#define UI_TEXT_LIGHT_OFF_AFTER_FI "Light AutoOff"
#define UI_TEXT_LIGHT_OFF_AFTER2A_FI "Dis. After:%il"
#define UI_TEXT_LIGHT_OFF_AFTER2B_FI "[min] 0=Off"
#define UI_TEXT_WARNING_POS_X_UNKNOWN_FI "Real X unknown !"
#define UI_TEXT_WARNING_POS_Y_UNKNOWN_FI "Real Y unknown !"
#define UI_TEXT_WARNING_POS_Z_UNKNOWN_FI "Real Z unknown !"
#define UI_TEXT_MOVING_FI "Moving..."
#define UI_TEXT_PREPARING_FI "Preparing..."
#define UI_TEXT_KEEP_LIGHT_ON_FI "Print w/LED:%lk"
#define UI_TEXT_LOADING_FILAMENT_FI "Loading \005"
#define UI_TEXT_UNLOADING_FILAMENT_FI "Unloading \005"
#define UI_TEXT_LOADUNLOAD_FILAMENT_FI "Load / Unload \005"
#define UI_TEXT_AUTOLEVEL_FI "Auto level bed"
#define UI_TEXT_Z_PROBE_FAILED_FI "Z-probe failed"
#define UI_TEXT_SAVE_FI "Save to EEPROM ?"
#define UI_TEXT_ZMIN_FI "zMin : %zm"
#define UI_TEXT_ZPROBING_FI "Probing..."
#define UI_TEXT_ZPOSITION_FI "Go position..."
#define UI_TEXT_DO_YOU_FI "Do you want to:"
#define UI_TEXT_CLEAN1_FI "Clean nozzle and"
#define UI_TEXT_CLEAN2_FI "plates first ?"
#define UI_TEXT_DISPLAY_MODE_FI "Mode:%Md"
#define UI_TEXT_EASY_MODE_FI "      Easy"
#define UI_TEXT_ADVANCED_MODE_FI "  Advanced"
#define UI_TEXT_POWER_SAVE_FI "Powersave:%Ps"
#define UI_TEXT_BADGE_LIGHT_ONOFF_FI "Badge light:%lb"
#define UI_TEXT_SOUND_ONOF_FI "Sound:      %so"
#define UI_TEXT_YES_SELECTED_FI "  [Yes]    No"
#define UI_TEXT_NO_SELECTED_FI "   Yes    [No] "
#define UI_TEXT_PLEASE_CONFIRM_FI "Please confirm"
#define UI_TEXT_CANCEL_ACTION_FI "Cancel:"
#define UI_TEXT_CONTINUE_ACTION_FI "Continue"
#define UI_TEXT_CANCELED_FI "Action canceled!"
#define UI_TEXT_CONFIRMED_FI "Action confirmed!"
#define UI_TEXT_CLEAN_NOZZLE_FI "Clean Nozzle"
#define UI_TEXT_CLEANING_NOZZLE_FI "Cleaning Nozzle"
#define UI_TEXT_CLEAN_DRIPBOX_1_FI "Please Take"
#define UI_TEXT_CLEAN_DRIPBOX_2_FI "And Clean"
#define UI_TEXT_PUSH_FILAMENT_FI "Push Filament"
#define UI_TEXT_WAIT_FILAMENT_FI "Push Ok to start"
#define UI_TEXT_LOAD_FAILSAFE2_FI "after loading ?"
#define UI_TEXT_LOAD_FAILSAFE_FI "Load Fail-Safe"
#define UI_TEXT_HEATING_FI "Heating..."
#define UI_TEXT_WAIT_FOR_OK_FI "Press Ok to end"
#if NUM_EXTRUDER == 1
#define UI_TEXT_MAIN_LINE1_FI  "\005%ec/%Ec\002"
#define UI_TEXT_MAIN_LINE2_FI  "\007%eB/%Eb\002"
#else
#define UI_TEXT_MAIN_LINE1_FI "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_MAIN_LINE2_FI "\007%eB/%Eb"
#endif
#define UI_TEXT_AUTOLEVEL_MATRIX_FI "Autolevel Matrix"
#if HAVE_HEATED_BED
///Davinci Specific, page 1-2-3, they are different if you have 1 or 2 extruders, and if you are in Easy Mode or Advanced Mode
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_FI "\005%ec/%Ec\007%eB/%Eb"
#define UI_TEXT_P1_L2_FI "Z:%x2"
#define UI_TEXT_P1_L3_FI "Mul:%om%%%Buf:%oB"

#define UI_TEXT_P1_L1_EASY_FI "\005%ec/%Ec\002C"
#define UI_TEXT_P1_L2_EASY_FI "\007%eB/%Eb\002C"
#define UI_TEXT_P1_L3_EASY_FI "Z:%x2"

#else
#define UI_TEXT_P1_L1_FI "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_FI "\007%eB/%EbMul:%om%%%"
#define UI_TEXT_P1_L3_FI "Z:%x2"

#define UI_TEXT_P1_L1_EASY_FI "\0051 %e0/%E0\002C"
#define UI_TEXT_P1_L2_EASY_FI "\0052 %e1/%E1\002C"
#define UI_TEXT_P1_L3_EASY_FI "\007 %eB/%Eb\002C"
#endif
#else
#if NUM_EXTRUDER == 1
#define UI_TEXT_P1_L1_FI "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_FI "Z:%x2"
#define UI_TEXT_P1_L3_FI "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_FI "\005%ec/%Ec\002"
#define UI_TEXT_P1_L2_EASY_FI "Z:%x2"
#define UI_TEXT_P1_L3_EASY_FI "Buffer:%oB"
#else
#define UI_TEXT_P1_L1_FI "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_FI "Z:%x2"
#define UI_TEXT_P1_L3_FI "Buffer:%oB"

#define UI_TEXT_P1_L1_EASY_FI "\005%e0/%E0\005%e1/%E1"
#define UI_TEXT_P1_L2_EASY_FI "Z:%x2"
#define UI_TEXT_P1_L3_EASY_FI "Buffer:%oB"
#endif
#endif
#define UI_TEXT_MENU_SCANNER_FI    "Scanner"
#define UI_TEXT_PREPARE_SCANNER_FI    "Prepare scan"
#define UI_TEXT_LASER1_ONOFF_FI        "Right laser:%PA"
#define UI_TEXT_LASER2_ONOFF_FI       "Left laser :%PB"
#define UI_TEXT_LED1_ONOFF_FI           "Right LED  :%P1"
#define UI_TEXT_LED2_ONOFF_FI          "Left LED   :%P2"
#define UI_TEXT_CHANGE_TABLE_SPEED_FI    "Speed :%PS"
#define UI_TEXT_ROTATION_TABLE_FI     "Rotate +/-"
#define UI_TEXT_RELEASE_TABLE_FI      "Release table"
#define UI_TEXT_Z_MIN_CALCULATION_FI      "Zmin evaluation"
